@@ -12,7 +12,7 @@
 #endif
 
 #if defined(_MSC_VER)
-#define HAS_UINT64 0
+#define HAS_UINT64 _MSC_VER >= 1300
 #else
 #define HAS_UINT64 1
 #endif
@@ -50,7 +50,7 @@ typedef Int32                    Long;
 typedef __int64                  Int64;
 typedef unsigned __int64         UInt64;
 #else
-typedef long long Int64;
+typedef long long                Int64;
 typedef unsigned long long       UInt64;
 #endif
 #endif
@@ -113,7 +113,8 @@ enum PyArray_TYPES {
 	PyArray_CFLOAT=tComplex32, 
 	PyArray_CDOUBLE=tComplex64,
 	PyArray_NOTYPE=tAny,
-	PyArray_OBJECT=tObject
+	PyArray_OBJECT=tObject,
+	PyArray_NTYPES = tMaxType,
 };
 
 typedef enum 
@@ -133,6 +134,7 @@ typedef enum
 #define NOTSWAPPED      0x200       /* byteorder == sys.byteorder    */
 #define WRITABLE        0x400       /* data buffer is writable       */
 #define CHECKOVERFLOW   0x800       /* Check overflows on value assignments */
+#define UPDATEDICT     0x1000       /* New views update dict from original */
 
 #define IS_CARRAY (CONTIGUOUS | ALIGNED | NOTSWAPPED)
 
@@ -176,8 +178,8 @@ struct s_PyArrayObject
 
 	char      byteorder;   /* NUM_BIG_ENDIAN, NUM_LITTLE_ENDIAN */
 
-	char      _aligned;    /* test override flag */      
-	char      _contiguous; /* test override flag */
+	char      _unused0;    /* XXX unused */
+	char      _unused1;    /* XXX unused */
 
 	Complex64      temp;   /* temporary for gettitem/setitem MACROS */
 	char *         wptr;   /* working pointer for getitem/setitem MACROS */
