@@ -5,6 +5,7 @@ typedef struct {
 	PyObject_HEAD
 	PyObject *(*compute)(PyObject*,PyObject*,PyObject*);   /* Must match PyOperatorObject */
 	PyObject *(*rebuffer)(PyObject*,PyObject*,PyObject*);
+	void (*clean)(PyObject*,PyObject*);
 	int       arr_position;
 	int       inb_position;
 	int       direction;
@@ -356,4 +357,11 @@ typedef enum
 #define END_THREADS
 #endif
 
+#include "numconfig.h"
+
+#define ADD_VERSION(m) \
+	if (PyModule_AddObject(m, "__version__", \
+            PyString_FromString(NUMARRAY_VERSION)) < 0) \
+		return;
+
 #endif /* _ndarraymacro */
