@@ -4,7 +4,7 @@ This is the base multidimensional array class which implements all
 structural array operations but treats the array contents as opaque
 objects
 
-$Id: generic.py,v 1.54 2004/08/12 21:27:21 jaytmiller Exp $
+$Id: generic.py,v 1.74 2005/04/06 19:24:31 jaytmiller Exp $
 """
 
 import types as _types
@@ -26,6 +26,8 @@ import _ufunc
 
 _PROTOTYPE = 0  # Set to 1 to activate Python prototypes of C code.
 
+_UPDATEDICT = 0x1000  # views update from dict of original
+
 def _buffer_reduce(b):
     """Converts a "buffer" object to a pickling reduction tuple."""
     return (memory.memory_from_string, (str(b),))
@@ -95,21 +97,58 @@ def _broadcast(arr, sshape):
     tarr._strides = tuple(astrides)
     return tarr
 
-def _forwardBroadcast( indexArrays ):
-    for i in range(len(indexArrays)-1):
-        indexArrays[i], indexArrays[i+1] = \
-                    indexArrays[i]._dualbroadcast(indexArrays[i+1])
-    return indexArrays
+def _common_shape(sshape, ashape):
+    """Return broadcast shape common to both sshape and ashape."""
+    # Just return both if they have the same shape
+    if sshape == ashape:
+        return sshape
+    srank, arank = len(sshape), len(ashape)
+    # do a special comparison of all dims with size>1
+    if srank > arank:
+        newrank = srank
+        ashape = sshape[:newrank-arank] + ashape
+    else:
+        newrank = arank
+        sshape = ashape[:newrank-srank] + sshape
+    newshape = list(sshape)
+    for i in range(newrank):
+        if sshape[i] != ashape[i]:
+            if sshape[i] == 1:
+                newshape[i] = ashape[i]
+            elif ashape[i] == 1:
+                newshape[i] = sshape[i]
+            else:
+                raise ValueError("Arrays have incompatible shapes");
+    return tuple(newshape)
+
+def _common_shapes(indexArrays):
+    """determines the mutual broadcast shape of indexArrays."""
+    cshape = ()
+    for i in indexArrays:
+        if isinstance(i, NDArray) and i._strides is not None:
+            cshape = _common_shape(cshape, i._shape)
+    return cshape
+
+def _broadcast_all(indexArrays, cshape):
+    """returns a list of views of 'indexArrays' broadcast to shape 'cshape'"""
+    result = []
+    for i in indexArrays:
+        if isinstance(i, NDArray) and i._strides is not None:
+            result.append(_broadcast(i, cshape))
+        else:
+            result.append(i)
+    return tuple(result)
 
 def _nWayBroadcast( indexArrays ):
-    # This is clear to me, if not super efficient.
-    if len(indexArrays) > 1:
-        indexArrays  = _forwardBroadcast( indexArrays )
-        indexArrays.reverse()
-        indexArrays  = _forwardBroadcast( indexArrays )
-        indexArrays.reverse()
+    """return views of indexArrays broadcast to their common shape,
+    modifying the indexArrays sequence but not the original arrays.
+    """
+    # original function was impure... so keep side effects.
+    cshape = _common_shapes(indexArrays)
+    indexArrays[:] = list(_broadcast_all(indexArrays, cshape))
     return indexArrays
 
+
 def _takeShape(scattered, indexArrays):
     """computes the shape of the result of a take/put operation"""
     
@@ -451,6 +490,22 @@ class NDArray(_ndarray):
 
     size = _ndarray.nelements  # more portable synonym
 
+    def __get_array_shape__(self):
+        return self._shape
+
+    __array_shape__ = property(__get_array_shape__, None, "")
+
+    def __get_array_data__(self):
+        return self._data
+
+    __array_data__ = property(__get_array_data__, None, "")
+
+    def __get_array_strides__(self):
+        return self._strides
+
+    __array_strides__ = property(__get_array_strides__, None, "")
+    
+
     def __getstate__(self):
         """returns state of NDArray for pickling."""
         state = copy.copy(self.__dict__)
@@ -461,6 +516,7 @@ class NDArray(_ndarray):
         state["_strides"]    = self._strides
         state["_data"]       = self._data
         state["_itemsize"]   = self._itemsize
+        state["_flags"]      = self._flags
         return state
     
     def __setstate__(self, state):
@@ -472,9 +528,13 @@ class NDArray(_ndarray):
         self._strides    = state["_strides"]
         self._data       = state["_data"]
         self._itemsize   = state["_itemsize"]
+        try:
+            self._flags      = state["_flags"]
+        except:
+            self._flags |= _UPDATEDICT  
 
     def __nonzero__(self):
-        raise RuntimeError("An array doesn't make sense as a truth value.  Use sometrue(a) or alltrue(a).")
+        raise RuntimeError("An array doesn't make sense as a truth value.  Use any(a) or all(a).")
     
     def __copy__(self):
         """support for copy.copy()"""
@@ -554,6 +614,7 @@ class NDArray(_ndarray):
 
     def _dualbroadcast(self, arr):
         """Return broadcast views both self and arr, else return (None,None)."""
+        
         sshape = self._shape
         ashape = arr._shape
         # Just return both if they have the same shape
@@ -673,13 +734,6 @@ class NDArray(_ndarray):
             copyfunction(arr._shape, self._data, self._byteoffset,
                          self._strides, arr._data, 0, arr._strides,
                          arr._itemsize)
-
-        # delete self-testing artifacts; copying eliminates bad properties.
-        if hasattr(arr, "_aligned"):
-            del arr._aligned
-        if hasattr(arr, "_contiguous"):
-            del arr._contiguous
-            
         return arr
 
     def tostring(self):
@@ -708,14 +762,15 @@ class NDArray(_ndarray):
         
         """
         name =  0
-        if type(file) == type(""):
+        if isinstance(file,(unicode,str)):
             name = 1
             file = open(file, 'wb')
         niter = _IOBLOCKSIZE // self._itemsize
         if niter > 0:
-            indexlevel, blockingparameters = \
-                        _ufunc._getBlockingParameters(self._shape, niter)
-            self._tofileByBlocks(file, [], indexlevel, blockingparameters)
+            if self.nelements() != 0: # skip zero length arrays.
+                indexlevel, blockingparameters = \
+                            _ufunc._getBlockingParameters(self._shape, niter)
+                self._tofileByBlocks(file, [], indexlevel, blockingparameters)
             if name:
                 file.close()
         else: # very large items 
@@ -776,14 +831,6 @@ class NDArray(_ndarray):
     def _clone(self, shape):
         """returns an empty array identical to 'self' but with 'shape'.
         """
-        c = self.view()
-        c._shape = shape
-        c._strides = _stridesFromShape(shape)
-        c._byteoffset = 0
-        c._data = memory.new_memory(product(shape)*self._itemsize)
-        return c
-
-    def _clone(self, shape):
         c = self.copy()
         c.resize(shape)
         return c
@@ -809,10 +856,6 @@ class NDArray(_ndarray):
         if not isinstance(values, self.__class__):
             values = self.factory( values )
         values = _broadcast( values, impliedShape )
-        if values.rank == 0:
-            values = values.view()
-            values._shape = (1,)
-            values._strides = (values._itemsize,)
         indices = self._fix_pt_indices(indices)
         self._putter(indices, values)
 
@@ -888,17 +931,24 @@ class NDArray(_ndarray):
     def astype(self, type=None): #default so numarray.array works for NDArrays
         return self.copy()
 
+    def sinfo(self):
+        """sinfo() returns a string describing key attributes of an array."""
+        s = ""
+        s += "class: " + repr(self.__class__) + "\n"
+        s += "shape: " + repr(self._shape) + "\n"
+        s += "strides: " + repr(self._strides) + "\n"
+        s += "byteoffset: " + repr(self._byteoffset) + "\n"
+        s += "bytestride: " + repr(self._bytestride) + "\n"
+        s += "itemsize: " + repr(self._itemsize) + "\n"
+        s += "aligned: " + repr(self.isaligned()) + "\n"
+        s += "contiguous: " + repr(self.iscontiguous()) + "\n"
+        s += "buffer: " + repr(self._data) + "\n"
+        s += "data pointer: 0x%08x (DEBUG ONLY)\n" % (self._fragile_data,)
+        return s
+
     def info(self):
         """info() prints out the key attributes of an array."""
-        print "class:", self.__class__
-        print "shape:", self._shape
-        print "strides:", self._strides
-        print "byteoffset:", self._byteoffset
-        print "bytestride:", self._bytestride
-        print "itemsize:", self._itemsize
-        print "aligned:", self.isaligned()
-        print "contiguous:", self.iscontiguous()
-        print "data:", repr(self._data)
+        sys.stdout.write(self.sinfo())
 
 def reshape(arr, shape, *args):
     """Returns a reshaped *view* of array if possible, else a *copy*. 
@@ -932,18 +982,48 @@ def transpose(array, axes=None):
     v.transpose(axes)
     return v
 
-def sort(array, axis=-1):
+def sort(array, axis=-1, kind=''):
     """Returns a sorted *copy* of array."""
     array = _nc.array(array)
-    array.sort(axis)
+    array.sort(axis,kind)
     return array
 
-def argsort(array, axis=-1):
+def argsort(array, axis=-1,kind=''):
     """Returns an array of indices which, if "taken" from 'array',
     would sort 'array'.
     """
     array = _nc.asarray(array)
-    return array.argsort(axis)
+    return array.argsort(axis,kind)
+
+def lexsort(keys, axis=-1):
+    """Returns index array that lexigraphically sorts on keys.
+
+    This is like taking the first letter from the first key,
+    the second from the second key, etc., and forming a 'word.'
+    The words are then indirectly sorted on the first key, then
+    the second, etc. This will order things into 'alphabetic order'
+    for the word *spelled backwards*. To get alphabetic order, the
+    sort must start on the last letter and proceed to the first.
+    """
+    if not isinstance(keys,(_types.ListType, _types.TupleType)) : keys = (keys,)
+    if axis == -1 :
+        keys = [_nc.asarray(x) for x in keys]
+        shps = [x.getshape() for x in keys]
+        ashp = shps[0]
+        if reduce(operator.__or__, [x != ashp for x in shps]) :
+            raise KeyError, 'key arrays must have the same shape'
+        w = _nc.array(shape=ashp, type=_nt.Long)
+        w[...,:] = _nc.arange(ashp[-1], type=_nt.Long)
+        for x in keys :
+            ufunc._broadcast_indirect_sort(x, w, 'amergesort')
+        return w
+    else :
+        map(lambda x : x.swapaxes(axis,-1), keys)
+        w = lexmergesort(keys)
+        map(lambda x : x.swapaxes(axis,-1), keys)
+        w.swapaxes(axis,-1)
+        return w
+
 
 def argmin(array, axis=-1):
     """Returns the indices of the minumum elements of 'array' taken
@@ -1110,8 +1190,11 @@ def putmask(array, mask, values):
     bmask = _nc.asarray(mask)
     bvalues = _nc.asarray(values)
     if bmask.nelements() == array.nelements():
-        bmask.setshape(array.getshape())
+        if bmask._shape != array._shape:
+            bmask = bmask.view()
+            bmask.setshape(array.getshape())
     else:
         bmask = _broadcast_or_resize(array, bmask)
     bvalues = _broadcast_or_resize(array, bvalues)
     choose(bmask != 0, (array, bvalues), array)
+
