@@ -1,4 +1,4 @@
-"""$Id: memmap.py,v 1.26 2004/08/25 10:13:25 jaytmiller Exp $
+"""$Id: memmap.py,v 1.28 2005/01/14 15:57:17 jaytmiller Exp $
 
 This module implements a class wrapper (Memmap) around the
 mmap type which adds some additional properties.
@@ -190,7 +190,7 @@ cannot be resized, flushed, or synced to the original file.
 >>> n = m[:]
 >>> n
 <MemmapSlice of length:100 writable>
->>> a = num.array(sequence=n, shape=(len(n),), type=num.Int8)
+>>> a = num.NumArray(buffer=n, shape=(len(n),), type=num.Int8)
 >>> a += 1
 >>> # it worked!
 >>> m.close()
@@ -199,7 +199,7 @@ Try a zero length memmap based on comments from Sebastian Hesse
 
 >>> m = Memmap("memmap.tst", mode='w+', len=0)
 >>> n = m.insert(0,0)
->>> a = num.array(sequence=n, type=num.UInt16, shape=(0,0,0))
+>>> a = num.NumArray(buffer=n, type=num.UInt16, shape=(0,0,0))
 >>> _junk = a.resize((100,100,100))
 >>> m.flush()
 >>> m.close()
@@ -212,11 +212,11 @@ of the restrictions of copy-on-write mappings.
 >>> n = m[:]
 >>> n
 <MemmapSlice of length:100 readonly>
->>> a = num.array(sequence=n, shape=(len(n),), type=num.Int8)
+>>> a = num.NumArray(buffer=n, shape=(len(n),), type=num.Int8)
 >>> a += 1  # can't work...  buffer is readonly
 Traceback (most recent call last):
 ...
-error: add_Int8_vector_scalar: Problem with write buffer[2].
+error: add_11x1_vsxv: Problem with write buffer[2].
 >>> b = a + 1  # this still works...
 >>>
 
