@@ -27,8 +27,9 @@ import mlab
 # import multiarray     # just used for fast 2D tranpose
 multiarray = num 
 
-# Error object
-LinAlgError = 'LinearAlgebraError'
+class LinearAlgebraError(Exception):
+    pass
+LinAlgError = LinearAlgebraError
 
 def __setup__():
     import sys
