@@ -57,6 +57,9 @@ class _MaskedPrintOption:
     def __str__ (self):
         return str(self._display)
 
+    def __repr__(self):
+        return str(self)
+
 #if you single index into a masked location you get this object.
 masked_print_option = _MaskedPrintOption('--')
 
@@ -181,7 +184,7 @@ def make_mask (m, copy=0, flag=0):
     else:
         result = Numeric.array(filled(m,1), MaskType, savespace=1)
 
-    if flag and not Numeric.sometrue(Numeric.ravel(result)):
+    if flag and not Numeric.any(result):
         return None
     else:
         return result
@@ -217,7 +220,10 @@ def filled (a, value = None):
     if isinstance(a, MaskedArray):
         return a.filled(value)
     elif isinstance(a, Numeric.ArrayType) and a.iscontiguous():
-        return a
+        if a.rank is 0:
+            return a[()]
+        else:
+            return a
     elif isinstance(a, types.DictType):
         return Numeric.array(a, 'O')
     else:
@@ -307,16 +313,21 @@ class masked_unary_operation:
         if m is None:
             result = self.f(d1, *args, **kwargs)
             if type(result) is Numeric.ArrayType:
-                return masked_array (result)
+                if result.rank > 0:
+                    return masked_array (result)
+                else:
+                    return result[()]                                 
             else:
                 return result
         else:
             dx = masked_array(d1, m)
             result = self.f(filled(dx, self.fill), *args, **kwargs)
-            if type(result) is Numeric.ArrayType:
+            if type(result) is Numeric.ArrayType and result.rank > 0:
                 return masked_array(result, m)
-            elif Numeric.sometrue(Numeric.ravel(m)):
+            elif Numeric.any(m):
                 return masked
+            elif type(result) is Numeric.ArrayType and result.rank == 0:
+                return result[()]
             else:
                 return result
 
@@ -352,23 +363,28 @@ class domained_binary_operation:
         d2 = filled(b, self.filly)
         t = self.domain(d1, d2)
         
-        if Numeric.sometrue(Numeric.ravel(t)):
+        if Numeric.any(t):
             d2 = where(t, self.filly, d2)
             mb = mask_or(mb, t)
         m = mask_or(ma, mb)
         if m is None:
             result =  self.f(d1, d2)
             if type(result) is Numeric.ArrayType:
-                return masked_array(result)
+                if result.rank > 0:
+                    return masked_array(result)
+                else:
+                    return result[()]
             else:
                 return result
         result = self.f(d1, d2)
-        if type(result) is Numeric.ArrayType:
+        if type(result) is Numeric.ArrayType and result.rank > 0:
             if m.shape != result.shape:
                 m = mask_or(getmaskarray(a), getmaskarray(b))
             return masked_array(result, m)
-        elif Numeric.sometrue(Numeric.ravel(m)):
+        elif Numeric.any(m):
             return masked
+        elif type(result) is Numeric.ArrayType and result.rank == 0:
+            return result[()]
         else:
             return result
     def __str__ (self):
@@ -392,18 +408,23 @@ class masked_binary_operation:
             d2 = filled(b, self.filly)
             result =  self.f(d1, d2, *args, **kwargs)
             if type(result) is Numeric.ArrayType:
-                return masked_array(result)
+                if result.rank > 0:
+                    return masked_array(result)
+                else:
+                    return result[()]
             else:
                 return result
         d1 = filled(a, self.fillx)
         d2 = filled(b, self.filly)
         result = self.f(d1, d2, *args, **kwargs)
-        if type(result) is Numeric.ArrayType:
+        if type(result) is Numeric.ArrayType and result.rank > 0:
             if m.shape != result.shape:
                 m = mask_or(getmaskarray(a), getmaskarray(b))
             return masked_array(result, m)
-        elif Numeric.sometrue(Numeric.ravel(m)):
+        elif Numeric.any(m):
             return masked
+        elif type(result) is Numeric.ArrayType and result.rank == 0:
+            return result[()]
         else:
             return result
 
@@ -522,6 +543,9 @@ rank = Numeric.rank
 shape = Numeric.shape
 size = Numeric.size
 
+def _isRank0(a):
+    return isinstance(a, Numeric.ArrayType) and a.rank is 0
+
 class MaskedArray(numarray.UsesOpPriority):
     """Arrays with possibly masked values. 
        Masked values of 1 exclude element from the computation.
@@ -616,9 +640,9 @@ class MaskedArray(numarray.UsesOpPriority):
             self._shared_mask = 0
         else:
             if not isinstance(data, MaskedArray):
-                if mask is 1:
+                if mask is 1 or (_isRank0(mask) and mask[()] is 1):
                     mask = Numeric.ones(self._data.shape, MaskType)
-                elif mask is 0:
+                elif mask is 0 or (_isRank0(mask) and mask[()] is 0):
                     mask = Numeric.zeros(self._data.shape, MaskType)
             # else:
             #     mask = data.mask()
@@ -647,7 +671,7 @@ class MaskedArray(numarray.UsesOpPriority):
     def __array__ (self, t = None):
         "Special hook for Numeric. Converts to Numeric if possible."
         if self._mask is not None:
-            if Numeric.sometrue(Numeric.ravel(self._mask)):
+            if Numeric.any(self._mask):
                 raise MAError, \
                 """Cannot automatically convert masked array to Numeric because data 
                    is masked in one or more locations.
@@ -1446,7 +1470,7 @@ def allclose (a, b, fill_value=1, rtol=1.e-5, atol=1.e-8):
     x = filled(array(d1, copy=0, mask=m), fill_value).astype(Float)
     y = filled(array(d2, copy=0, mask=m), 1).astype(Float)
     d = Numeric.less_equal(Numeric.absolute(x-y), atol + rtol * Numeric.absolute(y))
-    return Numeric.alltrue(Numeric.ravel(d))    
+    return Numeric.all(d)    
 
 def allequal (a, b, fill_value=1):
     """
@@ -1458,13 +1482,13 @@ def allequal (a, b, fill_value=1):
         x = filled(a)
         y = filled(b)
         d = Numeric.equal(x, y)
-        return Numeric.alltrue(Numeric.ravel(d))
+        return Numeric.all(d)
     elif fill_value:
         x = filled(a)
         y = filled(b)
         d = Numeric.equal(x, y)
         dm = array(d, mask=m, copy=0)
-        return Numeric.alltrue(Numeric.ravel(filled(dm, 1)))
+        return Numeric.all(filled(dm, 1))
     else:
         return 0
     
