@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Peter J. Verveer
+/* Copyright (C) 2003-2005 Peter J. Verveer
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -38,12 +38,11 @@
 #define BUFFER_SIZE 256000
 
 int NI_Correlate1D(PyArrayObject *input, double *weights, int filter_size,
-                   int axis, PyArrayObject **output, 
-                   PyObject *output_in, NI_ExtendMode mode, double cval, 
-                   int shift, NumarrayType output_type)
+                   int axis, PyArrayObject *output, NI_ExtendMode mode,
+                   double cval, int shift)
 {
   int symmetric = 0, ii, jj, ll, lines, length, size1, size2, more, irank;
-  int itype, idims[NI_MAXDIM];
+  int itype, otype, idims[NI_MAXDIM];
   double *ibuffer = NULL, *obuffer = NULL, *fw;
   NI_LineBuffer iline_buffer, oline_buffer;
 
@@ -62,24 +61,27 @@ int NI_Correlate1D(PyArrayObject *input, double *weights, int filter_size,
   if (axis < 0)
     axis += irank;
 
-  itype = NI_GetArrayType(input);
   /* check axis specification: */
   if (irank > 0 && (axis < 0 || axis >= irank)) {
     PyErr_SetString(PyExc_RuntimeError, "invalid axis specified");
     goto exit;
   }
 
+  /* check the output array: */
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, idims))
+    goto exit;
+
+  itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
+
   /* complex types not supported: */
   if (itype == tComplex32 || itype == tComplex64 ||
-      output_type == tComplex32 || output_type == tComplex64) {
+      otype == tComplex32 || otype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
 
-  /* if no output type was specified, use the input type: */
-  if (output_type == tAny)
-    output_type = (NumarrayType)itype;
-
   /* check the filter shift: */
   size1 = filter_size / 2;
   size2 = filter_size - size1 - 1;
@@ -111,19 +113,13 @@ int NI_Correlate1D(PyArrayObject *input, double *weights, int filter_size,
     }
   }
 
-  /* allocate the output array: */
-  NI_GetArrayDimensions(input, idims);
-  if (!NI_OutputArray(output_type, irank, idims, output_in, output))
-    goto exit;
-
-
   /* allocate and initialize the line buffers: */
   lines = -1;
   if (!NI_AllocateLineBuffer(input, axis, size1 + shift, size2 - shift,
                              &lines, BUFFER_SIZE, &ibuffer))
     goto exit;
 
-  if (!NI_AllocateLineBuffer(*output, axis, 0, 0, &lines, BUFFER_SIZE, 
+  if (!NI_AllocateLineBuffer(output, axis, 0, 0, &lines, BUFFER_SIZE, 
                              &obuffer))
     goto exit;
 
@@ -131,7 +127,7 @@ int NI_Correlate1D(PyArrayObject *input, double *weights, int filter_size,
                          ibuffer, mode, cval, &iline_buffer))
     goto exit;
 
-  if (!NI_InitLineBuffer(*output, axis, 0, 0, lines, obuffer, mode, 0.0, 
+  if (!NI_InitLineBuffer(output, axis, 0, 0, lines, obuffer, mode, 0.0, 
                          &oline_buffer))
     goto exit;
 
@@ -199,30 +195,34 @@ int NI_Correlate1D(PyArrayObject *input, double *weights, int filter_size,
 
 
 int NI_Correlate(PyArrayObject* input, PyArrayObject* weights, 
-                 PyArrayObject* footprint, PyArrayObject** output, 
-                 PyObject* output_in, NI_ExtendMode mode, double cvalue, 
-                 int *shifts, NumarrayType output_type)
+                 PyArrayObject* footprint, PyArrayObject* output, 
+                 NI_ExtendMode mode, double cvalue, int *shifts)
 {
   int filter_size = 0, *offsets = NULL, size, *oo, jj, wdims[NI_MAXDIM];
-  int border_flag_value, msize, wsize, kk, irank, itype, idims[NI_MAXDIM];
+  int border_flag_value, msize, wsize, kk, irank, idims[NI_MAXDIM];
+  int itype, otype, fdims[NI_MAXDIM];
   Bool *pm;
   NI_Iterator ii, io;
   NI_FilterIterator fi;
   char *pi, *po;
   Float64 *pw;
   Float64 *ww = NULL;
+  Bool *pf;
 
   /* complex types not supported: */
   itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
   if (itype == tComplex32 || itype == tComplex64 ||
-      output_type == tComplex32 || output_type == tComplex64) {
+      otype == tComplex32 || otype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
 
-  /* if no output type was specified, use the input type: */
-  if (output_type == tAny)
-    output_type = (NumarrayType)itype;
+  /* check the output array: */
+  irank = NI_GetArrayRank(input);
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, idims))
+    goto exit;
 
   if (!NI_ShapeEqual(weights, footprint)) {
     PyErr_SetString(PyExc_RuntimeError, 
@@ -242,8 +242,6 @@ int NI_Correlate(PyArrayObject* input, PyArrayObject* weights,
     goto exit;
   }
 
-  irank = NI_GetArrayRank(input);
-
   /* input and footprint must have the same rank: */
   if (irank != NI_GetArrayRank(footprint)) {
     PyErr_SetString(PyExc_RuntimeError, 
@@ -264,18 +262,15 @@ int NI_Correlate(PyArrayObject* input, PyArrayObject* weights,
                     "footprint array must be contiguous");
     goto exit;
   }
-
+  NI_GetArrayDimensions(footprint, fdims);  
+  pf = (Bool*)NI_GetArrayData(footprint);
+  
   /* get the size of the footprint: */
   pm = (Bool*)NI_GetArrayData(footprint);
-  msize = NI_Elements(footprint);
+  msize = NI_GetArraySize(footprint);
   for(jj = 0; jj < msize; jj++) 
     if (pm[jj]) ++filter_size;
 
-  /* allocate the output array: */
-  NI_GetArrayDimensions(input, idims);
-  if (!NI_OutputArray(output_type, irank, idims, output_in, output))
-    goto exit;
-
   /* filter must be not be emtpy: */
   if (filter_size < 1) {
     PyErr_SetString(PyExc_RuntimeError, "filter size must be > 0");
@@ -283,8 +278,8 @@ int NI_Correlate(PyArrayObject* input, PyArrayObject* weights,
   }
 
   /* calculate the filter offsets: */
-  if (!NI_InitFilterOffsetsFromArray(input, footprint, shifts, mode,
-                                     &offsets, &border_flag_value, NULL))
+  if (!NI_InitFilterOffsets(input, pf, fdims, shifts, mode,
+                            &offsets, &border_flag_value, NULL))
     goto exit;
 
   /* initialize input element iterator: */
@@ -292,7 +287,7 @@ int NI_Correlate(PyArrayObject* input, PyArrayObject* weights,
     goto exit;
 
   /* initialize output element iterator: */
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
   /* initialize filter iterator: */
@@ -303,10 +298,10 @@ int NI_Correlate(PyArrayObject* input, PyArrayObject* weights,
     
   /* get data pointers an sizes: */
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
+  po = NI_GetArrayData(output);
   pw = (Float64*)NI_GetArrayData(weights);
-  size = NI_Elements(input);
-  wsize = NI_Elements(weights);
+  size = NI_GetArraySize(input);
+  wsize = NI_GetArraySize(weights);
 
   ww = (double*)malloc(filter_size * sizeof(double));
   if (!ww) {
@@ -375,7 +370,7 @@ int NI_Correlate(PyArrayObject* input, PyArrayObject* weights,
       PyErr_SetString(PyExc_RuntimeError, "data type not supported");
       goto exit;
     }
-    switch (output_type) {
+    switch (otype) {
     case tBool:
       *(Bool*)po = (Bool)tmp;
       break;
@@ -425,12 +420,10 @@ int NI_Correlate(PyArrayObject* input, PyArrayObject* weights,
 }
 
 
-int NI_BoxcarFilter1D(PyArrayObject *input, int filter_size, int axis, 
-                      PyArrayObject** output, PyObject* output_in, 
-                      NI_ExtendMode mode, double cvalue, int shift, 
-                      NumarrayType output_type)
+int NI_UniformFilter1D(PyArrayObject *input, int filter_size, int axis, 
+       PyArrayObject* output, NI_ExtendMode mode, double cvalue, int shift)
 {
-  int ii, ll, lines, size1, size2, length, more, irank, itype;
+  int ii, ll, lines, size1, size2, length, more, irank, itype, otype;
   int idims[NI_MAXDIM];
   double tmp, *ibuffer = NULL, *obuffer = NULL, *l1, *l2;
   NI_LineBuffer iline_buffer, oline_buffer;
@@ -440,7 +433,11 @@ int NI_BoxcarFilter1D(PyArrayObject *input, int filter_size, int axis,
     goto exit;
   }
 
+  /* check the output array: */
   irank = NI_GetArrayRank(input);
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, idims))
+    goto exit;
 
   /* support negative axis specification: */
   if (axis < 0)
@@ -454,21 +451,13 @@ int NI_BoxcarFilter1D(PyArrayObject *input, int filter_size, int axis,
 
   /* complex types not supported: */
   itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
   if (itype == tComplex32 || itype == tComplex64 ||
-      output_type == tComplex32 || output_type == tComplex64) {
+      otype == tComplex32 || otype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
 
-  /* if no output type was specified, use the input type: */
-  if (output_type == tAny)
-    output_type = (NumarrayType)itype;
-
-  /* allocate the output array: */
-  NI_GetArrayDimensions(input, idims);
-  if (!NI_OutputArray(output_type, irank, idims, output_in, output))
-    goto exit;
-
   /* check the filter shift: */
   size1 = filter_size / 2;
   size2 = filter_size - size1 - 1;
@@ -484,14 +473,14 @@ int NI_BoxcarFilter1D(PyArrayObject *input, int filter_size, int axis,
                              &lines, BUFFER_SIZE, &ibuffer))
     goto exit;
   
-  if (!NI_AllocateLineBuffer(*output, axis, 0, 0, &lines, BUFFER_SIZE, 
+  if (!NI_AllocateLineBuffer(output, axis, 0, 0, &lines, BUFFER_SIZE, 
                              &obuffer))
     goto exit;
   
   if (!NI_InitLineBuffer(input, axis, size1 + shift, size2 - shift, 
                          lines, ibuffer, mode, cvalue, &iline_buffer))
     goto exit;
-  if (!NI_InitLineBuffer(*output, axis, 0, 0, lines, obuffer, mode, 
+  if (!NI_InitLineBuffer(output, axis, 0, 0, lines, obuffer, mode, 
                          cvalue, &oline_buffer))
     goto exit;
   
@@ -506,7 +495,7 @@ int NI_BoxcarFilter1D(PyArrayObject *input, int filter_size, int axis,
       /* get lines: */
       double *iline = NI_GET_LINE(iline_buffer, ii);
       double *oline = NI_GET_LINE(oline_buffer, ii);
-      /* do the boxcar filter: */
+      /* do the uniform filter: */
       tmp = 0.0;
       for(ll = 0; ll < filter_size; ll++)
         tmp += iline[ll];
@@ -533,11 +522,10 @@ int NI_BoxcarFilter1D(PyArrayObject *input, int filter_size, int axis,
 
 
 int NI_MinimumMaximumFilter1D(PyArrayObject* input, int filter_size, 
-                              int axis, PyArrayObject** output, 
-                              PyObject* output_in, NI_ExtendMode mode, 
-                              double cvalue, int shift, char minimum)
+                      int axis, PyArrayObject* output, NI_ExtendMode mode,
+                      double cvalue, int shift, char minimum)
 {
-  int size1, size2, length, lines, ll, ii, jj, more, irank, itype;
+  int size1, size2, length, lines, ll, ii, jj, more, irank, itype, otype;
   int idims[NI_MAXDIM];
   double *ibuffer = NULL, *obuffer = NULL;
   NI_LineBuffer iline_buffer, oline_buffer;
@@ -547,7 +535,11 @@ int NI_MinimumMaximumFilter1D(PyArrayObject* input, int filter_size,
     goto exit;
   }
 
+  /* check the output array: */
   irank = NI_GetArrayRank(input);
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, idims))
+    goto exit;
 
   /* support negative axis specification: */
   if (axis < 0)
@@ -561,17 +553,13 @@ int NI_MinimumMaximumFilter1D(PyArrayObject* input, int filter_size,
 
   /* complex types not supported: */
   itype = NI_GetArrayType(input);
-  if (itype == tComplex32 || itype == tComplex64) {
+  otype = NI_GetArrayType(output);
+  if (itype == tComplex32 || itype == tComplex64 ||
+      otype == tComplex32 || otype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
 
-  /* allocate the output array: */
-  NI_GetArrayDimensions(input, idims);
-  if (!NI_OutputArray((NumarrayType)itype, irank, idims, output_in,
-                      output))
-    goto exit;
-
   /* check the filter shift: */
   size1 = filter_size / 2;
   size2 = filter_size - size1 - 1;
@@ -586,13 +574,13 @@ int NI_MinimumMaximumFilter1D(PyArrayObject* input, int filter_size,
   if (!NI_AllocateLineBuffer(input, axis, size1 + shift, size2 - shift,
                              &lines, BUFFER_SIZE, &ibuffer))
     goto exit;
-  if (!NI_AllocateLineBuffer(*output, axis, 0, 0, &lines, BUFFER_SIZE,
+  if (!NI_AllocateLineBuffer(output, axis, 0, 0, &lines, BUFFER_SIZE,
                              &obuffer))
     goto exit;
   if (!NI_InitLineBuffer(input, axis, size1 + shift, size2 - shift, 
                          lines, ibuffer, mode, cvalue, &iline_buffer))
     goto exit;
-  if (!NI_InitLineBuffer(*output, axis, 0, 0, lines, obuffer, mode, 
+  if (!NI_InitLineBuffer(output, axis, 0, 0, lines, obuffer, mode, 
                          cvalue, &oline_buffer))
     goto exit;
   
@@ -634,12 +622,12 @@ int NI_MinimumMaximumFilter1D(PyArrayObject* input, int filter_size,
   return PyErr_Occurred() ? 0 : 1;
 }
 
-#define NI_MIN_MAX_FILTER_POINT(_pi, _po, _offsets, _filter_size,   \
+#define NI_MIN_MAX_FILTER_POINT(_pi, _val, _offsets, _filter_size,  \
                                 _cvalue, _type, _minimum, _mv, _ss) \
 {                                                                   \
   int _ii, _oo = *_offsets;                                         \
-  _type _cv = (_type)_cvalue;                                       \
-  _type _tmp, _val = _oo == _mv ? _cv : *(_type*)(_pi + _oo);       \
+  _type _cv = (_type)_cvalue, _tmp;                                 \
+  _val = _oo == _mv ? _cv : *(_type*)(_pi + _oo);                   \
   if (_ss)                                                          \
     _val += *_ss;                                                   \
   for(_ii = 1; _ii < _filter_size; _ii++) {                         \
@@ -655,35 +643,38 @@ int NI_MinimumMaximumFilter1D(PyArrayObject* input, int filter_size,
         _val = (_type)_tmp;                                         \
     }                                                               \
   }                                                                 \
-  *(_type*)_po = _val;                                              \
 }
 
 
-int NI_MinimumMaximumFilter(PyArrayObject* input, PyArrayObject* footprint, 
-                            PyArrayObject* structure, 
-                            PyArrayObject** output, PyObject* output_in,
-                            NI_ExtendMode mode, double cvalue, int *shifts, 
-                            char minimum)
+int NI_MinimumMaximumFilter(PyArrayObject* input, PyArrayObject* footprint,
+               PyArrayObject* structure, PyArrayObject* output,
+               NI_ExtendMode mode, double cvalue, int *shifts, char minimum)
 {
   int filter_size = 0, *offsets = NULL, size, *oo, jj, kk;     
   int border_flag_value;
-  int msize, irank, itype, idims[NI_MAXDIM], fdims[NI_MAXDIM];
+  int msize, irank, itype, otype, idims[NI_MAXDIM], fdims[NI_MAXDIM];
   Bool *pm;
   NI_Iterator ii, io;
   NI_FilterIterator fi;
   char *pi, *po;
   Float64 *ps = NULL;
-  double *ss = NULL;
+  double *ss = NULL, tmp;
+
+  /* check the output array: */
+  irank = NI_GetArrayRank(input);
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, idims))
+    goto exit;
 
   /* complex types not supported: */
   itype = NI_GetArrayType(input);
-  if (itype == tComplex32 || itype == tComplex64) {
+  otype = NI_GetArrayType(output);
+  if (itype == tComplex32 || itype == tComplex64 ||
+      itype == tComplex32 || itype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
 
-  irank = NI_GetArrayRank(input);
-
   /* input and footprint must have the same rank: */
   if (irank != NI_GetArrayRank(footprint)) {
     PyErr_SetString(PyExc_RuntimeError, 
@@ -706,7 +697,7 @@ int NI_MinimumMaximumFilter(PyArrayObject* input, PyArrayObject* footprint,
 
   /* get the size of the footprint: */
   pm = (Bool*)NI_GetArrayData(footprint);
-  msize = NI_Elements(footprint);
+  msize = NI_GetArraySize(footprint);
   for(jj = 0; jj < msize; jj++) 
     if (pm[jj]) ++filter_size;
 
@@ -745,12 +736,6 @@ int NI_MinimumMaximumFilter(PyArrayObject* input, PyArrayObject* footprint,
         ss[jj++] = minimum ? -ps[kk] : ps[kk];
   }
 
-  /* allocate the output array: */
-  NI_GetArrayDimensions(input, idims);
-  if (!NI_OutputArray((NumarrayType)itype, irank, idims, output_in,
-                      output))
-    goto exit;
-
   /* filter must be not be emtpy: */
   if (filter_size < 1) {
     PyErr_SetString(PyExc_RuntimeError, "filter size must be > 0");
@@ -758,8 +743,9 @@ int NI_MinimumMaximumFilter(PyArrayObject* input, PyArrayObject* footprint,
   }
 
   /* calculate the filter offsets: */
-  if (!NI_InitFilterOffsetsFromArray(input, footprint, shifts, mode,
-                                     &offsets, &border_flag_value, NULL))
+  NI_GetArrayDimensions(footprint, fdims);
+  if (!NI_InitFilterOffsets(input, pm, fdims, shifts, mode,
+                            &offsets, &border_flag_value, NULL))
     goto exit;
 
   /* initialize input element iterator: */
@@ -767,74 +753,113 @@ int NI_MinimumMaximumFilter(PyArrayObject* input, PyArrayObject* footprint,
     goto exit;
 
   /* initialize output element iterator: */
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
   /* initialize filter iterator: */
-  NI_GetArrayDimensions(footprint, fdims);
   if (!NI_InitFilterIterator(irank, fdims, filter_size, idims, shifts,
                              &fi))
     goto exit;
     
   /* get data pointers an size: */
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
-  size = NI_Elements(input);
+  po = NI_GetArrayData(output);
+  size = NI_GetArraySize(input);
 
   /* iterator over the elements: */
   oo = offsets;
   for(jj = 0; jj < size; jj++) {
     switch (itype) {
     case tBool:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, Bool,
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Bool,
                               minimum, border_flag_value, ss);
       break;
     case tUInt8:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, UInt8, 
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, UInt8, 
                               minimum, border_flag_value, ss);
       break;
     case tUInt16:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, UInt16, 
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, UInt16, 
                               minimum, border_flag_value, ss);
       break;
     case tUInt32:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, UInt32, 
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, UInt32, 
                               minimum, border_flag_value, ss);
       break;
 #if HAS_UINT64
     case tUInt64:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, UInt64, 
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, UInt64, 
                               minimum, border_flag_value, ss);
       break;
 #endif
     case tInt8:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, Int8, 
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Int8, 
                               minimum, border_flag_value, ss);
       break;
     case tInt16:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, Int16, 
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Int16, 
                               minimum, border_flag_value, ss);
       break;
     case tInt32:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, Int32,
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Int32,
                               minimum, border_flag_value, ss);
       break;
     case tInt64:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, Int64,
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Int64,
                               minimum, border_flag_value, ss);
       break;
     case tFloat32:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, Float32,
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Float32,
                               minimum, border_flag_value, ss);
       break;
     case tFloat64:
-      NI_MIN_MAX_FILTER_POINT(pi, po, oo, filter_size, cvalue, Float64, 
+      NI_MIN_MAX_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Float64, 
                               minimum, border_flag_value, ss);
       break;
     default:
       PyErr_SetString(PyExc_RuntimeError, "data type not supported");
       goto exit;
     }
+    switch (otype) {
+    case tBool:
+      *(Bool*)po = (Bool)tmp;
+      break;
+    case tUInt8:
+      *(UInt8*)po = (UInt8)tmp;
+      break;
+    case tUInt16:
+      *(UInt16*)po = (UInt16)tmp;
+      break;
+    case tUInt32:
+      *(UInt32*)po = (UInt32)tmp;
+      break;
+#if HAS_UINT64
+    case tUInt64:
+      *(UInt64*)po = (UInt64)tmp;
+      break;
+#endif
+    case tInt8:
+      *(Int8*)po = (Int8)tmp;
+      break;
+    case tInt16:
+      *(Int16*)po = (Int16)tmp;
+      break;
+    case tInt32:
+      *(Int32*)po = (Int32)tmp;
+      break;
+    case tInt64:
+      *(Int64*)po = (Int64)tmp;
+      break;
+    case tFloat32:
+      *(Float32*)po = (Float32)tmp;
+      break;
+    case tFloat64:
+      *(Float64*)po = (Float64)tmp;
+      break;
+    default:
+      PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+      goto exit;
+    }
     NI_FILTER_NEXT2(fi, ii, io, oo, pi, po);
   }
 
@@ -880,43 +905,48 @@ static double NI_Select(double *buffer, int min, int max, int rank)
     return NI_Select(buffer, jj + 1, max, rank - ii);                
 }
 
-#define NI_RANK_FILTER_POINT(_pi, _po, _offsets, _filter_size,           \
-                             _cvalue, _type, _buffer, _rank, _mv)        \
-{                                                                        \
-  int _ii;                                                               \
-  for(_ii = 0; _ii < _filter_size; _ii++) {                              \
-    int _offset = _offsets[_ii];                                         \
-    if (_offset == _mv)                                                  \
-      _buffer[_ii] = (_type)_cvalue;                                     \
-    else                                                                 \
-      _buffer[_ii] = *(_type*)(_pi + _offsets[_ii]);                     \
-  }                                                                      \
-  *(_type*)_po = (_type) NI_Select(_buffer, 0, _filter_size - 1, _rank); \
+#define NI_RANK_FILTER_POINT(_pi, _tmp, _offsets, _filter_size,   \
+                             _cvalue, _type, _buffer, _rank, _mv) \
+{                                                                 \
+  int _ii;                                                        \
+  for(_ii = 0; _ii < _filter_size; _ii++) {                       \
+    int _offset = _offsets[_ii];                                  \
+    if (_offset == _mv)                                           \
+      _buffer[_ii] = (_type)_cvalue;                              \
+    else                                                          \
+      _buffer[_ii] = *(_type*)(_pi + _offsets[_ii]);              \
+  }                                                               \
+  _tmp = (_type) NI_Select(_buffer, 0, _filter_size - 1, _rank);  \
 }
 
-int NI_RankFilter(PyArrayObject* input, int filter_rank, 
-                    double percentile, PyArrayObject* footprint, 
-                    PyArrayObject** output, PyObject* output_in, 
-                    NI_ExtendMode mode, double cvalue, int *shifts, 
-                    char percentile_filter)
+int NI_RankFilter(PyArrayObject* input, int filter_rank, double percentile,
+                  PyArrayObject* footprint, PyArrayObject* output,
+                  NI_ExtendMode mode, double cvalue, int *shifts,
+                  char percentile_filter)
 {
   int filter_size = 0, *offsets = NULL, size, *oo, jj, border_flag_value;
-  int msize, irank, itype, idims[NI_MAXDIM], fdims[NI_MAXDIM];
+  int msize, irank, itype, otype, idims[NI_MAXDIM], fdims[NI_MAXDIM];
   NI_Iterator ii, io;
   NI_FilterIterator fi;
   Bool *pm;
   char *pi, *po;
-  double *buffer = NULL;
+  double *buffer = NULL, tmp;
+
+  /* check the output array: */
+  irank = NI_GetArrayRank(input);
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, idims))
+    goto exit;
 
   /* complex types not supported: */
   itype = NI_GetArrayType(input);
-  if (itype == tComplex32 || itype == tComplex64) {
+  otype = NI_GetArrayType(output);
+  if (itype == tComplex32 || itype == tComplex64 ||
+      otype == tComplex32 || otype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
 
-  irank = NI_GetArrayRank(input);
-
   if (irank != NI_GetArrayRank(footprint)) {
     PyErr_SetString(PyExc_RuntimeError, 
                     "footprint and input arrays must have equal rank");
@@ -937,16 +967,10 @@ int NI_RankFilter(PyArrayObject* input, int filter_rank,
   }
 
   pm = (Bool*)NI_GetArrayData(footprint);
-  msize = NI_Elements(footprint);
+  msize = NI_GetArraySize(footprint);
   for(jj = 0; jj < msize; jj++) 
     if (pm[jj]) ++filter_size;
 
-  /* allocate the output array: */
-  NI_GetArrayDimensions(input, idims);
-  if (!NI_OutputArray((NumarrayType)itype, irank,  idims, output_in,
-                      output))
-    goto exit;
-
   /* filter must be not be emtpy: */
   if (filter_size < 1) {
     PyErr_SetString(PyExc_RuntimeError, "filter size must be > 0");
@@ -977,8 +1001,9 @@ int NI_RankFilter(PyArrayObject* input, int filter_rank,
   }
 
   /* calculate the filter offsets: */
-  if (!NI_InitFilterOffsetsFromArray(input, footprint, shifts, mode,
-                                     &offsets, &border_flag_value, NULL))
+  NI_GetArrayDimensions(footprint, fdims);
+  if (!NI_InitFilterOffsets(input, pm, fdims, shifts, mode,
+                            &offsets, &border_flag_value, NULL))
     goto exit;
 
   /* initialize input element iterator: */
@@ -986,18 +1011,17 @@ int NI_RankFilter(PyArrayObject* input, int filter_rank,
     goto exit;
 
   /* initialize output element iterator: */
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
-  NI_GetArrayDimensions(footprint, fdims);
   if (!NI_InitFilterIterator(irank, fdims, filter_size, idims, shifts,
                              &fi))
     goto exit;
     
   /* get data pointers an size: */
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
-  size = NI_Elements(input);
+  po = NI_GetArrayData(output);
+  size = NI_GetArraySize(input);
 
   /* buffer for rank calculation: */
   buffer = (double*)malloc(filter_size * sizeof(double));
@@ -1011,55 +1035,95 @@ int NI_RankFilter(PyArrayObject* input, int filter_rank,
   for(jj = 0; jj < size; jj++) {
     switch (itype) {
     case tBool:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, Bool,
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Bool,
                            buffer, filter_rank, border_flag_value);
       break;
     case tUInt8:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, UInt8,
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, UInt8,
                            buffer, filter_rank, border_flag_value);
       break;
     case tUInt16:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, UInt16,
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, UInt16,
                            buffer, filter_rank, border_flag_value);
       break;
     case tUInt32:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, UInt32,
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, UInt32,
                            buffer, filter_rank, border_flag_value);
       break;
 #if HAS_UINT64
     case tUInt64:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, UInt64,
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, UInt64,
                            buffer, filter_rank, border_flag_value);
       break;
 #endif
     case tInt8:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, Int8,
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Int8,
                            buffer, filter_rank, border_flag_value);
       break;
     case tInt16:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, Int16,
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Int16,
                            buffer, filter_rank, border_flag_value);
       break;
     case tInt32:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, Int32,
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Int32,
                            buffer, filter_rank, border_flag_value);
       break;
     case tInt64:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, Int64,
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Int64,
                            buffer, filter_rank, border_flag_value);
       break;
     case tFloat32:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, Float32, 
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Float32, 
                            buffer, filter_rank, border_flag_value);
       break;
     case tFloat64:
-      NI_RANK_FILTER_POINT(pi, po, oo, filter_size, cvalue, Float64, 
+      NI_RANK_FILTER_POINT(pi, tmp, oo, filter_size, cvalue, Float64, 
                            buffer, filter_rank, border_flag_value);
       break;
     default:
       PyErr_SetString(PyExc_RuntimeError, "data type not supported");
       goto exit;
     }
+    switch (otype) {
+    case tBool:
+      *(Bool*)po = (Bool)tmp;
+      break;
+    case tUInt8:
+      *(UInt8*)po = (UInt8)tmp;
+      break;
+    case tUInt16:
+      *(UInt16*)po = (UInt16)tmp;
+      break;
+    case tUInt32:
+      *(UInt32*)po = (UInt32)tmp;
+      break;
+#if HAS_UINT64
+    case tUInt64:
+      *(UInt64*)po = (UInt64)tmp;
+      break;
+#endif
+    case tInt8:
+      *(Int8*)po = (Int8)tmp;
+      break;
+    case tInt16:
+      *(Int16*)po = (Int16)tmp;
+      break;
+    case tInt32:
+      *(Int32*)po = (Int32)tmp;
+      break;
+    case tInt64:
+      *(Int64*)po = (Int64)tmp;
+      break;
+    case tFloat32:
+      *(Float32*)po = (Float32)tmp;
+      break;
+    case tFloat64:
+      *(Float64*)po = (Float64)tmp;
+      break;
+    default:
+      PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+      goto exit;
+    }
     NI_FILTER_NEXT2(fi, ii, io, oo, pi, po);
   }
 
@@ -1088,35 +1152,34 @@ int NI_RankFilter(PyArrayObject* input, int filter_rank,
   }                                                                   \
 }
 
-int NI_GenericFilter(PyArrayObject* input, 
-                     int (*function)(double*, int, double*, void*),
-                     void *data, PyArrayObject* footprint, 
-                     PyArrayObject** output, PyObject* output_in, 
-                     NI_ExtendMode mode, double cvalue, int *shifts,
-                     NumarrayType output_type)
+int NI_GenericFilter(PyArrayObject* input, int (*function)(double*, int,
+      double*, void*), void *data, PyArrayObject* footprint, 
+      PyArrayObject* output, NI_ExtendMode mode, double cvalue,
+      int *shifts)
 {
   int filter_size = 0, *offsets = NULL, size, *oo, jj, border_flag_value;
-  int msize, irank, itype, idims[NI_MAXDIM], fdims[NI_MAXDIM];
+  int msize, irank, itype, otype, idims[NI_MAXDIM], fdims[NI_MAXDIM];
   NI_Iterator ii, io;
   NI_FilterIterator fi;
   Bool *pm;
   char *pi, *po;
   double *buffer = NULL;
 
+  /* check the output array: */
+  irank = NI_GetArrayRank(input);
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, idims))
+    goto exit;
+
   /* complex types not supported: */
   itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
   if (itype == tComplex32 || itype == tComplex64 ||
-      output_type == tComplex32 || output_type == tComplex64) {
+      otype == tComplex32 || otype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
 
-  /* if no output type was specified, use the input type: */
-  if (output_type == tAny)
-    output_type = (NumarrayType)itype;
-
-  irank = NI_GetArrayRank(input);
-
   if (irank != NI_GetArrayRank(footprint)) {
     PyErr_SetString(PyExc_RuntimeError, 
                     "footprint and input arrays must have equal rank");
@@ -1137,25 +1200,20 @@ int NI_GenericFilter(PyArrayObject* input,
   }
 
   pm = (Bool*)NI_GetArrayData(footprint);
-  msize = NI_Elements(footprint);
+  msize = NI_GetArraySize(footprint);
   for(jj = 0; jj < msize; jj++) 
     if (pm[jj]) ++filter_size;
 
-  /* allocate the output array: */
-  NI_GetArrayDimensions(input, idims);
-  if (!NI_OutputArray(output_type, irank,  idims, output_in, output))
-    goto exit;
-
   /* filter must be not be emtpy: */
   if (filter_size < 1) {
     PyErr_SetString(PyExc_RuntimeError, "filter size must be > 0");
     goto exit;
   }
 
-
   /* calculate the filter offsets: */
-  if (!NI_InitFilterOffsetsFromArray(input, footprint, shifts, mode,
-                                     &offsets, &border_flag_value, NULL))
+  NI_GetArrayDimensions(footprint, fdims);
+  if (!NI_InitFilterOffsets(input, pm, fdims, shifts, mode,
+                            &offsets, &border_flag_value, NULL))
     goto exit;
 
   /* initialize input element iterator: */
@@ -1163,18 +1221,17 @@ int NI_GenericFilter(PyArrayObject* input,
     goto exit;
 
   /* initialize output element iterator: */
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
-  NI_GetArrayDimensions(footprint, fdims);
   if (!NI_InitFilterIterator(irank, fdims, filter_size, idims, shifts,
                              &fi))
     goto exit;
     
   /* get data pointers an size: */
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
-  size = NI_Elements(input);
+  po = NI_GetArrayData(output);
+  size = NI_GetArraySize(input);
 
   /* buffer for filter calculation: */
   buffer = (double*)malloc(filter_size * sizeof(double));
@@ -1238,7 +1295,7 @@ int NI_GenericFilter(PyArrayObject* input,
       PyErr_SetString(PyExc_RuntimeError, "data type not supported");
       goto exit;
     }
-    switch (output_type) {
+    switch (otype) {
     case tBool:
       *(Bool*)po = (Bool)tmp;
       break;
@@ -1288,28 +1345,30 @@ int NI_GenericFilter(PyArrayObject* input,
 }
 
 
-int NI_GenericFilter1D(PyArrayObject *input, 
-                       int (*function)(double*, int, double*, int, void*),
-                       void *data, int filter_size, int axis,
-                       PyArrayObject **output, PyObject *output_in,
-                       NI_ExtendMode mode, double cval, int shift, 
-                       NumarrayType output_type)
+int NI_GenericFilter1D(PyArrayObject *input, int (*function)(double*, int,
+        double*, int, void*), void *data, int filter_size, int axis,
+        PyArrayObject *output, NI_ExtendMode mode, double cval, int shift)
 {
   int ii, lines, length, more, irank, size1, size2;
-  int itype, idims[NI_MAXDIM];
+  int itype, otype, idims[NI_MAXDIM];
   double *ibuffer = NULL, *obuffer = NULL;
   NI_LineBuffer iline_buffer, oline_buffer;
 
   assert(input != NULL);
   assert(output != NULL);
 
+  /* check the output array: */
+  irank = NI_GetArrayRank(input);
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, idims))
+    goto exit;
+
   /* check filter sizes: */
   if (filter_size < 1) {
     PyErr_SetString(PyExc_RuntimeError, "filter size must be > 0");
     goto exit;
   }
 
-  irank = NI_GetArrayRank(input);
   /* support negative axis specification: */
   if (axis < 0)
     axis += irank;
@@ -1321,22 +1380,14 @@ int NI_GenericFilter1D(PyArrayObject *input,
   }
 
   itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
   /* complex types not supported: */
   if (itype == tComplex32 || itype == tComplex64 ||
-      output_type == tComplex32 || output_type == tComplex64) {
+      otype == tComplex32 || otype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
 
-  /* if no output type was specified, use the input type: */
-  if (output_type == tAny)
-    output_type = (NumarrayType)itype;
-
-  /* allocate the output array: */
-  NI_GetArrayDimensions(input, idims);
-  if (!NI_OutputArray(output_type, irank, idims, output_in, output))
-    goto exit;
-
   /* check the filter shift: */
   size1 = filter_size / 2;
   size2 = filter_size - size1 - 1;
@@ -1352,7 +1403,7 @@ int NI_GenericFilter1D(PyArrayObject *input,
                              &lines, BUFFER_SIZE, &ibuffer))
     goto exit;
 
-  if (!NI_AllocateLineBuffer(*output, axis, 0, 0, &lines, BUFFER_SIZE, 
+  if (!NI_AllocateLineBuffer(output, axis, 0, 0, &lines, BUFFER_SIZE, 
                              &obuffer))
     goto exit;
 
@@ -1360,7 +1411,7 @@ int NI_GenericFilter1D(PyArrayObject *input,
                          ibuffer, mode, cval, &iline_buffer))
     goto exit;
 
-  if (!NI_InitLineBuffer(*output, axis, 0, 0, lines, obuffer, mode, 0.0, 
+  if (!NI_InitLineBuffer(output, axis, 0, 0, lines, obuffer, mode, 0.0, 
                          &oline_buffer))
     goto exit;
 
