@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Peter J. Verveer
+/* Copyright (C) 2003-2005 Peter J. Verveer
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,7 +39,7 @@
 #endif
 
 int NI_FourierGaussian(PyArrayObject *input, double* isigmas, int n,
-                       int axis, PyArrayObject** output, PyObject* output_in)
+                       int axis, PyArrayObject* output)
 {
   NI_Iterator ii, io;
   char *pi, *po;
@@ -63,11 +63,33 @@ int NI_FourierGaussian(PyArrayObject *input, double* isigmas, int n,
     goto exit;
   }
 
+  itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
+  if (itype == tComplex32 || itype == tComplex64) {
+    if (!(otype == tComplex32 || otype == tComplex64)) {
+      PyErr_SetString(PyExc_RuntimeError,
+                      "ouput array type must be complex");
+      goto exit;
+    }
+  } else {
+    if (!(otype == tFloat32 || otype == tFloat64 ||
+        otype == tComplex32 || otype == tComplex64)) {
+      PyErr_SetString(PyExc_RuntimeError,
+                      "ouput array type must be float or complex");
+      goto exit;  
+    }
+  }
+
   /* output shape is equal to the input shape: */
   NI_GetArrayDimensions(input, idims);
   for(kk = 0; kk < irank; kk++)
     oshape[kk] = idims[kk];
   
+  /* check the output array: */
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, oshape))
+    goto exit;
+
   /* precalculate the sigmas: */
   sigmas = (double*)malloc(irank * sizeof(double));
   if (!sigmas) {
@@ -125,100 +147,111 @@ int NI_FourierGaussian(PyArrayObject *input, double* isigmas, int n,
     }
   }
   
-  itype = NI_GetArrayType(input);
-  switch(itype) {
-  case tBool:
-  case tUInt8:
-  case tUInt16:
-  case tUInt32:
-#if HAS_UINT64
-  case tUInt64:
-#endif
-  case tInt8:
-  case tInt16:
-  case tInt32:
-  case tInt64:
-    otype = tFloat64;
-    break;
-  case tFloat32:
-  case tFloat64:
-  case tComplex32:
-  case tComplex64:
-    otype = itype;
-    break;
-  default:
-    PyErr_SetString(PyExc_RuntimeError, "array type not supported");
-    goto exit;
-  }
-
-  /* allocate the output array: */
-  if (!NI_OutputArray(otype, irank, oshape, output_in, output))
-    goto exit;
-
   /* initialize input element iterator: */
   if (!NI_InitPointIterator(input, &ii))
     goto exit;
 
   /* initialize output element iterator: */
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
-  size = NI_Elements(input);
+  po = NI_GetArrayData(output);
+  size = NI_GetArraySize(input);
 
   /* iterator over the elements: */
   for(hh = 0; hh < size; hh++) {
     double tmp = params[0][ii.coordinates[0]];
     for(kk = 1; kk < irank; kk++)
       tmp *= params[kk][ii.coordinates[kk]];
-    switch (itype) {
-    case tBool:
-      *(Float64*)po = *(Bool*)pi * tmp;
-      break;
-    case tUInt8:
-      *(Float64*)po = *(UInt8*)pi * tmp;
-      break;
-    case tUInt16:
-      *(Float64*)po = *(UInt16*)pi * tmp;
-      break;
-    case tUInt32:
-      *(Float64*)po = *(UInt32*)pi * tmp;
-      break;
+    if (itype == tComplex32 || itype == tComplex64) {
+      double tmp_r = 0.0, tmp_i = 0.0;
+      switch (itype) {
+      case tComplex32:
+        tmp_r = (*(Complex32*)pi).r * tmp;
+        tmp_i = (*(Complex32*)pi).i * tmp;
+        break;
+      case tComplex64:
+        tmp_r = (*(Complex64*)pi).r * tmp;
+        tmp_i = (*(Complex64*)pi).i * tmp;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
+      switch (otype) {
+      case tComplex32:
+        (*(Complex32*)po).r = tmp_r;
+        (*(Complex32*)po).i = tmp_i;
+        break;
+      case tComplex64:
+        (*(Complex64*)po).r = tmp_r;
+        (*(Complex64*)po).i = tmp_i;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
+    } else {
+      switch (itype) {
+      case tBool:
+        tmp *= *(Bool*)pi;
+        break;
+      case tUInt8:
+        tmp *= *(UInt8*)pi;
+        break;
+      case tUInt16:
+        tmp *= *(UInt16*)pi;
+        break;
+      case tUInt32:
+        tmp *= *(UInt32*)pi;
+        break;
 #if HAS_UINT64
-    case tUInt64:
-      *(Float64*)po = *(UInt64*)pi * tmp;
-      break;
+      case tUInt64:
+        tmp *= *(UInt64*)pi;
+        break;
 #endif
-    case tInt8:
-      *(Float64*)po = *(Int8*)pi * tmp;
-      break;
-    case tInt16:
-      *(Float64*)po = *(Int16*)pi * tmp;
-      break;
-    case tInt32:
-      *(Float64*)po = *(Int32*)pi * tmp;
-      break;
-    case tInt64:
-      *(Float64*)po = *(Int64*)pi * tmp;
-      break;
-    case tFloat32:
-      *(Float32*)po = *(Float32*)pi * tmp;
-      break;
-    case tFloat64:
-      *(Float64*)po = *(Float64*)pi * tmp;
-      break;
-    case tComplex32:
-      (*(Complex32*)po).r = (*(Complex32*)pi).r * tmp;
-      (*(Complex32*)po).i = (*(Complex32*)pi).i * tmp;
-      break;
-    case tComplex64:
-      (*(Complex64*)po).r = (*(Complex64*)pi).r * tmp;
-      (*(Complex64*)po).i = (*(Complex64*)pi).i * tmp;
-      break;
-    default:
-      PyErr_SetString(PyExc_RuntimeError, "data type not supported");
-      goto exit;
+      case tInt8:
+        tmp *= *(Int8*)pi;
+        break;
+      case tInt16:
+        tmp *= *(Int16*)pi;
+        break;
+      case tInt32:
+        tmp *= *(Int32*)pi;
+        break;
+      case tInt64:
+        tmp *= *(Int64*)pi;
+        break;
+      case tFloat32:
+        tmp *= *(Float32*)pi;
+        break;
+      case tFloat64:
+        tmp *= *(Float64*)pi;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
+      switch (otype) {
+      case tFloat32:
+        *(Float32*)po = tmp;
+        break;
+      case tFloat64:
+        *(Float64*)po = tmp;
+        break;
+      case tComplex32:
+        (*(Complex32*)po).r = tmp;
+        (*(Complex32*)po).i = 0.0;
+        break;
+      case tComplex64:
+        (*(Complex64*)po).r = tmp;
+        (*(Complex64*)po).i = 0.0;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
     }
     NI_ITERATOR_NEXT2(ii, io, pi, po);
   }
@@ -236,8 +269,8 @@ int NI_FourierGaussian(PyArrayObject *input, double* isigmas, int n,
 }
 
 
-int NI_FourierBoxcar(PyArrayObject *input, double* sizes, int n,
-                     int axis, PyArrayObject** output, PyObject* output_in)
+int NI_FourierUniform(PyArrayObject *input, double* sizes, int n,
+                      int axis, PyArrayObject* output)
 {
   NI_Iterator ii, io;
   char *pi, *po;
@@ -261,11 +294,33 @@ int NI_FourierBoxcar(PyArrayObject *input, double* sizes, int n,
     goto exit;
   }
 
+  itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
+  if (itype == tComplex32 || itype == tComplex64) {
+    if (!(otype == tComplex32 || otype == tComplex64)) {
+      PyErr_SetString(PyExc_RuntimeError,
+                      "ouput array type must be complex");
+      goto exit;
+    }
+  } else {
+    if (!(otype == tFloat32 || otype == tFloat64 ||
+        otype == tComplex32 || otype == tComplex64)) {
+      PyErr_SetString(PyExc_RuntimeError,
+                      "ouput array type must be float or complex");
+      goto exit;  
+    }
+  }
+
   /* output shape is equal to the input shape: */
   NI_GetArrayDimensions(input, idims);
   for(kk = 0; kk < irank; kk++)
     oshape[kk] = idims[kk];
   
+  /* check the output array: */
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, oshape))
+    goto exit;
+  
   /* allocate memory for tables: */
   params = (double**) malloc(irank * sizeof(double*));
   if (!params) {
@@ -305,100 +360,111 @@ int NI_FourierBoxcar(PyArrayObject *input, double* sizes, int n,
     }
   }
 
-  itype = NI_GetArrayType(input);
-  switch(itype) {
-  case tBool:
-  case tUInt8:
-  case tUInt16:
-  case tUInt32:
-#if HAS_UINT64
-  case tUInt64:
-#endif
-  case tInt8:
-  case tInt16:
-  case tInt32:
-  case tInt64:
-    otype = tFloat64;
-    break;
-  case tFloat32:
-  case tFloat64:
-  case tComplex32:
-  case tComplex64:
-    otype = itype;
-    break;
-  default:
-    PyErr_SetString(PyExc_RuntimeError, "array type not supported");
-    goto exit;
-  }
-
-  /* allocate the output array: */
-  if (!NI_OutputArray(otype, irank, oshape, output_in, output))
-    goto exit;
-
   /* initialize input element iterator: */
   if (!NI_InitPointIterator(input, &ii))
     goto exit;
 
   /* initialize output element iterator: */
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
-  size = NI_Elements(input);
+  po = NI_GetArrayData(output);
+  size = NI_GetArraySize(input);
 
   /* iterator over the elements: */
   for(hh = 0; hh < size; hh++) {
     double tmp = params[0][ii.coordinates[0]];
     for(kk = 1; kk < irank; kk++)
       tmp *= params[kk][ii.coordinates[kk]];
-    switch (itype) {
-    case tBool:
-      *(Float64*)po = *(Bool*)pi * tmp;
-      break;
-    case tUInt8:
-      *(Float64*)po = *(UInt8*)pi * tmp;
-      break;
-    case tUInt16:
-      *(Float64*)po = *(UInt16*)pi * tmp;
-      break;
-    case tUInt32:
-      *(Float64*)po = *(UInt32*)pi * tmp;
-      break;
+    if (itype == tComplex32 || itype == tComplex64) {
+      double tmp_r = 0.0, tmp_i = 0.0;
+      switch (itype) {
+      case tComplex32:
+        tmp_r = (*(Complex32*)pi).r * tmp;
+        tmp_i = (*(Complex32*)pi).i * tmp;
+        break;
+      case tComplex64:
+        tmp_r = (*(Complex64*)pi).r * tmp;
+        tmp_i = (*(Complex64*)pi).i * tmp;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
+      switch (otype) {
+      case tComplex32:
+        (*(Complex32*)po).r = tmp_r;
+        (*(Complex32*)po).i = tmp_i;
+        break;
+      case tComplex64:
+        (*(Complex64*)po).r = tmp_r;
+        (*(Complex64*)po).i = tmp_i;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
+    } else {
+      switch (itype) {
+      case tBool:
+        tmp *= *(Bool*)pi;
+        break;
+      case tUInt8:
+        tmp *= *(UInt8*)pi;
+        break;
+      case tUInt16:
+        tmp *= *(UInt16*)pi;
+        break;
+      case tUInt32:
+        tmp *= *(UInt32*)pi;
+        break;
 #if HAS_UINT64
-    case tUInt64:
-      *(Float64*)po = *(UInt64*)pi * tmp;
-      break;
+      case tUInt64:
+        tmp *= *(UInt64*)pi;
+        break;
 #endif
-    case tInt8:
-      *(Float64*)po = *(Int8*)pi * tmp;
-      break;
-    case tInt16:
-      *(Float64*)po = *(Int16*)pi * tmp;
-      break;
-    case tInt32:
-      *(Float64*)po = *(Int32*)pi * tmp;
-      break;
-    case tInt64:
-      *(Float64*)po = *(Int64*)pi * tmp;
-      break;
-    case tFloat32:
-      *(Float32*)po = *(Float32*)pi * tmp;
-      break;
-    case tFloat64:
-      *(Float64*)po = *(Float64*)pi * tmp;
-      break;
-    case tComplex32:
-      (*(Complex32*)po).r = (*(Complex32*)pi).r * tmp;
-      (*(Complex32*)po).i = (*(Complex32*)pi).i * tmp;
-      break;
-    case tComplex64:
-      (*(Complex64*)po).r = (*(Complex64*)pi).r * tmp;
-      (*(Complex64*)po).i = (*(Complex64*)pi).i * tmp;
-      break;
-    default:
-      PyErr_SetString(PyExc_RuntimeError, "data type not supported");
-      goto exit;
+      case tInt8:
+        tmp *= *(Int8*)pi;
+        break;
+      case tInt16:
+        tmp *= *(Int16*)pi;
+        break;
+      case tInt32:
+        tmp *= *(Int32*)pi;
+        break;
+      case tInt64:
+        tmp *= *(Int64*)pi;
+        break;
+      case tFloat32:
+        tmp *= *(Float32*)pi;
+        break;
+      case tFloat64:
+        tmp *= *(Float64*)pi;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
+      switch (otype) {
+      case tFloat32:
+        *(Float32*)po = tmp;
+        break;
+      case tFloat64:
+        *(Float64*)po = tmp;
+        break;
+      case tComplex32:
+        (*(Complex32*)po).r = tmp;
+        (*(Complex32*)po).i = 0.0;
+        break;
+      case tComplex64:
+        (*(Complex64*)po).r = tmp;
+        (*(Complex64*)po).i = 0.0;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
     }
     NI_ITERATOR_NEXT2(ii, io, pi, po);
   }
@@ -525,7 +591,7 @@ static double _bessel_j1(double x)
 
 
 int NI_FourierEllipsoid(PyArrayObject *input, double* sizes, int n,
-                        int axis, PyArrayObject** output, PyObject* output_in)
+                        int axis, PyArrayObject* output)
 {
   NI_Iterator ii, io;
   char *pi, *po;
@@ -555,11 +621,33 @@ int NI_FourierEllipsoid(PyArrayObject *input, double* sizes, int n,
     goto exit;
   }
 
+  itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
+  if (itype == tComplex32 || itype == tComplex64) {
+    if (!(otype == tComplex32 || otype == tComplex64)) {
+      PyErr_SetString(PyExc_RuntimeError,
+                      "ouput array type must be complex");
+      goto exit;
+    }
+  } else {
+    if (!(otype == tFloat32 || otype == tFloat64 ||
+        otype == tComplex32 || otype == tComplex64)) {
+      PyErr_SetString(PyExc_RuntimeError,
+                      "ouput array type must be float or complex");
+      goto exit;  
+    }
+  }
+
   /* output shape is equal to the input shape: */
   NI_GetArrayDimensions(input, idims);
   for(kk = 0; kk < irank; kk++)
     oshape[kk] = idims[kk];
   
+  /* check the output array: */
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, oshape))
+    goto exit;
+  
   /* allocate memory for tables: */
   params = (double**) malloc(irank * sizeof(double*));
   if (!params) {
@@ -603,47 +691,18 @@ int NI_FourierEllipsoid(PyArrayObject *input, double* sizes, int n,
       for(kk = 0; kk < oshape[hh]; kk++)
         params[hh][kk] = params[hh][kk] * params[hh][kk];
 
-  itype = NI_GetArrayType(input);
-  switch(itype) {
-  case tBool:
-  case tUInt8:
-  case tUInt16:
-  case tUInt32:
-#if HAS_UINT64
-  case tUInt64:
-#endif
-  case tInt8:
-  case tInt16:
-  case tInt32:
-  case tInt64:
-    otype = tFloat64;
-    break;
-  case tFloat32:
-  case tFloat64:
-  case tComplex32:
-  case tComplex64:
-    otype = itype;
-    break;
-  default:
-    PyErr_SetString(PyExc_RuntimeError, "array type not supported");
-    goto exit;
-  }
-
-  /* allocate the output array: */
-  if (!NI_OutputArray(otype, irank, oshape, output_in, output))
-    goto exit;
 
   /* initialize input element iterator: */
   if (!NI_InitPointIterator(input, &ii))
     goto exit;
 
   /* initialize output element iterator: */
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
-  size = NI_Elements(input);
+  po = NI_GetArrayData(output);
+  size = NI_GetArraySize(input);
 
   /* iterator over the elements: */
   for(hh = 0; hh < size; hh++) {
@@ -675,53 +734,94 @@ int NI_FourierEllipsoid(PyArrayObject *input, double* sizes, int n,
       }
       break;
     }
-    switch (itype) {
-    case tBool:
-      *(Float64*)po = *(Bool*)pi * tmp;
-      break;
-    case tUInt8:
-      *(Float64*)po = *(UInt8*)pi * tmp;
-      break;
-    case tUInt16:
-      *(Float64*)po = *(UInt16*)pi * tmp;
-      break;
-    case tUInt32:
-      *(Float64*)po = *(UInt32*)pi * tmp;
-      break;
+    if (itype == tComplex32 || itype == tComplex64) {
+      double tmp_r = 0.0, tmp_i = 0.0;
+      switch (itype) {
+      case tComplex32:
+        tmp_r = (*(Complex32*)pi).r * tmp;
+        tmp_i = (*(Complex32*)pi).i * tmp;
+        break;
+      case tComplex64:
+        tmp_r = (*(Complex64*)pi).r * tmp;
+        tmp_i = (*(Complex64*)pi).i * tmp;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
+      switch (otype) {
+      case tComplex32:
+        (*(Complex32*)po).r = tmp_r;
+        (*(Complex32*)po).i = tmp_i;
+        break;
+      case tComplex64:
+        (*(Complex64*)po).r = tmp_r;
+        (*(Complex64*)po).i = tmp_i;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
+    } else {
+      switch (itype) {
+      case tBool:
+        tmp *= *(Bool*)pi;
+        break;
+      case tUInt8:
+        tmp *= *(UInt8*)pi;
+        break;
+      case tUInt16:
+        tmp *= *(UInt16*)pi;
+        break;
+      case tUInt32:
+        tmp *= *(UInt32*)pi;
+        break;
 #if HAS_UINT64
-    case tUInt64:
-      *(Float64*)po = *(UInt64*)pi * tmp;
-      break;
+      case tUInt64:
+        tmp *= *(UInt64*)pi;
+        break;
 #endif
-    case tInt8:
-      *(Float64*)po = *(Int8*)pi * tmp;
-      break;
-    case tInt16:
-      *(Float64*)po = *(Int16*)pi * tmp;
-      break;
-    case tInt32:
-      *(Float64*)po = *(Int32*)pi * tmp;
-      break;
-    case tInt64:
-      *(Float64*)po = *(Int64*)pi * tmp;
-      break;
-    case tFloat32:
-      *(Float32*)po = *(Float32*)pi * tmp;
-      break;
-    case tFloat64:
-      *(Float64*)po = *(Float64*)pi * tmp;
-      break;
-    case tComplex32:
-      (*(Complex32*)po).r = (*(Complex32*)pi).r * tmp;
-      (*(Complex32*)po).i = (*(Complex32*)pi).i * tmp;
-      break;
-    case tComplex64:
-      (*(Complex64*)po).r = (*(Complex64*)pi).r * tmp;
-      (*(Complex64*)po).i = (*(Complex64*)pi).i * tmp;
-      break;
-    default:
-      PyErr_SetString(PyExc_RuntimeError, "data type not supported");
-      goto exit;
+      case tInt8:
+        tmp *= *(Int8*)pi;
+        break;
+      case tInt16:
+        tmp *= *(Int16*)pi;
+        break;
+      case tInt32:
+        tmp *= *(Int32*)pi;
+        break;
+      case tInt64:
+        tmp *= *(Int64*)pi;
+        break;
+      case tFloat32:
+        tmp *= *(Float32*)pi;
+        break;
+      case tFloat64:
+        tmp *= *(Float64*)pi;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
+      switch (otype) {
+      case tFloat32:
+        *(Float32*)po = tmp;
+        break;
+      case tFloat64:
+        *(Float64*)po = tmp;
+        break;
+      case tComplex32:
+        (*(Complex32*)po).r = tmp;
+        (*(Complex32*)po).i = 0.0;
+        break;
+      case tComplex64:
+        (*(Complex64*)po).r = tmp;
+        (*(Complex64*)po).i = 0.0;
+        break;
+      default:
+        PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+        goto exit;
+      }
     }
     NI_ITERATOR_NEXT2(ii, io, pi, po);
   }
@@ -738,7 +838,7 @@ int NI_FourierEllipsoid(PyArrayObject *input, double* sizes, int n,
 
 
 int NI_FourierShift(PyArrayObject *input, double* ishift, int n, int axis, 
-                    PyArrayObject** output, PyObject* output_in)
+                    PyArrayObject* output)
 {
   NI_Iterator ii, io;
   char *pi, *po;
@@ -762,11 +862,24 @@ int NI_FourierShift(PyArrayObject *input, double* ishift, int n, int axis,
     goto exit;
   }
 
+  itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
+  if (!(otype == tComplex32 || otype == tComplex64)) {
+    PyErr_SetString(PyExc_RuntimeError,
+                    "ouput array type must be complex");
+    goto exit;
+  }
+
   /* output shape is equal to the input shape: */
   NI_GetArrayDimensions(input, idims);
   for(kk = 0; kk < irank; kk++)
     oshape[kk] = idims[kk];
   
+  /* check the output array: */
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, oshape))
+    goto exit;
+  
   /* precalculate the shifts: */
   shifts = (double*)malloc(irank * sizeof(double));
   if (!shifts) {
@@ -815,62 +928,21 @@ int NI_FourierShift(PyArrayObject *input, double* ishift, int n, int axis,
     }
   }
 
-  itype = NI_GetArrayType(input);
-  otype = itype;
-  if (otype == tFloat32)
-    otype = tComplex32;
-  if (otype == tFloat64)
-    otype = tComplex64;
-  switch(itype) {
-  case tBool:
-  case tUInt8:
-  case tUInt16:
-  case tUInt32:
-#if HAS_UINT64
-  case tUInt64:
-#endif
-  case tInt8:
-  case tInt16:
-  case tInt32:
-  case tInt64:
-    otype = tComplex64;
-    break;
-  case tFloat32:
-    otype = tComplex32;
-    break;
-  case tFloat64:
-    otype = tComplex64;
-    break;
-  case tComplex32:
-    otype = tComplex32;
-    break;
-  case tComplex64:
-    otype = tComplex64;
-    break;
-  default:
-    PyErr_SetString(PyExc_RuntimeError, "array type not supported");
-    goto exit;
-  }
-
-  /* allocate the output array: */
-  if (!NI_OutputArray(otype, irank, oshape, output_in, output))
-    goto exit;
-
   /* initialize input element iterator: */
   if (!NI_InitPointIterator(input, &ii))
     goto exit;
 
   /* initialize output element iterator: */
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
-  size = NI_Elements(input);
+  po = NI_GetArrayData(output);
+  size = NI_GetArraySize(input);
 
   /* iterator over the elements: */
   for(hh = 0; hh < size; hh++) {
-    double tmp = 0.0, sint, cost;
+    double tmp = 0.0, sint, cost, r = 0.0, i = 0.0;
     for(kk = 0; kk < irank; kk++)
       tmp += params[kk][ii.coordinates[kk]];
     sint = sin(tmp);
@@ -878,76 +950,81 @@ int NI_FourierShift(PyArrayObject *input, double* ishift, int n, int axis,
     switch (itype) {
     case tBool:
       tmp = *(Bool*)pi;
-      (*(Complex64*)po).r = tmp * cost;
-      (*(Complex64*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
     case tUInt8:
       tmp = *(UInt8*)pi;
-      (*(Complex64*)po).r = tmp * cost;
-      (*(Complex64*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
     case tUInt16:
       tmp = *(UInt16*)pi;
-      (*(Complex64*)po).r = tmp * cost;
-      (*(Complex64*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
     case tUInt32:
       tmp = *(UInt32*)pi;
-      (*(Complex64*)po).r = tmp * cost;
-      (*(Complex64*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
 #if HAS_UINT64
     case tUInt64:
       tmp = *(UInt64*)pi;
-      (*(Complex64*)po).r = tmp * cost;
-      (*(Complex64*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
 #endif
     case tInt8:
       tmp = *(Int8*)pi;
-      (*(Complex64*)po).r = tmp * cost;
-      (*(Complex64*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
     case tInt16:
       tmp = *(Int16*)pi;
-      (*(Complex64*)po).r = tmp * cost;
-      (*(Complex64*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
     case tInt32:
       tmp = *(Int32*)pi;
-      (*(Complex64*)po).r = tmp * cost;
-      (*(Complex64*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
     case tInt64:
       tmp = *(Int64*)pi;
-      (*(Complex64*)po).r = tmp * cost;
-      (*(Complex64*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
     case tFloat32:
       tmp = *(Float32*)pi;
-      (*(Complex32*)po).r = tmp * cost;
-      (*(Complex32*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
     case tFloat64:
       tmp = *(Float64*)pi;
-      (*(Complex64*)po).r = tmp * cost;
-      (*(Complex64*)po).i = tmp * sint;
+      r = tmp * cost;
+      i = tmp * sint;
       break;
     case tComplex32:
-      {
-        double r = (*(Complex32*)pi).r * cost - (*(Complex32*)pi).i * sint;
-        double i = (*(Complex32*)pi).r * sint + (*(Complex32*)pi).i * cost;
-        (*(Complex32*)po).r = r;
-        (*(Complex32*)po).i = i;
-      }
+      r = (*(Complex32*)pi).r * cost - (*(Complex32*)pi).i * sint;
+      i = (*(Complex32*)pi).r * sint + (*(Complex32*)pi).i * cost;
       break;
     case tComplex64:
-      {
-        double r = (*(Complex64*)pi).r * cost - (*(Complex64*)pi).i * sint;
-        double i = (*(Complex64*)pi).r * sint + (*(Complex64*)pi).i * cost;
-        (*(Complex64*)po).r = r;
-        (*(Complex64*)po).i = i;
-      }
+      r = (*(Complex64*)pi).r * cost - (*(Complex64*)pi).i * sint;
+      i = (*(Complex64*)pi).r * sint + (*(Complex64*)pi).i * cost;
+      break;
+    default:
+      PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+      goto exit;
+    }
+    switch (otype) {
+    case tComplex32:
+      (*(Complex32*)po).r = r;
+      (*(Complex32*)po).i = i;
+      break;
+    case tComplex64:
+      (*(Complex64*)po).r = r;
+      (*(Complex64*)po).i = i;
       break;
     default:
       PyErr_SetString(PyExc_RuntimeError, "data type not supported");
