@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Peter J. Verveer
+/* Copyright (C) 2003-2005 Peter J. Verveer
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -203,10 +203,9 @@ map_coordinate(double in, int len, int mode, int border_flag_value)
 
 /* one-dimensional spline filter: */
 int NI_SplineFilter1D(PyArrayObject *input, int order, int axis, 
-                      PyArrayObject **output, PyObject *output_in, 
-                      NumarrayType output_type)
+                      PyArrayObject *output)
 {
-  int hh, kk, ll, lines, len, npoles = 0, more, irank, itype;
+  int hh, kk, ll, lines, len, npoles = 0, more, irank, itype, otype;
   int idims[NI_MAXDIM];
   double *buffer = NULL, weight, pole[2];
   NI_LineBuffer iline_buffer, oline_buffer;
@@ -233,24 +232,20 @@ int NI_SplineFilter1D(PyArrayObject *input, int order, int axis,
     goto exit;
   }
 
+  /* check the output array: */
+  NI_GetArrayDimensions(input, idims);
+  if (!NI_CheckArray(output, tAny, irank, idims))
+    goto exit;
+
   /* complex data not supported: */
   itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
   if (itype == tComplex32 || itype == tComplex64 ||
-      output_type == tComplex32 || output_type == tComplex64) {
+      otype == tComplex32 || otype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
 
-  /* if no output type is specifified, use input type: */
-  if (output_type == tAny)
-    output_type = (NumarrayType)itype;
-
-  /* allocate output array: */
-  NI_GetArrayDimensions(input, idims);
-  if (!NI_OutputArray(output_type, irank, idims, output_in, output))
-    goto exit;
-
-
   len = irank > 0 ? idims[axis] : 1;
   if (len < 1)
     goto exit;
@@ -292,7 +287,7 @@ int NI_SplineFilter1D(PyArrayObject *input, int order, int axis,
   if (!NI_InitLineBuffer(input, axis, 0, 0, lines, buffer,
                          NI_EXTEND_DEFAULT, 0.0, &iline_buffer))
     goto exit;
-  if (!NI_InitLineBuffer(*output, axis, 0, 0, lines, buffer,
+  if (!NI_InitLineBuffer(output, axis, 0, 0, lines, buffer,
                          NI_EXTEND_DEFAULT, 0.0, &oline_buffer))
     goto exit;
   
@@ -353,21 +348,17 @@ int NI_SplineFilter1D(PyArrayObject *input, int order, int axis,
 
 
 static int 
-NI_GeometricTransformLow(PyArrayObject *input, 
-                         int (*map)(int*, double*, int, int, void*),
-                         void* map_data, double* matrix, double* shift,
-                         PyArrayObject *coordinates, 
-                         int *output_dimensions,
-                         int output_rank, NumarrayType output_type, 
-                         PyArrayObject **output, PyObject *output_in,
-                         int order, int mode, double cval)
+NI_GeometricTransformLow(PyArrayObject *input, int (*map)(int*, double*, 
+        int, int, void*), void* map_data, double* matrix, double* shift, 
+        PyArrayObject *coordinates, PyArrayObject *output, int order, int 
+        mode, double cval)
 {
   char *po, *pi, *pc = NULL;
   int **edge_offsets = NULL, **data_offsets = NULL, filter_size, size;
   int ftmp[NI_MAXDIM], *fcoordinates = NULL, *foffsets = NULL, jj, kk, hh;
-  int cstride = 0, irank, itype, coordinates_type = tAny, idims[NI_MAXDIM];
+  int cstride = 0, irank, orank, itype, otype, coordinates_type = tAny;
   int coordinates_dims[NI_MAXDIM], coordinates_strides[NI_MAXDIM];
-  int istrides[NI_MAXDIM], ll;
+  int istrides[NI_MAXDIM], ll, idims[NI_MAXDIM], odims[NI_MAXDIM];
   double **splvals = NULL, icoor[NI_MAXDIM];
   NI_Iterator io, ic;
 
@@ -375,10 +366,12 @@ NI_GeometricTransformLow(PyArrayObject *input,
   assert(output != NULL);
 
   irank = NI_GetArrayRank(input);
+  orank = NI_GetArrayRank(output);
   NI_GetArrayDimensions(input, idims);
+  NI_GetArrayDimensions(output, odims);
 
   /* check rank: */
-  if (irank < 1 || output_rank < 1) {
+  if (irank < 1 || orank < 1) {
     PyErr_SetString(PyExc_RuntimeError,
                     "input and output ranks must be > 0");
     goto exit;
@@ -392,8 +385,9 @@ NI_GeometricTransformLow(PyArrayObject *input,
 
   /* complex types are not supported: */
   itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
   if (itype == tComplex32 || itype == tComplex64 ||
-      output_type == tComplex32 || output_type == tComplex64) {
+      otype == tComplex32 || otype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
@@ -401,15 +395,15 @@ NI_GeometricTransformLow(PyArrayObject *input,
   /* if the mapping is from array coordinates: */
   if (coordinates) {
     /* check coordinates rank: */
-    if (output_rank != NI_GetArrayRank(coordinates) - 1) {
+    if (orank != NI_GetArrayRank(coordinates) - 1) {
       PyErr_SetString(PyExc_RuntimeError, 
                       "coordinate array rank incorrect");
       goto exit;
     }
     /* check coordinates shape: */
     NI_GetArrayDimensions(coordinates, coordinates_dims);
-    for(jj = 0; jj < output_rank; jj++)
-      if (output_dimensions[jj] != coordinates_dims[jj + 1]) {
+    for(jj = 0; jj < orank; jj++)
+      if (odims[jj] != coordinates_dims[jj + 1]) {
         PyErr_SetString(PyExc_RuntimeError, 
                         "coordinate array shape incorrect");
         goto exit;
@@ -421,7 +415,8 @@ NI_GeometricTransformLow(PyArrayObject *input,
     }
     /* complex type not supported: */
     coordinates_type = NI_GetArrayType(coordinates);
-    if (coordinates_type == tComplex32 || coordinates_type == tComplex64) {
+    if (coordinates_type == tComplex32 || 
+        coordinates_type == tComplex64) {
       PyErr_SetString(PyExc_RuntimeError, 
                       "complex coordinate arrays not supported");
       goto exit;
@@ -476,18 +471,13 @@ NI_GeometricTransformLow(PyArrayObject *input,
   for(jj = 0; jj < irank; jj++)
     filter_size *= order + 1;
 
-  /* allocate output: */
-  if (!NI_OutputArray(output_type, output_rank, output_dimensions, 
-                      output_in, output))
-    goto exit;
-
   /* initialize output iterator: */
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
   /* get data pointers: */
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
+  po = NI_GetArrayData(output);
     
 
   /* make a table of all possible coordinates within the spline filter: */
@@ -518,14 +508,14 @@ NI_GeometricTransformLow(PyArrayObject *input,
     }
   }
 
-  size = NI_Elements(*output);
+  size = NI_GetArraySize(output);
 
   for(kk = 0; kk < size; kk++) {
     double t = 0.0;
     int constant = 0, edge = 0, offset = 0;
     if (map) {
       /* call mappint functions: */
-      if (!map(io.coordinates, icoor, output_rank, irank, map_data)) {
+      if (!map(io.coordinates, icoor, orank, irank, map_data)) {
         if (!PyErr_Occurred())
           PyErr_SetString(PyExc_RuntimeError,
                           "unknown error in mapping function");
@@ -536,7 +526,7 @@ NI_GeometricTransformLow(PyArrayObject *input,
       double *p = matrix;
       for(hh = 0; hh < irank; hh++) {
         icoor[hh] = 0.0;
-        for(ll = 0; ll < output_rank; ll++)
+        for(ll = 0; ll < orank; ll++)
           icoor[hh] += io.coordinates[ll] * *p++;
         icoor[hh] += shift[hh];
       }
@@ -734,7 +724,7 @@ NI_GeometricTransformLow(PyArrayObject *input,
       t = cval;
     }
     /* store output value: */
-    switch (output_type) {
+    switch (otype) {
     case tBool:
       *(Bool*)po = (Bool)t;
       break;
@@ -801,74 +791,65 @@ NI_GeometricTransformLow(PyArrayObject *input,
   return PyErr_Occurred() ? 0 : 1;
 }
 
-int NI_GeometricTransform(PyArrayObject *input, 
-                          int (*map)(int*, double*, int, int, void*),
-                          void* map_data, int *output_dimensions,
-                          int output_rank, NumarrayType output_type, 
-                          PyArrayObject **output, PyObject *output_in,
-                          int order, int mode, double cval)
+int NI_GeometricTransform(PyArrayObject *input,  int (*map)(int*, double*, 
+              int, int, void*), void* map_data, PyArrayObject *output,
+              int order, int mode, double cval)
 {
   assert(map != NULL);
 
   return NI_GeometricTransformLow(input, map, map_data, NULL, NULL, NULL,
-                                  output_dimensions, output_rank, output_type, 
-                                  output, output_in, order, mode, cval);
+                                  output, order, mode, cval);
 }
 
 int NI_MapCoordinates(PyArrayObject *input, PyArrayObject *coordinates,
-                      int *output_dimensions, int output_rank, 
-                      NumarrayType output_type, PyArrayObject **output, 
-                      PyObject *output_in, int order, int mode, 
-                      double cval)
+                  PyArrayObject *output, int order, int mode, double cval)
 {
   assert(coordinates != NULL);
 
   return NI_GeometricTransformLow(input, NULL, NULL, NULL, NULL,
-                                  coordinates, output_dimensions,
-                                  output_rank, output_type, output,
-                                  output_in, order, mode, cval);
+        coordinates, output, order, mode, cval);
 }
 
-int NI_AffineTransform(PyArrayObject *input, double *matrix, double* shift,
-                       int *output_dimensions, int output_rank, 
-                       NumarrayType output_type, PyArrayObject **output, 
-                       PyObject *output_in, int order, int mode, 
-                       double cval)
+int NI_AffineTransform(PyArrayObject *input, double *matrix, double* shift, 
+                   PyArrayObject *output, int order, int mode, double cval)
 {
   assert(matrix != NULL && shift != NULL);
 
   return NI_GeometricTransformLow(input, NULL, NULL, matrix, shift, NULL,
-                                  output_dimensions, output_rank,
-                                  output_type, output, output_in, order,
-                                  mode, cval);
+                                  output, order, mode, cval);
 }
 
 int NI_ZoomShift(PyArrayObject *input, double* zooms, double* shifts,
-                   int *output_dimensions, int output_rank, 
-                   NumarrayType output_type, PyArrayObject **output, 
-                   PyObject *output_in, int order, int mode, double cval)
+                 PyArrayObject *output, int order, int mode, double cval)
 {
   char *pi, *po;
-  int **zeros = NULL, **offsets = NULL, ***edge_offsets = NULL, jj, hh, kk;
-  int ftmp[NI_MAXDIM], *fcoordinates = NULL, *foffsets = NULL, filter_size;
-  int size, irank, itype, idims[NI_MAXDIM], istrides[NI_MAXDIM];
+  int **zeros = NULL, **offsets = NULL, ***edge_offsets = NULL;
+  int ftmp[NI_MAXDIM], *fcoordinates = NULL, *foffsets = NULL; 
+  int jj, hh, kk, filter_size, size, irank, orank, itype, otype;
+  int idims[NI_MAXDIM], odims[NI_MAXDIM], istrides[NI_MAXDIM];
   double ***splvals = NULL;
   NI_Iterator io;
 
   assert(input != NULL);
   assert(zooms != NULL || shifts != NULL);
-  assert(output_dimensions != NULL);
   assert(output != NULL);
 
   irank = NI_GetArrayRank(input);
+  orank = NI_GetArrayRank(output);
   NI_GetArrayDimensions(input, idims);
+  NI_GetArrayDimensions(output, odims);
 
   /* check ranks: */
-  if (irank < 1 || output_rank < 1) {
+  if (irank < 1 || orank < 1) {
     PyErr_SetString(PyExc_RuntimeError, 
                     "input and output ranks must be > 0");
     goto exit;
   }
+  if (irank != orank) {
+    PyErr_SetString(PyExc_RuntimeError, 
+                    "input and output ranks must be equal");
+    goto exit;
+  }
 
   /* check spline order: */
   if (order < 0 || order > 5) {
@@ -878,8 +859,9 @@ int NI_ZoomShift(PyArrayObject *input, double* zooms, double* shifts,
 
   /* complex types not supported: */
   itype = NI_GetArrayType(input);
+  otype = NI_GetArrayType(output);
   if (itype == tComplex32 || itype == tComplex64 ||
-      output_type == tComplex32 || output_type == tComplex64) {
+      otype == tComplex32 || otype == tComplex64) {
     PyErr_SetString(PyExc_RuntimeError, "complex arrays not supported");
     goto exit;
   }
@@ -900,7 +882,7 @@ int NI_ZoomShift(PyArrayObject *input, double* zooms, double* shifts,
     for(jj = 0; jj < irank; jj++)
       zeros[jj] = NULL;
     for(jj = 0; jj < irank; jj++) {
-      zeros[jj] = (int*)malloc(output_dimensions[jj] * sizeof(int));
+      zeros[jj] = (int*)malloc(odims[jj] * sizeof(int));
       if(!zeros[jj]) {
         PyErr_NoMemory();
         goto exit;
@@ -924,16 +906,14 @@ int NI_ZoomShift(PyArrayObject *input, double* zooms, double* shifts,
     edge_offsets[jj] = NULL;
   }
   for(jj = 0; jj < irank; jj++) {
-    offsets[jj] = (int*)malloc(output_dimensions[jj] * sizeof(int));
-    splvals[jj] = (double**)malloc(output_dimensions[jj] *
-                                   sizeof(double*));
-    edge_offsets[jj] = (int**)malloc(output_dimensions[jj] * 
-                                     sizeof(double*));
+    offsets[jj] = (int*)malloc(odims[jj] * sizeof(int));
+    splvals[jj] = (double**)malloc(odims[jj] * sizeof(double*));
+    edge_offsets[jj] = (int**)malloc(odims[jj] * sizeof(double*));
     if (!offsets[jj] || !splvals[jj] || !edge_offsets[jj]) {
       PyErr_NoMemory();
       goto exit;
     }  
-    for(hh = 0; hh < output_dimensions[jj]; hh++) {
+    for(hh = 0; hh < odims[jj]; hh++) {
       splvals[jj][hh] = NULL;
       edge_offsets[jj][hh] = NULL;
     }
@@ -948,7 +928,7 @@ int NI_ZoomShift(PyArrayObject *input, double* zooms, double* shifts,
       shift = shifts[jj];
     if (zooms)
       zoom = zooms[jj];
-    for(kk = 0; kk < output_dimensions[jj]; kk++) {
+    for(kk = 0; kk < odims[jj]; kk++) {
       double cc = (double)kk;
       if (shifts)
         cc += shift;
@@ -1008,16 +988,11 @@ int NI_ZoomShift(PyArrayObject *input, double* zooms, double* shifts,
   for(jj = 0; jj < irank; jj++)
     filter_size *= order + 1;
 
-  /* allocate output: */
-  if (!NI_OutputArray(output_type, output_rank, output_dimensions, 
-                      output_in, output))
-    goto exit;
-
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
+  po = NI_GetArrayData(output);
     
   /* store all coordinates and offsets with filter: */
   fcoordinates = (int*)malloc(irank * filter_size * sizeof(int));
@@ -1046,7 +1021,7 @@ int NI_ZoomShift(PyArrayObject *input, double* zooms, double* shifts,
     }
   }
 
-  size = NI_Elements(*output);
+  size = NI_GetArraySize(output);
 
   for(kk = 0; kk < size; kk++) {                                         
     double t = 0.0;                                                         
@@ -1132,7 +1107,7 @@ int NI_ZoomShift(PyArrayObject *input, double* zooms, double* shifts,
       t = cval;
     }                
     /* store output: */
-    switch (output_type) {
+    switch (otype) {
     case tBool:
       *(Bool*)po = (Bool)t;
       break;
@@ -1191,7 +1166,7 @@ int NI_ZoomShift(PyArrayObject *input, double* zooms, double* shifts,
   if (splvals) {
     for(jj = 0; jj < irank; jj++) {
       if (splvals[jj]) {
-        for(hh = 0; hh < output_dimensions[jj]; hh++)
+        for(hh = 0; hh < odims[jj]; hh++)
           if (splvals[jj][hh])
             free(splvals[jj][hh]);
         free(splvals[jj]);
@@ -1202,7 +1177,7 @@ int NI_ZoomShift(PyArrayObject *input, double* zooms, double* shifts,
   if (edge_offsets) {
     for(jj = 0; jj < irank; jj++) {
       if (edge_offsets[jj]) {
-        for(hh = 0; hh < output_dimensions[jj]; hh++)
+        for(hh = 0; hh < odims[jj]; hh++)
           if (edge_offsets[jj][hh])
             free(edge_offsets[jj][hh]);
         free(edge_offsets[jj]);
