@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Peter J. Verveer
+/* Copyright (C) 2003-2005 Peter J. Verveer
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -42,12 +42,12 @@ typedef struct {
 } _index_pair;
 
 int NI_Label(PyArrayObject* input, PyArrayObject* strct, int *max_label,
-             PyArrayObject** output, PyObject* output_in)
+             PyArrayObject* output)
 {
-  int kk, jj, ssize, size, filter_size, shifts[NI_MAXDIM], *offsets = NULL;
+  int kk, jj, ssize, size, filter_size, shifts[NI_MAXDIM];
+  int *offsets = NULL;
   int mask_value, *oo, irank, itype, idims[NI_MAXDIM], sdims[NI_MAXDIM];
-  PyArrayObject *tmp = NULL;
-  Bool *ps;
+  Bool *ps, *footprint = NULL;
   char *pi, *po;
   Int32 index = 0, *index_map = NULL;
   NI_FilterIterator fi;
@@ -77,6 +77,14 @@ int NI_Label(PyArrayObject* input, PyArrayObject* strct, int *max_label,
                     "structure rank must be equal to input rank");
     goto exit;
   }
+
+  /* the footprint array must be contigous: */
+  if (!PyArray_ISCONTIGUOUS(strct)) {
+    PyErr_SetString(PyExc_RuntimeError,
+                    "structure array must be contiguous");
+    goto exit;
+  }
+
   ssize = 1;
   NI_GetArrayDimensions(strct, sdims);
   for(kk = 0; kk < irank; kk++) {
@@ -90,32 +98,33 @@ int NI_Label(PyArrayObject* input, PyArrayObject* strct, int *max_label,
 
   /* we only use the first half of the structure data, so we make a 
      temporary structure for use with the filter functions: */
-  tmp = NI_NewArray(tBool, irank, sdims);
-  if (!tmp) {
+  footprint = (Bool*)malloc(ssize * sizeof(Bool));
+  if (!footprint) {
     PyErr_NoMemory();
     goto exit;
   }
-  NI_CopyArray(tmp, strct);
-  ps = (Bool*)NI_GetArrayData(tmp);
+  ps = (Bool*)NI_GetArrayData(strct);
   filter_size = 0;
-  for(kk = 0; kk < ssize / 2; kk++)
+  for(kk = 0; kk < ssize / 2; kk++) {
+    footprint[kk] = ps[kk];
     if (ps[kk])
       ++filter_size;
+  }
   for(kk = ssize / 2; kk < ssize; kk++)
-    ps[kk] = 0;
+    footprint[kk] = 0;
   
   /* get output array */
   NI_GetArrayDimensions(input, idims);
-  if (!NI_OutputArray(tInt32, irank, idims, output_in, output))
+  if (!NI_CheckArray(output, tInt32, irank, idims))
     goto exit;
 
   /* get data and size */
   pi = NI_GetArrayData(input);
-  po = NI_GetArrayData(*output);
-  size = NI_Elements(*output);
+  po = NI_GetArrayData(output);
+  size = NI_GetArraySize(output);
   if (!NI_InitPointIterator(input, &ii))
     goto exit;
-  if (!NI_InitPointIterator(*output, &io))
+  if (!NI_InitPointIterator(output, &io))
     goto exit;
 
   /* set all elements in the output corresponding to non-zero elements
@@ -169,9 +178,8 @@ int NI_Label(PyArrayObject* input, PyArrayObject* strct, int *max_label,
   for(kk = 0; kk < irank; kk++)
     shifts[kk] = 0;
   /* calculate the filter offsets: */
-  if (!NI_InitFilterOffsetsFromArray(*output, tmp, shifts,
-                                     NI_EXTEND_CONSTANT, &offsets,
-                                     &mask_value, NULL))
+  if (!NI_InitFilterOffsets(output, footprint, sdims, shifts,
+                        NI_EXTEND_CONSTANT, &offsets, &mask_value, NULL))
     goto exit;
   /* initialize filter iterator: */
   if (!NI_InitFilterIterator(irank, sdims, filter_size,  idims, shifts,
@@ -180,7 +188,7 @@ int NI_Label(PyArrayObject* input, PyArrayObject* strct, int *max_label,
   
   /* reset output iterator: */
   NI_ITERATOR_RESET(io);
-  po = NI_GetArrayData(*output);
+  po = NI_GetArrayData(output);
 
   /* iterator over the elements: */
   oo = offsets;
@@ -287,7 +295,7 @@ int NI_Label(PyArrayObject* input, PyArrayObject* strct, int *max_label,
   if (index_map) {
     *max_label = 0;
     NI_ITERATOR_RESET(io);
-    po = NI_GetArrayData(*output);
+    po = NI_GetArrayData(output);
     for(jj = 0; jj < size; jj++) {
       Int32 p = *(Int32*)po;
       if (p > 0 )
@@ -306,41 +314,40 @@ int NI_Label(PyArrayObject* input, PyArrayObject* strct, int *max_label,
     pairs = (_index_pair*)pairs->next;
     free(tp);
   }
-  Py_XDECREF(tmp);
+  if (footprint)
+    free(footprint);
   return PyErr_Occurred() ? 0 : 1;
 }
 
-#define NI_FIND_OBJECT_POINT(_pi, _regions, _rank,  _dimensions, \
-                             _max_label, _ii, _type)             \
-{                                                                \
-  int _kk, _sindex = *(_type*)_pi - 1;                           \
-  if (_sindex >= 0 && _sindex < _max_label) {                    \
-    if (!_regions[_sindex]) {                                    \
-      /* this is a new object: */                                \
-      _regions[_sindex] =                                        \
-        (NI_ObjectRegion*)malloc(sizeof(NI_ObjectRegion));       \
-      if (!_regions[_sindex]) {                                  \
-        PyErr_NoMemory();                                        \
-        goto exit;                                               \
-      }                                                          \
-      for(_kk = 0; _kk < _rank; _kk++) {                         \
-        _regions[_sindex]->start[_kk] = _dimensions[_kk];        \
-        _regions[_sindex]->end[_kk] = 0;                         \
-      }                                                          \
-    }                                                            \
-    /* adapt the object region size and position: */             \
-    for(_kk = 0; _kk < _rank; _kk++) {                           \
-      int _cc = _ii.coordinates[_kk];                            \
-      if (_cc < _regions[_sindex]->start[_kk])                   \
-        _regions[_sindex]->start[_kk] = _cc;                     \
-      if (_cc + 1 > _regions[_sindex]->end[_kk])                 \
-        _regions[_sindex]->end[_kk] = _cc + 1;                   \
-    }                                                            \
-  }                                                              \
+#define NI_FIND_OBJECT_POINT(_pi, _regions, _rank, _dimensions, \
+                             _max_label, _ii, _type)            \
+{                                                               \
+  int _kk, _sindex = *(_type*)_pi - 1;                          \
+  if (_sindex >= 0 && _sindex < _max_label) {                   \
+    if (_rank > 0) {                                            \
+      _sindex *= 2 * _rank;                                     \
+      if (_regions[_sindex] < 0) {                              \
+        for(_kk = 0; _kk < _rank; _kk++) {                      \
+          int _cc = _ii.coordinates[_kk];                       \
+          _regions[_sindex + _kk] = _cc;                        \
+          _regions[_sindex + _kk + _rank] = _cc + 1;            \
+        }                                                       \
+      } else {                                                  \
+        for(_kk = 0; _kk < _rank; _kk++) {                      \
+          int _cc = _ii.coordinates[_kk];                       \
+          if (_cc < _regions[_sindex + _kk])                    \
+            _regions[_sindex + _kk] = _cc;                      \
+          if (_cc + 1 > _regions[_sindex + _kk + _rank])        \
+            _regions[_sindex + _kk + _rank] = _cc + 1;          \
+        }                                                       \
+      }                                                         \
+    } else {                                                    \
+      _regions[_sindex] = 1;                                    \
+    }                                                           \
+  }                                                             \
 }
 
-int NI_FindObjects(PyArrayObject* input, int max_label, 
-                   NI_ObjectRegion** regions)
+int NI_FindObjects(PyArrayObject* input, int max_label, int* regions)
 {
   int size, jj, irank, itype, idims[NI_MAXDIM];
   NI_Iterator ii;
@@ -351,13 +358,21 @@ int NI_FindObjects(PyArrayObject* input, int max_label,
 
   /* get input data, size and iterator: */
   pi = NI_GetArrayData(input);
-  size = NI_Elements(input);
+  size = NI_GetArraySize(input);
   if (!NI_InitPointIterator(input, &ii))
     goto exit;
 
   irank = NI_GetArrayRank(input);
   itype = NI_GetArrayType(input);
   NI_GetArrayDimensions(input, idims);
+  
+  if (irank > 0) {
+    for(jj = 0; jj < 2 * irank * max_label; jj++)
+      regions[jj] = -1;
+  } else {
+    for(jj = 0; jj < max_label; jj++)
+      regions[jj] = -1;
+  }
 
   /* iterate over all points: */
   for(jj = 0 ; jj < size; jj++) {
@@ -408,16 +423,7 @@ int NI_FindObjects(PyArrayObject* input, int max_label,
   }
 
  exit:
-  if (PyErr_Occurred()) {
-    for(jj = 0; jj < max_label; jj++)
-      if (regions[jj]) {
-        free(regions[jj]);
-        regions[jj] = NULL;
-      }
-    return 0;
-  } else {
-    return 1;
-  }
+  return PyErr_Occurred() ? 0 : 1;
 }
 
 
@@ -627,7 +633,7 @@ int NI_Statistics(PyArrayObject *input, PyArrayObject *labels,
   }
 
   /* input size: */
-  size = NI_Elements(input);
+  size = NI_GetArraySize(input);
   
   for(jj = 0; jj < n_results; jj++) {
     if (sum)
@@ -780,7 +786,7 @@ int NI_CenterOfMass(PyArrayObject *input, PyArrayObject *labels,
   }
 
   /* input size: */
-  size = NI_Elements(input);
+  size = NI_GetArraySize(input);
 
   sum = (double*)malloc(n_results * sizeof(double));
   if (!sum) {
@@ -884,11 +890,6 @@ int NI_Histogram(PyArrayObject *input, PyArrayObject *labels,
     goto exit;
   }
   for(jj = 0; jj < n_results; jj++) {
-    histograms[jj] = NI_NewArray(tInt32, 1, &nbins);
-    if (!histograms[jj]) {
-      PyErr_NoMemory();
-      goto exit;
-    }
     ph[jj] = (Int32*)NI_GetArrayData(histograms[jj]);
     for(kk = 0; kk < nbins; kk++)
       ph[jj][kk] = 0;
@@ -897,7 +898,7 @@ int NI_Histogram(PyArrayObject *input, PyArrayObject *labels,
   bsize = (max - min) / (double)nbins;
 
   /* input size: */
-  size = NI_Elements(input);
+  size = NI_GetArraySize(input);
   /* iterate over array: */
   for(jj = 0; jj < size; jj++) {
     NI_GET_LABEL(pm, label, mtype);
@@ -933,6 +934,23 @@ int NI_Histogram(PyArrayObject *input, PyArrayObject *labels,
   return  PyErr_Occurred() == NULL;
 }
 
+#define WS_GET_INDEX(_index, _c_strides, _b_strides, _rank, _out, \
+                     _contiguous, _type)                          \
+do {                                                              \
+  if (_contiguous) {                                              \
+    _out = _index * sizeof(_type);                                \
+  } else {                                                        \
+    int _qq, _cc, _idx = _index;                                  \
+    _out = 0;                                                     \
+    for (_qq = 0; _qq < _rank; _qq++) {                           \
+      _cc = _idx / _c_strides[_qq];                               \
+      _idx -= _cc * _c_strides[_qq];                              \
+      _out += _b_strides[_qq] * _cc;                              \
+    }                                                             \
+  }                                                               \
+} while(0)
+
+
 #define DONE_TYPE UInt8
 #define COST_TYPE UInt16
 #define WS_MAXDIM 7
@@ -945,21 +963,23 @@ typedef struct {
 } NI_WatershedElement;
 
 int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers, 
-                    PyArrayObject* strct, PyArrayObject** output, 
-                    PyObject* output_in)
+                    PyArrayObject* strct, PyArrayObject* output)
 {
-  char *pv, *pl;
-  int irank, itype, mtype, idims[NI_MAXDIM], sdims[NI_MAXDIM], ssize;
+  char *pl, *pm, *pi;
+  int irank, itype, mtype, otype, idims[NI_MAXDIM], sdims[NI_MAXDIM];
   int size, jj, hh, kk, maxval, strides[WS_MAXDIM], coordinates[WS_MAXDIM];
-  int *nstrides = NULL, nneigh;
+  int *nstrides = NULL, nneigh, ssize, i_contiguous, o_contiguous;
+  int istrides[WS_MAXDIM], ostrides[WS_MAXDIM];
   NI_WatershedElement *temp = NULL, **first = NULL, **last = NULL;
-  PyArrayObject *values = NULL;
   Bool *ps = NULL;
+  NI_Iterator mi, ii, li;
   
   /* get input rank, type and shape */
   irank = NI_GetArrayRank(input);
   itype = NI_GetArrayType(input);
   NI_GetArrayDimensions(input, idims);
+  NI_GetArrayStrides(input, istrides);
+  i_contiguous = PyArray_ISCONTIGUOUS(input);
 
   /* structuring element must be of bool type: */
   if (NI_GetArrayType(strct) != tBool) {
@@ -1010,13 +1030,27 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
     goto exit;
   }
 
+  /* check output */
+  if (!NI_CheckArray(output, tAny, irank, idims))
+    goto exit;
+  NI_GetArrayStrides(output, ostrides);
+  o_contiguous = PyArray_ISCONTIGUOUS(output);
+
+  otype = NI_GetArrayType(output);
+  if (!(otype == tInt8 || otype == tInt16 || otype == tInt32 ||
+        otype == tUInt8 || otype == tUInt16 || otype == tUInt32)) {
+    PyErr_SetString(PyExc_RuntimeError, 
+                    "output array must be of an integer type");
+    goto exit;
+  }
+
   if (!NI_ShapeEqual(input, markers)) {
     PyErr_SetString(PyExc_RuntimeError,  
                     "input and markers must have equal shape");
     goto exit;
   }
   
-  size = NI_Elements(input);
+  size = NI_GetArraySize(input);
   /* Storage for the temporary queue data. */
   temp = (NI_WatershedElement*)malloc(size * sizeof(NI_WatershedElement));
   if (!temp) {
@@ -1024,19 +1058,9 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
     goto exit;
   }
 
-  /* Make sure that the input is contiguous in memory. */
-  values = NI_ContiguousArray(input);
-  if (!values) {
-    PyErr_SetString(PyExc_RuntimeError, "cannot convert input");
+  pi = NI_GetArrayData(input);
+  if (!NI_InitPointIterator(input, &ii))
     goto exit;
-  }
-
-  /* allocate output */
-  if (!NI_OutputArray(mtype, irank, idims, output_in, output))
-    goto exit;
-  NI_CopyArray(*output, markers);
-
-  pv = NI_GetArrayData(values);
 
   /* Initialization and find the maximum of the input. */
   maxval = 0;
@@ -1044,10 +1068,10 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
     int ival = 0;
     switch(itype) {
     case tUInt8:
-      ival = ((UInt8*)pv)[jj];
+      ival = *((UInt8*)pi);
       break;
     case tUInt16:
-      ival = ((UInt16*)pv)[jj];
+      ival = *((UInt16*)pi);
       break;
     default:
       PyErr_SetString(PyExc_RuntimeError, "data type not supported");
@@ -1057,7 +1081,9 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
     temp[jj].done = 0;
     if (ival > maxval)
       maxval = ival;
+    NI_ITERATOR_NEXT(ii, pi);
   }
+  pi = NI_GetArrayData(input);
 
   /* Allocate and initialize the storage for the queue. */
   first = (NI_WatershedElement**)malloc((maxval + 1) * 
@@ -1072,8 +1098,13 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
     first[hh] = NULL;
     last[hh] = NULL;
   }
-
-  pl = NI_GetArrayData(*output);
+  
+  if (!NI_InitPointIterator(markers, &mi))
+    goto exit;
+  if (!NI_InitPointIterator(output, &li))
+    goto exit;
+  pm = NI_GetArrayData(markers);
+  pl = NI_GetArrayData(output);
 
   /* initialize all nodes */
   for(jj = 0; jj < irank; jj++)
@@ -1083,27 +1114,51 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
     int label = 0;
     switch(mtype) {
     case tInt8:
-      label = ((Int8*)pl)[jj];
+      label = *(Int8*)pm;
       break;
     case tInt16:
-      label = ((Int16*)pl)[jj];
+      label = *(Int16*)pm;
       break;
     case tInt32:
-      label = ((Int32*)pl)[jj];
+      label = *(Int32*)pm;
       break;
     case tUInt8:
-      label = ((UInt8*)pl)[jj];
+      label = *(UInt8*)pm;
       break;
     case tUInt16:
-      label = ((UInt16*)pl)[jj];
+      label = *(UInt16*)pm;
       break;
     case tUInt32:
-      label = ((UInt32*)pl)[jj];
+      label = *(UInt32*)pm;
       break;
     default:
       PyErr_SetString(PyExc_RuntimeError, "data type not supported");
       goto exit;
     }
+    switch(otype) {
+    case tInt8:
+      *((Int8*)pl) = label;
+      break;
+    case tInt16:
+      *((Int16*)pl) = label;
+      break;
+    case tInt32:
+      *((Int32*)pl) = label;
+      break;
+    case tUInt8:
+      *((UInt8*)pl) = label;
+      break;
+    case tUInt16:
+      *((UInt16*)pl) = label;
+      break;
+    case tUInt32:
+      *((UInt32*)pl) = label;
+      break;
+    default:
+      PyErr_SetString(PyExc_RuntimeError, "data type not supported");
+      goto exit;
+    }
+    NI_ITERATOR_NEXT2(mi, li, pm, pl);
 
     if (label != 0) {
       /* This node is a marker */
@@ -1145,6 +1200,8 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
         coordinates[hh] = 0;                        
       }        
   }
+  
+  pl = NI_GetArrayData(output);
 
   ps = (Bool*)NI_GetArrayData(strct);
   nneigh = 0;
@@ -1217,15 +1274,23 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
           NI_WatershedElement *p = &(temp[p_index]);
           if (!(p->done)) {
             /* If the neighbor was not processed yet: */
-            int max, pval, vval, wvp, pcost, label;
+            int max, pval, vval, wvp, pcost, label, p_idx, v_idx;
             switch(itype) {
             case tUInt8:
-              vval = ((UInt8*)pv)[v_index];
-              pval = ((UInt8*)pv)[p_index];
+              WS_GET_INDEX(v_index, strides, istrides, irank, p_idx,
+                           i_contiguous, UInt8);
+              WS_GET_INDEX(p_index, strides, istrides, irank, v_idx,
+                           i_contiguous, UInt8);
+              vval = *(UInt8*)(pi + v_idx);
+              pval = *(UInt8*)(pi + p_idx);
               break;
             case tUInt16:
-              vval = ((UInt16*)pv)[v_index];
-              pval = ((UInt16*)pv)[p_index];
+              WS_GET_INDEX(v_index, strides, istrides, irank, p_idx,
+                           i_contiguous, UInt16);
+              WS_GET_INDEX(p_index, strides, istrides, irank, v_idx,
+                           i_contiguous, UInt16);
+              vval = *(UInt16*)(pi + v_idx);
+              pval = *(UInt16*)(pi + p_idx);
               break;
             default:
               PyErr_SetString(PyExc_RuntimeError,
@@ -1243,31 +1308,74 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
             if (max < pcost) {
               /* If this maximum is less than the neighbors cost,
                  adapt the cost and the label of the neighbor: */
+              int idx;
               p->cost = max;
-              switch(mtype) {
+              switch(otype) {
               case tInt8:
-                label = ((Int8*)pl)[v_index];
-                ((Int8*)pl)[p_index] = label;
+                WS_GET_INDEX(v_index, strides, ostrides, irank, idx,
+                             o_contiguous, Int8);
+                label = *(Int8*)(pl + idx);
                 break;
               case tInt16:
-                label = ((Int16*)pl)[v_index];
-                ((Int16*)pl)[p_index] = label;
+                WS_GET_INDEX(v_index, strides, ostrides, irank, idx,
+                             o_contiguous, Int16);
+                label = *(Int16*)(pl + idx);
                 break;
               case tInt32:
-                label = ((Int32*)pl)[v_index];
-                ((Int32*)pl)[p_index] = label;
+                WS_GET_INDEX(v_index, strides, ostrides, irank, idx,
+                             o_contiguous, Int32);
+                label = *(Int32*)(pl + idx);
                 break;
               case tUInt8:
-                label = ((UInt8*)pl)[v_index];
-                ((UInt8*)pl)[p_index] = label;
+                WS_GET_INDEX(v_index, strides, ostrides, irank, idx,
+                             o_contiguous, UInt8);
+                label = *(UInt8*)(pl + idx);
                 break;
               case tUInt16:
-                label = ((UInt16*)pl)[v_index];
-                ((UInt16*)pl)[p_index] = label;
+                WS_GET_INDEX(v_index, strides, ostrides, irank, idx,
+                             o_contiguous, UInt16);
+                label = *(UInt16*)(pl + idx);
                 break;
               case tUInt32:
-                label = ((UInt32*)pl)[v_index];
-                ((UInt32*)pl)[p_index] = label;
+                WS_GET_INDEX(v_index, strides, ostrides, irank, idx,
+                             o_contiguous, UInt32);
+                label = *(UInt32*)(pl + idx);
+                break;
+              default:
+                PyErr_SetString(PyExc_RuntimeError,
+                                "data type not supported");
+                goto exit;
+              }
+              switch(otype) {
+              case tInt8:
+                WS_GET_INDEX(p_index, strides, ostrides, irank, idx,
+                             o_contiguous, Int8);
+                *(Int8*)(pl + idx) = label;
+                break;
+              case tInt16:
+                WS_GET_INDEX(p_index, strides, ostrides, irank, idx,
+                             o_contiguous, Int16);
+                *(Int16*)(pl + idx) = label;
+                break;
+              case tInt32:
+                WS_GET_INDEX(p_index, strides, ostrides, irank, idx,
+                             o_contiguous, Int32);
+                *(Int32*)(pl + idx) = label;
+                break;
+              case tUInt8:
+                WS_GET_INDEX(p_index, strides, ostrides, irank, idx,
+                             o_contiguous, UInt8);
+                *(UInt8*)(pl + idx) = label;
+                break;
+              case tUInt16:
+                WS_GET_INDEX(p_index, strides, ostrides, irank, idx,
+                             o_contiguous, UInt16);
+                *(UInt16*)(pl + idx) = label;
+                break;
+              case tUInt32:
+                WS_GET_INDEX(p_index, strides, ostrides, irank, idx,
+                             o_contiguous, UInt32);
+                *(UInt32*)(pl + idx) = label;
                 break;
               default:
                 PyErr_SetString(PyExc_RuntimeError,
@@ -1320,6 +1428,5 @@ int NI_WatershedIFT(PyArrayObject* input, PyArrayObject* markers,
     free(last);
   if (nstrides)
     free(nstrides);
-  Py_XDECREF(values);
   return PyErr_Occurred() ? 0 : 1;
 }
