@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Peter J. Verveer
+/* Copyright (C) 2003-2005 Peter J. Verveer
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -33,16 +33,13 @@
 #define NI_MORPHOLOGY_H
 
 int NI_BinaryErosion(PyArrayObject*, PyArrayObject*, PyArrayObject*, 
-                     PyArrayObject**, PyObject*, int, int*, int, int, 
-                     int*, NI_CoordinateList**);
+           PyArrayObject*, int, int*, int, int, int*, NI_CoordinateList**);
 int NI_BinaryErosion2(PyArrayObject*, PyArrayObject*, PyArrayObject*,
                       int, int*, int, NI_CoordinateList**);
-int NI_DistanceTransformBruteForce(PyArrayObject*, int, double*, int, int,
-                                   PyArrayObject**, PyArrayObject**, 
-                                   PyObject*, PyObject*);
+int NI_DistanceTransformBruteForce(PyArrayObject*, int, double*,
+                                   PyArrayObject*, PyArrayObject*);
 int NI_DistanceTransformOnePass(PyArrayObject*, PyArrayObject *,
                                 PyArrayObject*);
-int NI_EuclideanFeatureTransform(PyArrayObject*, double*, PyArrayObject**, 
-                                 PyObject*);
+int NI_EuclideanFeatureTransform(PyArrayObject*, double*, PyArrayObject*);
 
 #endif
