@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Peter J. Verveer
+/* Copyright (C) 2003-2005 Peter J. Verveer
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -108,7 +108,8 @@ int NI_InitSubSpaceIterator(PyArrayObject *array, NI_Iterator *iterator,
 }
 
 /* initialize iteration over array lines: */
-int NI_InitLineIterator(PyArrayObject *array, NI_Iterator *iterator, int axis)
+int NI_InitLineIterator(PyArrayObject *array, NI_Iterator *iterator,
+                        int axis)
 {
   int arank, ii;
   UInt32 axes = 0;
@@ -141,8 +142,7 @@ int NI_InitLineIterator(PyArrayObject *array, NI_Iterator *iterator, int axis)
 
 /* Allocate line buffer data */
 int NI_AllocateLineBuffer(PyArrayObject* array, int axis, int size1, 
-                          int size2, int *lines, int max_size, 
-                          double **buffer)
+                     int size2, int *lines, int max_size, double **buffer)
 {
   int line_size, max_lines, arank, atype, adims[NI_MAXDIM];
 
@@ -168,7 +168,7 @@ int NI_AllocateLineBuffer(PyArrayObject* array, int axis, int size1,
 
   /* the number of lines of the array is an upper limit for the
      number of lines in the buffer: */
-  max_lines = NI_Elements(array);
+  max_lines = NI_GetArraySize(array);
   if (arank > 0 && adims[axis] > 0)
     max_lines /= adims[axis];
   
@@ -199,9 +199,8 @@ int NI_AllocateLineBuffer(PyArrayObject* array, int axis, int size1,
 
 /* Initialize a line buffer */
 int NI_InitLineBuffer(PyArrayObject *array, int axis, int size1, 
-                      int size2, int buffer_lines, 
-                      double *buffer_data, NI_ExtendMode extend_mode, 
-                      double extend_value, NI_LineBuffer *buffer)
+    int size2, int buffer_lines, double *buffer_data,
+    NI_ExtendMode extend_mode, double extend_value, NI_LineBuffer *buffer)
 {
   int line_length = 0, array_lines = 0, size, arank, atype;
   int adims[NI_MAXDIM], astrides[NI_MAXDIM];
@@ -227,7 +226,7 @@ int NI_InitLineBuffer(PyArrayObject *array, int axis, int size1,
     return 0;
   }
 
-  size = NI_Elements(array);
+  size = NI_GetArraySize(array);
   /* check if the buffer is big enough: */
   if (size > 0 && buffer_lines < 1) {
     PyErr_SetString(PyExc_RuntimeError, "buffer too small");
@@ -476,7 +475,7 @@ int NI_ArrayToLineBuffer(NI_LineBuffer *buffer, int *number_of_lines,
     pb += buffer->line_length + buffer->size1 + buffer->size2;
   }
 
-  /* if not all array lines were processed until now, *more is set true: */
+  /* if not all array lines were processed, *more is set true: */
   *more = buffer->next_line < buffer->array_lines;
 
   return 1;
@@ -623,35 +622,14 @@ int NI_InitFilterIterator(int rank, int *filter_shape, int size,
 
 /* Calculate the offsets to the filter points, for all border regions and
    the interior of the array: */
-int NI_InitFilterOffsetsFromArray(PyArrayObject *array, 
-                                  PyArrayObject *footprint, int* shifts, 
-                                  NI_ExtendMode extend_mode, int **offsets, 
-                                  int *border_flag_value,
-                                  int **coordinate_offsets)
+int NI_InitFilterOffsets(PyArrayObject *array,
+        Bool *pm, int *fdims, int* shifts, NI_ExtendMode extend_mode,
+        int **offsets, int *border_flag_value, int **coordinate_offsets)
 {
-  int dimensions[NI_MAXDIM], strides[NI_MAXDIM], arank;
-
-  assert(array != NULL);
-
-  arank = NI_GetArrayRank(array);
-  NI_GetArrayDimensions(array, dimensions);
-  NI_GetArrayStrides(array, strides);
-  return NI_InitFilterOffsets(arank, dimensions, strides, footprint,
-                              shifts, extend_mode, offsets,
-                              border_flag_value, coordinate_offsets);
-}
-
-/* Calculate the offsets to the filter points, for all border regions and
-   the interior of the array: */
-int NI_InitFilterOffsets(int rank, int* dimensions, int* strides,
-                         PyArrayObject *footprint, int* shifts, 
-                         NI_ExtendMode extend_mode, int **offsets, 
-                         int *border_flag_value, int **coordinate_offsets)
-{
-  int filter_size, footprint_size = 0, offsets_size = 1, ii, jj, kk;
+  int filter_size = 1, footprint_size = 0, offsets_size = 1, ii, jj, kk;
   int shape[NI_MAXDIM], coordinates[NI_MAXDIM], position[NI_MAXDIM];
-  int max_size = 0, max_stride = 0, fdims[NI_MAXDIM], *po, *pc = NULL;
-  Bool *pm;
+  int max_size = 0, max_stride = 0, *po, *pc = NULL;
+  int rank, dimensions[NI_MAXDIM], strides[NI_MAXDIM];
   
   assert(dimensions != NULL);
   assert(strides != NULL);
@@ -659,24 +637,11 @@ int NI_InitFilterOffsets(int rank, int* dimensions, int* strides,
   assert(offsets != NULL && *offsets == NULL);
   assert(border_flag_value != NULL);
 
-  /* array and footprint must have the same rank: */
-  if (rank != NI_GetArrayRank(footprint)) {
-    PyErr_SetString(PyExc_RuntimeError, 
-                    "Filter and footprint rank must be equal");
-    goto exit;
-  }
-  /* footprint must be of Bool type: */
-  if (NI_GetArrayType(footprint) != tBool) {
-    PyErr_SetString(PyExc_RuntimeError, "footprint type must be Bool");
-    goto exit;
-  }
-  /* footprint data must be contiguous: */
-  if (!PyArray_ISCONTIGUOUS(footprint)) {
-    PyErr_SetString(PyExc_RuntimeError, "footprint must be contiguous");
-    goto exit;
-  }
+  rank = NI_GetArrayRank(array);
+  NI_GetArrayDimensions(array, dimensions);
+  NI_GetArrayStrides(array, strides);
+
   /* check footprint shape: */
-  NI_GetArrayDimensions(footprint, fdims);
   for(ii = 0; ii < rank; ii++) {
     shape[ii] = fdims[ii];
     if (shape[ii] < 0) {
@@ -686,10 +651,10 @@ int NI_InitFilterOffsets(int rank, int* dimensions, int* strides,
     }
   }
   /* the size of the footprint array: */
-  filter_size = NI_Elements(footprint);
+  for(ii = 0; ii < rank; ii++)
+    filter_size *= fdims[ii];
 
   /* calculate the number of non-zero elements in the footprint: */
-  pm = (Bool*)NI_GetArrayData(footprint);
   for(ii = 0; ii < filter_size; ii++)
     if (pm[ii])
       ++footprint_size;
@@ -898,6 +863,7 @@ int NI_InitFilterOffsets(int rank, int* dimensions, int* strides,
   }
 }
 
+
 NI_CoordinateList* NI_InitCoordinateList(int size, int rank)
 {
   NI_CoordinateList *list = \
