@@ -1,4 +1,4 @@
-/* Copyright (C) 2003, 2004 Peter J. Verveer
+/* Copyright (C) 2003-2005 Peter J. Verveer
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -164,12 +164,13 @@ typedef struct {
 } NI_LineBuffer;
 
 /* Get the next line being processed: */
-#define NI_GET_LINE(_buffer, _line)                                       \
-  ((_buffer).buffer_data + (_line) * ((_buffer).line_length +             \
+#define NI_GET_LINE(_buffer, _line)                                      \
+  ((_buffer).buffer_data + (_line) * ((_buffer).line_length +            \
                                       (_buffer).size1 + (_buffer).size2))
 
 /* Allocate line buffer data */
-int NI_AllocateLineBuffer(PyArrayObject*, int, int, int, int*, int, double**);
+int NI_AllocateLineBuffer(PyArrayObject*, int, int, int, int*, int,
+                          double**);
 
 /* Initialize a line buffer */
 int NI_InitLineBuffer(PyArrayObject*, int, int, int,  int, double*, 
@@ -294,10 +295,8 @@ int NI_InitFilterIterator(int, int*, int, int*, int*, NI_FilterIterator*);
 
 /* Calculate the offsets to the filter points, for all border regions and
    the interior of the array: */
-int NI_InitFilterOffsets(int, int*, int*, PyArrayObject*, int*, 
+int NI_InitFilterOffsets(PyArrayObject*, Bool*, int*, int*,
                          NI_ExtendMode, int**, int*, int**);
-int NI_InitFilterOffsetsFromArray(PyArrayObject*, PyArrayObject*, int*, 
-                                  NI_ExtendMode, int**, int*, int**);
 
 typedef struct {
     int *coordinates, size;
