@@ -1,10 +1,11 @@
-/* $Id: _chararraymodule.c,v 1.13.2.1 2004/11/16 23:02:47 jaytmiller Exp $ */
+/* $Id: _chararraymodule.c,v 1.21 2005/02/18 15:58:56 jaytmiller Exp $ */
 
 #include <Python.h>
 
 #include <stdio.h>
 #include <math.h>
 #include <signal.h>
+#include <ctype.h>
 
 #include <libnumarray.h>
 
@@ -14,7 +15,7 @@ static PyObject *_Error;
 #if !defined(strnlen)
 size_t strnlen(const char *s, size_t maxlen)
 {
-	int i;
+	size_t i;
 	for(i=0; i<maxlen; i++)
 		if (s[i] == 0)
 			return i;
@@ -82,11 +83,11 @@ static void rstripw(char *s, int n)
 
 /* padw pads string 's' with blanks, for a total length of 'n' characters. */
 static void 
-padw(char *s, int n)
+padw(char *s, int n, char padc)
 {
 	int i;
 	for(i=strnlen(s, n); i<n; i++)
-		s[i] = ' ';
+		s[i] = padc;
 }
 
 /* Strip executes rstripw on a python string. */
@@ -116,14 +117,15 @@ Pad(PyObject *obj, PyObject *args)
 {
 	char *s, *t;
 	int  slen, rsize;
+	char padc;
 
-	if (!PyArg_ParseTuple(args, "s#i:Pad", &s, &slen, &rsize))
+	if (!PyArg_ParseTuple(args, "s#ib:Pad", &s, &slen, &rsize, &padc))
 		return NULL;
 	rsize = max(slen, rsize);
 	if ((t = mystrdup(s, rsize)))
 	{
-		padw(t, rsize);
-		obj = Py_BuildValue("s#", t, strnlen(t, rsize));
+		padw(t, rsize, padc);
+		obj = Py_BuildValue("s#", t, rsize);
 		PyMem_Free(t);
 		return obj;
 	}
@@ -134,7 +136,7 @@ Pad(PyObject *obj, PyObject *args)
 static int 
 Concat(PyObject *aux, long nnumarray, PyArrayObject *numarray[])
 {
-	int rused, bused;
+	size_t rused, bused;
 	PyArrayObject *ai = numarray[0];
 	PyArrayObject *bi = numarray[1];
 	PyArrayObject *ri = numarray[2];
@@ -213,63 +215,103 @@ PadAll( PyObject *aux, int nnumarray, PyArrayObject *numarray[])
 	char   *a  = (char *) ai->data;
 
 	if (nnumarray != 1) {
-		PyErr_Format(_Error, "PadAll: invalid parameters.");
+		PyErr_Format(PyExc_ValueError, 
+			     "PadAll: invalid parameters.");
 		return -1;
 	}
 	if (!PyArray_ISWRITABLE(ai)) {
-		PyErr_Format(_Error, "PadAll: result array not writeable.");
+		PyErr_Format(PyExc_ValueError, 
+			     "PadAll: result array not writeable.");
 		return -1;
 	}
-	padw(a, ai->itemsize);
+	if (!PyString_Check(aux) || PyString_Size(aux) != 1) {
+		PyErr_Format(PyExc_ValueError, 
+			     "aux parameter must be a len-1-padding-string");
+		return -1;
+	}	
+	padw(a, ai->itemsize, *PyString_AsString(aux)); 
 	return 0;
 }
 
 NSTRIDING_DESCR1(PadAll);
 
-/* MaxLen computes the maximum length of all stripped elements in a CharArray.
-This information can be used to reduce the itemsize of the array.
-*/
-static PyObject *
-MaxLen(PyObject *obj, PyObject *args)
+static int 
+ToUpper( PyObject *aux, int nnumarray, PyArrayObject *numarray[])
 {
-	int        i, len=0;
-	PyArrayObject *ao;
-	char       *a;
-  
-	if (!PyArg_ParseTuple(args, "O:MaxLen", &ao))
-		return NULL;
+	PyArrayObject *ai = numarray[0];
+	char   *a  = (char *) ai->data;
+	long i;
 
-	if (!NA_NDArrayCheck((PyObject *) ao))
-		return PyErr_Format(PyExc_TypeError, 
-				    "MaxLen: object not an NDArray");
-
-	if (!NA_updateDataPtr(ao))
-		return NULL;
-
-	a = (char *) ao->data;
-
-	for(i=0; i<NA_elements(ao); i++, a+=ao->bytestride)
-	{
-		rstripw(a, ao->itemsize);
-		len = max(len, strnlen(a, ao->itemsize));
+	if (nnumarray != 1) {
+		PyErr_Format(_Error, "ToUpper: invalid parameters.");
+		return -1;
 	}
-
-	return Py_BuildValue("i", len);
+	if (!PyArray_ISWRITABLE(ai)) {
+		PyErr_Format(_Error, "ToUpper: result array not writeable.");
+		return -1;
+	}
+	for(i=0; (a[i]!=0) && (i<ai->itemsize); i++)
+		a[i] = toupper(a[i]);
+	return 0;
 }
 
+NSTRIDING_DESCR1(ToUpper);
+
+static int 
+ToLower( PyObject *aux, int nnumarray, PyArrayObject *numarray[])
+{
+	PyArrayObject *ai = numarray[0];
+	char   *a  = (char *) ai->data;
+	long i;
+
+	if (nnumarray != 1) {
+		PyErr_Format(_Error, "ToLower: invalid parameters.");
+		return -1;
+	}
+	if (!PyArray_ISWRITABLE(ai)) {
+		PyErr_Format(_Error, "ToLower: result array not writeable.");
+		return -1;
+	}
+	for(i=0; (a[i]!=0) && (i<ai->itemsize); i++)
+		a[i] = tolower(a[i]);
+	return 0;
+}
+
+NSTRIDING_DESCR1(ToLower);
+
+/* StrLen computes the lengths of all the items in a chararray. */
+static int 
+StrLen(PyObject *args, int nnumarray, PyArrayObject *numarray[])
+{
+	PyArrayObject  *ai = numarray[0];
+	PyArrayObject  *ni = numarray[1];
+	char *a;
+	Int32 *n;
+	if ((nnumarray < 2) || 
+	    !NA_NDArrayCheck((PyObject *)ai) || 
+	    !NA_NumArrayCheck((PyObject *)ni)) {
+		PyErr_Format(PyExc_ValueError, 
+			     "StrLen requires one string and one numerical array");
+		return -1;
+	}
+	a = (char *) ai->data;
+	n = (Int32 *) ni->data;
+	rstripw(a, ai->itemsize);
+	*n = strnlen(a, ai->itemsize);
+	return 0;
+}
+NSTRIDING_DESCR2(StrLen);
+
 static int 
 Eval(PyObject *args, int nnumarray, PyArrayObject *numarray[])
 {
-	PyArrayObject *ai = numarray[0],
-		       *ni = numarray[1];
+	PyArrayObject *ai = numarray[0];
+	PyArrayObject *ni = numarray[1];
 	char *a = (char *) ai->data;
 	Float64 *n = (Float64 *) ni->data;
 	char buffer[64], *ptr;
 	int len;
 
-	if (!PyArg_ParseTuple(args, ":Eval")) 
-		return -1;
-	
 	len = strnlen(a, ai->itemsize);
 	if  (len > ELEM(buffer)-1) {
 		PyErr_Format(PyExc_ValueError, 
@@ -320,7 +362,6 @@ NSTRIDING_DESCR2(Format);
 static PyMethodDef _chararrayMethods[] = {
 	{"Strip",  Strip, METH_VARARGS},
 	{"Pad",    Pad, METH_VARARGS},
-	{"MaxLen",  MaxLen, METH_VARARGS},
 	{NULL,     NULL}        /* Sentinel */
 };
 
@@ -333,15 +374,20 @@ void init_chararray(void) {
 	PyObject *m, *d;
 	m = Py_InitModule("_chararray", _chararrayMethods);
 	d = PyModule_GetDict(m);
-	_Error = PyErr_NewException("_chararray.error", NULL, NULL);
+	_Error = PyErr_NewException("numarray._chararray.error", NULL, NULL);
 	PyDict_SetItemString(d, "error", _Error);
 
+	ADD_VERSION(m);
+
 	import_libnumarray();
 
 	NA_add_cfunc(d, "Concat", &Concat_descr);
 	NA_add_cfunc(d, "StrCmp", &StrCmp_descr);
 	NA_add_cfunc(d, "Format", &Format_descr);
 	NA_add_cfunc(d, "Eval", &Eval_descr);
+	NA_add_cfunc(d, "StrLen", &StrLen_descr);
 	NA_add_cfunc(d, "StripAll", &StripAll_descr);
 	NA_add_cfunc(d, "PadAll", &PadAll_descr);
+	NA_add_cfunc(d, "ToUpper", &ToUpper_descr);
+	NA_add_cfunc(d, "ToLower", &ToLower_descr);
 }
