@@ -12,10 +12,22 @@ char *_converter__doc__ =
 
 staticforward PyTypeObject _converter_type;
 
-static PyObject *_converter_compute(PyObject *self, PyObject *indices, 
-				    PyObject*shape);
-static PyObject *_converter_rebuffer(PyObject *self, PyObject *arr,
-				     PyObject *inbuffer);
+#define REPLACE(x0, y0) {             \
+     PyObject *x = (PyObject *) x0;   \
+     PyObject *y = (PyObject *) y0;   \
+     Py_INCREF(y);                    \
+     x0 = y;                          \
+     Py_DECREF(x);                    \
+}
+
+static PyObject *
+_converter_compute(PyObject *self, PyObject *indices, PyObject*shape);
+
+static PyObject *
+_converter_rebuffer(PyObject *self, PyObject *arr, PyObject *inbuffer);
+
+static void
+_converter_clean(PyObject *me, PyObject *arr);
  
 static PyObject *
 _converter_arr_position_get(PyConverterObject *self)
@@ -186,9 +198,8 @@ _converter_buffers_set(PyConverterObject *self, PyObject *s)
 		return -1;
 	}
 	for(i=0; i<ELEM(self->buffers); i++) {
-		Py_XDECREF(self->buffers[i]);
-		self->buffers[i] = PyList_GET_ITEM(s, i);
-		Py_INCREF(self->buffers[i]);
+		PyObject *new = PyList_GET_ITEM(s, i);
+		REPLACE(self->buffers[i], new);
 	}
 	return 0;
 }
@@ -222,9 +233,8 @@ _converter_bytestrides_set(PyConverterObject *self, PyObject *s)
 		return -1;
 	}
 	for(i=0; i<ELEM(self->bytestrides); i++) {
-		Py_XDECREF(self->bytestrides[i]);
-		self->bytestrides[i] = PyList_GET_ITEM(s, i);
-		Py_INCREF(self->bytestrides[i]);
+		PyObject *new = PyList_GET_ITEM(s, i);
+		REPLACE(self->bytestrides[i], new);
 	}
 	return 0;
 }
@@ -289,9 +299,9 @@ _converter_result_buff_get(PyConverterObject *self)
 static int
 _converter_result_buff_set(PyConverterObject *self, PyObject *s)
 {
-	if (!s) { 
-		PyErr_Format(PyExc_RuntimeError, "can't delete _result_buff"); 
-		return -1; 
+	if (!s) {
+		PyErr_Format(PyExc_RuntimeError, "can't delete _result_buff");
+		return -1;
 	}
 	Py_INCREF(s);
 	Py_DECREF(self->result_buff);
@@ -366,6 +376,7 @@ _converter_new(PyTypeObject *type, PyObject  *args, PyObject *kwds)
 
 	self->rebuffer = _converter_rebuffer;
 	self->compute = _converter_compute;
+	self->clean = _converter_clean;
 
 	self->result_buff = self->convfunction = 
 		self->stridefunction = Py_None;
@@ -405,30 +416,45 @@ _stridesFromShape(PyArrayObject *arr, maybelong *strides)
 	    return 0;
 }
 
+static void
+_converter_clean(PyObject *me, PyObject *arr)
+{
+	PyConverterObject *self = (PyConverterObject *) me;
+	int i;
+
+	assert(libnumarray_API);
+	assert(NA_ConverterCheck(me));
+	assert(NA_NumArrayCheck(arr));
+
+	for(i=0; i<ELEM(self->buffers); i++)
+		if (self->buffers[i] == arr) {
+			REPLACE(self->buffers[i], Py_None);
+		}
+	if (self->result_buff == arr) {
+		REPLACE(self->result_buff, Py_None);
+	}	
+}
+
 static PyObject *
 _converter_rebuffer(PyObject *me, PyObject *arr, PyObject *inbuffer)
 {
 	PyConverterObject *self = (PyConverterObject *) me;
 
+	assert(NA_ConverterCheck(me));
+	assert(NA_NumArrayCheck(arr) || arr == Py_None);
+	assert(NA_NumArrayCheck(inbuffer) || inbuffer == Py_None);
+
 	if (!self->conversion_required) {
-		Py_INCREF(arr); 
-		Py_DECREF(self->result_buff);
-		self->result_buff = arr;
+		REPLACE(self->result_buff, arr);
 		Py_INCREF(arr);
 		return arr;
 	}
 
-	Py_INCREF(arr);
-	Py_DECREF(self->buffers[ self->arr_position] );
-	self->buffers[ self->arr_position ] = arr;
+	REPLACE(self->buffers[self->arr_position], arr);
 
 	if (!self->generated && inbuffer != Py_None) {
-		Py_INCREF(inbuffer);
-		Py_DECREF(self->buffers[self->inb_position]);
-		self->buffers[self->inb_position] = inbuffer;
-		Py_INCREF(inbuffer);
-		Py_DECREF(self->result_buff);
-		self->result_buff = inbuffer;
+		REPLACE(self->buffers[self->inb_position], inbuffer);
+		REPLACE(self->result_buff, inbuffer);
 	}
 	
 	if (arr != Py_None && (self->bytestrides[0] != Py_None || 
@@ -494,6 +520,8 @@ _converter_convert(PyConverterObject *self, int buf, PyObject *indiceso, PyObjec
 	maybelong nindices, indices[MAXDIM];
 	PyArrayObject  *arr1, *arr2;
 
+	assert(NA_ConverterCheck((PyObject *)self));
+
 	arr1 = (PyArrayObject *) self->buffers[buf*2+0];
 	arr2 = (PyArrayObject *) self->buffers[buf*2+1];
 
@@ -530,8 +558,7 @@ _converter_stride(PyConverterObject *self, int buf, PyObject *indiceso,
 {
 	PyObject *result, *buffers[2], *bytestrides[2];
 	long offsets[2];
-	maybelong nindices, indices[MAXDIM],
-		nshape, ishape[MAXDIM], 
+	maybelong nindices, indices[MAXDIM], nshape, ishape[MAXDIM], 
 		nstrides[2], istrides[2][MAXDIM];
 	PyArrayObject  *arr1, *arr2;
 
@@ -639,7 +666,7 @@ static PyMethodDef _converter_methods[] = {
 static PyTypeObject _converter_type = {
 	PyObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type))
 	0,
-	"_converter._converter",
+	"numarray._converter._converter",
 	sizeof(PyConverterObject),
 	0,
 	_converter_dealloc,	                /* tp_dealloc */
@@ -710,5 +737,7 @@ init_converter(void)
 			       (PyObject *) &_converter_type) < 0)
 		return;
 
+	ADD_VERSION(m);
+
 	import_libnumarray();
 }
