@@ -4,6 +4,8 @@
 
 #include <math.h>
 
+#define DEBUG(x) 
+
 /* Set _PYTHON_CALLBACKS to 1 for debugging with prototype Python code */
 #define _PYTHON_CALLBACKS 0
 
@@ -38,6 +40,23 @@ _ndarray_flags_get(PyArrayObject *self)
 	return PyInt_FromLong(self->flags);
 }
 
+static int
+_ndarray_flags_set(PyArrayObject *self, PyObject *f)
+{
+	if (!f) {
+		PyErr_Format( PyExc_RuntimeError, "can't delete flags");
+		return -1;			
+	}
+	if (!PyInt_Check(f)) {
+		PyErr_Format( PyExc_TypeError, "flags must be an integer value");
+		return -1;
+	}
+	self->flags = PyInt_AsLong(f);
+	NA_updateStatus(self);
+	return 0;
+}
+
+
 static PyObject *
 _ndarray_shape_get(PyArrayObject *self)
 {
@@ -47,12 +66,15 @@ _ndarray_shape_get(PyArrayObject *self)
 static int
 _ndarray_shape_set(PyArrayObject *self, PyObject *s)
 {
-	long ndim;
+	long i, ndim;
 	if (!s) { 
 		PyErr_Format(PyExc_RuntimeError, "can't delete _shape"); 
 		return -1; 
 	}
 	ndim = NA_maybeLongsFromIntTuple(MAXDIM, self->dimensions, s);
+	for(i=0; i<ndim; i++)
+		if (self->dimensions[i] < 0)
+			ndim = -1;
 	if (ndim < 0) {
 		PyErr_Format( PyExc_ValueError, "invalid shape tuple");
 		return -1;
@@ -107,6 +129,12 @@ _ndarray_data_get(PyArrayObject *self)
 	return self->_data;
 }
 
+static PyObject *
+_ndarray_fragile_data_get(PyArrayObject *self)
+{
+	return PyLong_FromUnsignedLong((unsigned long) self->data);
+}
+
 static int
 _ndarray_data_set(PyArrayObject *self, PyObject *s)
 {
@@ -173,11 +201,6 @@ _ndarray_bytestride_set(PyArrayObject *self, PyObject *s)
 			     "_ndarray_init: non-integer bytestride.");
 			return -1;
 		}
-		if (self->bytestride < self->itemsize) {
-			PyErr_Format(PyExc_ValueError,
-			     "_ndarray_init: bytestride must be >= itemsize.");
-			return -1;
-		}
 	}
 	NA_updateContiguous(self);
 	return rval;
@@ -209,85 +232,15 @@ _ndarray_byteoffset_set(PyArrayObject *self, PyObject *s)
 	return rval;
 }
 
-/* _aligned is an "override" attribute used to support testing and debug. 
-
-Setting _aligned *defines* an array's alignment state.  
-*/
-static PyObject *
-_ndarray_aligned_get(PyArrayObject *self)
-{
-	if (self->_aligned == MUST_BE_COMPUTED)
-		return NULL;
-	return PyInt_FromLong(self->_aligned);
-}
-
-static int
-_ndarray_aligned_set(PyArrayObject *self, PyObject *s)
-{
-	if (!s) {
-		self->_aligned = MUST_BE_COMPUTED;
-	} else if (!PyInt_Check(s)) {
-		PyErr_Format(PyExc_TypeError, 
-			     "_aligned must be an integer flag: 0 or 1.");
-		return -1;
-	} else {
-		long v = PyInt_AsLong(s);
-		if ((v >= 0) || (v <=2))
-			self->_aligned = v;
-		else {
-			PyErr_Format(PyExc_ValueError, 
-				     "_aligned must be 0, 1 or 2.");
-			return -1;
-		}
-			
-	}
-	NA_updateStatus(self);
-	return 0;
-}
-
-/* _contigous is an "override" attribute used to support testing and debug. 
-
-Setting _contiguous *defines* an array's contiguousness.  Other independent
-checks in libnumarray should keep numarray from crashing when you set this.
-*/
-static PyObject *
-_ndarray_contiguous_get(PyArrayObject *self)
-{
-	return PyInt_FromLong(self->_contiguous);
-}
-
-static int
-_ndarray_contiguous_set(PyArrayObject *self, PyObject *s)
-{
-	if (!s) {
-		self->_contiguous = MUST_BE_COMPUTED;
-	} else if (!PyInt_Check(s)) {
-		PyErr_Format(PyExc_TypeError, 
-			     "_contiguous must be an integer flag: 0 or 1.");
-		return -1;
-	} else {
-		long v = PyInt_AsLong(s);
-		if ((v >= 0) || (v <=2))
-			self->_contiguous = v;
-		else {
-			PyErr_Format(PyExc_ValueError, 
-				     "_contiguous must be 0, 1 or 2.");
-			return -1;
-		}
-	}
-	NA_updateStatus(self);
-	return 0;
-}
-
 static PyGetSetDef _ndarray_getsets[] = {
 	{"rank", 
 	 (getter)_ndarray_rank_get, 
 	 (setter) 0, 
-	 "tuple of array dimensions"}, 
-	{"flags", 
+	 "number of array dimensions"}, 
+	{"_flags", 
 	 (getter)_ndarray_flags_get, 
-	 (setter) 0, 
-	 "integer value of flags"}, 
+	 (setter)_ndarray_flags_set, 
+	 "integer value of _flags"}, 
 	{"_shape", 
 	 (getter)_ndarray_shape_get, 
 	 (setter)_ndarray_shape_set, 
@@ -299,7 +252,11 @@ static PyGetSetDef _ndarray_getsets[] = {
 	{"_data", 
 	 (getter)_ndarray_data_get,
 	 (setter)_ndarray_data_set, 
-	 "tuple of byte steps"}, 
+	 "buffer object holding array data"}, 
+	{"_fragile_data", 
+	 (getter)_ndarray_fragile_data_get,
+	 (setter)NULL, 
+	 "pointer to first array element, DEBUG ONLY"}, 
 	{"_byteoffset", 
 	 (getter)_ndarray_byteoffset_get,
 	 (setter)_ndarray_byteoffset_set, 
@@ -312,26 +269,14 @@ static PyGetSetDef _ndarray_getsets[] = {
 	 (getter)_ndarray_itemsize_get,
 	 (setter)_ndarray_itemsize_set, 
 	 "length of one element in bytes"}, 
-	{"_aligned",
-	 (getter)_ndarray_aligned_get,
-	 (setter)_ndarray_aligned_set,
-	 "aligment test/debug override"},
-	{"_contiguous",
-	 (getter)_ndarray_contiguous_get,
-	 (setter)_ndarray_contiguous_set,
-	 "contiguous test/debug override"},
 	{0}
 };
 
 static PyObject *
 _ndarray_new(PyTypeObject *type, PyObject  *args, PyObject *kwds)
 {
-	PyArrayObject *self;
-	
-	self = (PyArrayObject *) PyType_GenericNew(type, args, kwds);
-	if (!self)
-		return NULL;
-
+	PyArrayObject *self = (PyArrayObject *) type->tp_alloc(type, 0);
+	if (!self) return NULL;
 	Py_INCREF(Py_None);
 	self->_data = Py_None;
 	self->data = NULL;
@@ -348,8 +293,6 @@ _ndarray_new(PyTypeObject *type, PyObject  *args, PyObject *kwds)
 	self->itemsize = 0;
 	self->bytestride = 0;
 	self->byteoffset = 0;
-	self->_aligned = MUST_BE_COMPUTED;
-	self->_contiguous = MUST_BE_COMPUTED;
 	self->_shadows = NULL;
 	return (PyObject *) self;
 }
@@ -439,8 +382,44 @@ _ndarray_init(PyArrayObject *self, PyObject *args, PyObject *kwds)
 static void
 _ndarray_dealloc(PyObject *self)
 {
-	Py_XDECREF(((PyArrayObject *)self)->_data);
-	_ndarray_type.tp_base->tp_dealloc(self);
+	PyArrayObject *me = (PyArrayObject *) self;
+	PyObject *shadowed = me->_shadows;
+	me->_shadows = NULL;
+	if (shadowed) {
+		PyObject *result, *err_type, *err_value, *err_traceback;
+		int have_error = PyErr_Occurred() ? 1 : 0;
+		if (have_error)
+			PyErr_Fetch(&err_type, &err_value, &err_traceback);
+		DEBUG(fprintf(stderr, "_ndarray_dealloc: shadowed.\n"));
+		me->ob_refcnt = 1;
+		result = PyObject_CallMethod(
+			shadowed, "_copyFrom", "(O)", me);
+		assert(me->ob_refcnt > 0);
+		if (--me->ob_refcnt == 0) {
+			DEBUG(fprintf(stderr, "_ndarray_dealloc: zero references remaining.\n"));
+			Py_DECREF(shadowed);
+			if (result == NULL) {
+				PyObject *context = PyObject_GetAttrString(
+					shadowed, "_copyFrom");
+				if (context) {
+					PyErr_WriteUnraisable( context );
+					Py_DECREF(context);
+				}
+			} else
+				Py_DECREF(result);
+		} else {  /* _copyFrom brought self back to life */
+			int refcnt = me->ob_refcnt;
+			DEBUG(fprintf(stderr, "_ndarray_dealloc: resurrected.\n"));
+ 			_Py_NewReference((PyObject *)me);
+			me->ob_refcnt = refcnt;
+			return;
+		}
+		if (have_error)
+			PyErr_Restore(err_type, err_value, err_traceback);
+	}
+	Py_XDECREF(me->_data);
+	me->_data = NULL;
+	self->ob_type->tp_free(self);
 }
 
 static int
@@ -533,8 +512,11 @@ _simpleIndexingCore(PyArrayObject *self, long offset, int nindices, PyObject *va
 		}
 	} else {  /* partially subscripted --> subarray */
 		long i;
-		result = (PyArrayObject *) 
-			PyObject_CallMethod((PyObject *) self,"view",NULL);
+		if (NA_NumArrayCheck((PyObject *)self))		      
+			result = (PyArrayObject *) _view(self);
+		else
+			result = (PyArrayObject *) PyObject_CallMethod(
+				(PyObject *) self,"view",NULL);
 		if (!result) goto _exit;
 		
 		result->nd = result->nstrides = self->nd - nindices;
@@ -985,18 +967,28 @@ _ndarray_universalIndexing(PyArrayObject *self,
 	return _universalIndexing(self, key, value);
 }
 
+
 static PyObject*
 _ndarray_subscript(PyArrayObject* self, PyObject* key)
-	
+   
 {
-	PyObject *result;
+    PyObject *result;
+    if (PyInt_CheckExact(key)) {
+            maybelong ikey = PyInt_AsLong(key);
+            long offset;
+            if (NA_getByteOffset(self, 1, &ikey, &offset) < 0)
+		    return NULL;
+            if (!NA_updateDataPtr(self))
+		    return NULL;
+            return _simpleIndexingCore(self, offset, 1, Py_None);
+    }
 #if _PYTHON_CALLBACKS
-	result = PyObject_CallMethod(
-		(PyObject *) self, "_universalIndexing", "(OO)", key, Py_None);
+    result = PyObject_CallMethod(
+        (PyObject *) self, "_universalIndexing", "(OO)", key, Py_None);
 #else
-	result = _universalIndexing(self, key, Py_None);
+    result = _universalIndexing(self, key, Py_None);
 #endif
-	return result;
+    return result;
 }
 
 static int 
@@ -1037,12 +1029,13 @@ _ndarray_as_mapping = {
 static PyObject *
 _ndarray_item(PyArrayObject *self, int i)
 {
-	PyObject *result;
-	PyObject *key = PyInt_FromLong(i);
-	if (!key) return NULL;
-	result = _universalIndexing(self, key, Py_None);
-	Py_DECREF(key);
-	return result;
+    long offset;
+    maybelong mi = i;
+    if (NA_getByteOffset(self, 1, &mi, &offset) < 0)
+        return NULL;
+    if (!NA_updateDataPtr(self))
+        return NULL;
+    return _simpleIndexingCore(self, offset, 1, Py_None);
 }
 
 static PySequenceMethods 
@@ -1054,6 +1047,112 @@ _ndarray_as_sequence = {
 };
 #endif
 
+/* Beginning of methods added by Scott N. Gunyan
+   to alloy export of array as a Python buffer object.
+
+   ported from Numeric to numarray by Todd Miller
+*/
+
+#define NBYTES(a) (NA_elements(a) * a->itemsize)
+
+static int array_getsegcount(PyArrayObject *self, int *lenp) {
+    int i, elsize;
+    int num_segments=1;
+
+    if (lenp) 
+	    *lenp = NBYTES(self);
+
+    elsize = self->itemsize;  /* self->descr->elsize; */
+
+    /* Check how self is CONTIGUOUS:  see if strides match 
+       dimensions */
+    for (i=self->nd-1; i>=0; i--) {
+	if (self->strides[i] == elsize) {
+	    elsize *= self->dimensions[i];
+	} else {
+	    break;
+	}
+    }
+    if (i >= 0) {
+        for (; i >= 0; i--) {
+            num_segments *= self->dimensions[i];
+	}
+    }
+    return num_segments;
+}
+
+
+/* this will probably need some work, could be faster */
+int get_segment_pointer(PyArrayObject *self, int segment, int i) {
+    if (i >= 0) {
+    return ((segment%self->dimensions[i])*self->strides[i] +
+            get_segment_pointer(self,segment/self->dimensions[i],i-1));
+    } else {
+        return 0;
+    }
+}
+
+
+
+static int array_getreadbuf(PyArrayObject *self, int segment, void **ptrptr) {
+    int num_segments, product=1, i=0;
+    
+    if ((segment < 0) || 
+        (segment > (num_segments = array_getsegcount(self,NULL)))) {
+        PyErr_SetString(PyExc_SystemError,
+                        "Accessing non-existent array segment");
+        return -1;
+    }
+
+    if (!PyArray_ISALIGNED(self) || PyArray_ISBYTESWAPPED(self)) {
+	    PyErr_SetString(PyExc_ValueError, 
+			    "Can't get buffer pointer from byteswapped or misaligned array.");
+	    return -1;
+    }
+    
+    if (num_segments > 1) {
+        /* find where the break in contiguity is */    
+        while ((product != num_segments) && (i < self->nd)) {
+            product *= self->dimensions[i++];
+        }
+        /* now get the right segment pointer (i is indexing break in contiguity) */
+        *ptrptr = self->data + get_segment_pointer(self,segment,i-1);
+    } else {
+        *ptrptr = self->data;
+    }
+    /* return the size in bytes of the segment (i is indexing break in contiguity) */
+    return NBYTES(self);
+}
+
+
+static int array_getwritebuf(PyArrayObject *self, int segment, void **ptrptr) {
+    if (!PyArray_ISWRITABLE(self)) {
+	    PyErr_SetString(PyExc_ValueError,
+			    "Can't get writable buffer from read-only array.");
+	    return -1;
+    }
+    return array_getreadbuf(self, segment, (void **) ptrptr);
+}
+
+static int array_getcharbuf(PyArrayObject *self, int segment, const char **ptrptr) {
+    if (self->descr->type_num == PyArray_CHAR)
+        return array_getreadbuf(self, segment, (void **) ptrptr);
+    else {
+        PyErr_SetString(PyExc_TypeError, 
+                        "Non-character array cannot be interpreted as character buffer.");
+        return -1;
+    }
+}
+
+static PyBufferProcs array_as_buffer = {
+    (getreadbufferproc)array_getreadbuf,    /*bf_getreadbuffer*/
+    (getwritebufferproc)array_getwritebuf,  /*bf_getwritebuffer*/
+    (getsegcountproc)array_getsegcount,     /*bf_getsegcount*/
+    (getcharbufferproc)array_getcharbuf,    /*bf_getcharbuffer*/
+};
+/* End methods added by Scott N. Gunyan for buffer extension */
+
+
 static PyObject *
 _ndarray_nelements(PyArrayObject *self, PyObject  *args)
 {
@@ -1126,13 +1225,13 @@ _view(PyArrayObject *self)
 		self->ob_type, NullArgs, NullKeywds);
 	if (!view) goto _fail;
 
-	vd = PyObject_GetAttrString((PyObject *) view, "__dict__");
-	if (!vd) goto _fail;
-
-	sd = PyObject_GetAttrString((PyObject *) self, "__dict__");
-	if (!sd) goto _fail;
-
-	if (PyDict_Update(vd, sd) < 0) goto _fail;
+	if (self->flags & UPDATEDICT) {
+		vd = PyObject_GetAttrString((PyObject *) view, "__dict__");
+		if (!vd) goto _fail;		
+		sd = PyObject_GetAttrString((PyObject *) self, "__dict__");
+		if (!sd) goto _fail;	      
+		if (PyDict_Update(vd, sd) < 0) goto _fail;
+	}
 
 	for(i=0; i<self->nd; i++)
 		view->dimensions[i] = self->dimensions[i];
@@ -1144,8 +1243,6 @@ _view(PyArrayObject *self)
 	view->byteoffset = self->byteoffset;
 	view->bytestride = self->bytestride;
 	view->itemsize = self->itemsize;
-	view->_aligned = self->_aligned;
-	view->_contiguous = self->_contiguous;
 	view->flags = self->flags;
 	
 	/* Numarray hack here; since fields are always present, copy them. */
@@ -1155,7 +1252,8 @@ _view(PyArrayObject *self)
 	Py_XDECREF(view->_data);
 	view->_data = self->_data;  
 	Py_INCREF(self->_data);
-	if (!NA_updateDataPtr(view)) goto _fail;
+	if (!NA_updateDataPtr(view)) 
+		goto _fail;
 
 	NA_updateStatus(view);
 	goto _exit;
@@ -1395,7 +1493,7 @@ static PyTypeObject
 _ndarray_type = {
 	PyObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type))
 	0,
-	"_ndarray._ndarray",
+	"numarray._ndarray._ndarray",
 	sizeof(PyArrayObject),
 	0,
 	_ndarray_dealloc,	                /* tp_dealloc */
@@ -1416,7 +1514,7 @@ _ndarray_type = {
 	0,					/* tp_str */
 	0,					/* tp_getattro */
 	0,					/* tp_setattro */
-	0,					/* tp_as_buffer */
+	&array_as_buffer,			/* tp_as_buffer */
 	Py_TPFLAGS_DEFAULT | 
         Py_TPFLAGS_BASETYPE,                    /* tp_flags */
 	0,					/* tp_doc */
@@ -1435,8 +1533,9 @@ _ndarray_type = {
 	0,					/* tp_descr_set */
 	0,					/* tp_dictoffset */
 	(initproc)_ndarray_init,		/* tp_init */
-	0, 		                        /* tp_alloc */
+	0,      	                        /* tp_alloc */
 	_ndarray_new,				/* tp_new */
+	0,                                      /* tp_free */
 };
 
 static PyObject *
@@ -1476,6 +1575,19 @@ _ndarray_hasUInt64(PyObject *module, PyObject *args)
 	return PyInt_FromLong(HAS_UINT64);
 }
 
+
+static PyObject *
+_ndarray_is_buffer(PyObject *module, PyObject *args)
+{
+	PyObject *o;
+	int ans;
+	if (!PyArg_ParseTuple(args, "O:is_buffer", &o))
+		return NULL;
+	if ((ans = isBuffer(o)) < 0) 
+		return NULL;
+	return PyInt_FromLong(ans);
+}
+
 static PyMethodDef 
 _ndarray_functions[] = {
 	{"_isIntegerSequence", 
@@ -1494,6 +1606,9 @@ _ndarray_functions[] = {
 	 (PyCFunction) _ndarray_hasUInt64,
 	 METH_VARARGS,
 	 "hasUInt64()  returns flag indicating whether this system has 64-bit unsigned integer support."},
+	{"is_buffer", (PyCFunction) _ndarray_is_buffer,
+	 METH_VARARGS,
+	 "is_buffer(o)  returns 1 iff 'o' should work as a buffer."},
 	{NULL,		NULL}		/* sentinel */
 };
 
@@ -1534,6 +1649,8 @@ init_ndarray(void)
 	if (PyModule_AddObject(m, "_alignment", _alignment) < 0)
 		return;
 
+	ADD_VERSION(m);
+
 	import_libnumarray();
 
 	pNewMemoryFunc = NA_initModuleGlobal("numarray.memory","new_memory");
