@@ -1,4 +1,4 @@
-/* $Id: _numarraymodule.c,v 1.62 2004/11/01 15:12:29 jaytmiller Exp $ */
+/* $Id: _numarraymodule.c,v 1.71 2005/04/05 20:18:38 jaytmiller Exp $ */
 
 #include <Python.h>
 #include <stdio.h>
@@ -20,6 +20,11 @@ char *_numarray__doc__ =
 
 staticforward PyTypeObject _numarray_type;
 
+
+static PyObject *
+_copyFrom(PyObject *self, PyObject *arr0);
+
+
 static int
 _numarray_init(PyArrayObject *self, PyObject *args, PyObject *kwds)
 {
@@ -104,26 +109,6 @@ _numarray_init(PyArrayObject *self, PyObject *args, PyObject *kwds)
 	return 0;
 }
 
-static PyObject *
-_numarray_shadows_get(PyArrayObject *self)
-{
-	if (self->_shadows) {
-		Py_INCREF(self->_shadows);
-		return self->_shadows;
-	} else {
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
-}
-
-static int
-_numarray_shadows_set(PyArrayObject *self, PyObject *s)
-{
-	Py_XDECREF(self->_shadows);
-	if (s) Py_INCREF(s);
-	self->_shadows = s;
-	return 0;
-}
 
 static PyObject *
 _numarray_type_get(PyArrayObject *self)
@@ -218,9 +203,6 @@ _numarray_check_overflow_set(PyArrayObject *self, PyObject *s)
 }
 
 static PyGetSetDef _numarray_getsets[] = {
- 	{"_shadows", 
-	 (getter)_numarray_shadows_get, 
-	 (setter) _numarray_shadows_set, "numeric shadows object"}, 
  	{"_type", 
 	 (getter)_numarray_type_get, 
 	 (setter) _numarray_type_set, "numeric type object"}, 
@@ -348,9 +330,14 @@ _copyFrom(PyObject *self, PyObject *arr0)
 			return result;
 		}
 	} 
-	barr = PyObject_CallMethod(
-		self, "_broadcast", "(O)", arr);
+	barr = PyObject_CallMethod(self, "_broadcast", "(O)", arr);
 	Py_DECREF(arr);
+	if (barr == Py_None) {
+		Py_DECREF(barr);
+		return PyErr_Format(
+			PyExc_ValueError, "array sizes must be consistent.");
+				    
+	}
 	if (!barr) return NULL;
 	result = PyObject_CallFunction(
 		p_copyFromAndConvert, "(OO)", barr, self);
@@ -415,10 +402,10 @@ static PyMethodDef _numarray_methods[] = {
 static PyTypeObject _numarray_type = {
 	PyObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type))
 	0,
-	"_numarray._numarray",
+	"numarray._numarray._numarray",
 	sizeof(PyArrayObject),
         0,
-	0,			                /* tp_dealloc */
+	0,                  			/* tp_dealloc */
 	0,					/* tp_print */
 	0,					/* tp_getattr */
 	0,					/* tp_setattr */
@@ -539,10 +526,11 @@ _rank0_to_rank1(PyArrayObject *ao)
 }
 
 static PyObject *
-_innerproduct(PyArrayObject *a, PyArrayObject *b, NumarrayType maxt)
+_innerproduct(PyArrayObject *a, PyArrayObject *b, NumarrayType maxt,
+	      char *kind)
 {
 	PyArrayObject *r;
-	maybelong i, imax, jmax, kmax, dots[2*MAXDIM];
+	maybelong ae, be, i, imax, jmax, kmax, dots[2*MAXDIM];
 	char *ap, *bp ,*rp;
 
 	a = _rank0_to_rank1(a);
@@ -563,8 +551,13 @@ _innerproduct(PyArrayObject *a, PyArrayObject *b, NumarrayType maxt)
 	  return NULL;
 	}
 
-	imax = NA_elements(a) / a->dimensions[ a->nd - 1];
-	jmax = NA_elements(b) / b->dimensions[ b->nd - 1];
+	ae = NA_elements(a);
+	be = NA_elements(b);
+	if (ae==0 || be==0)
+		return (PyObject *) r;
+
+	imax = ae / a->dimensions[ a->nd - 1];
+	jmax = be / b->dimensions[ b->nd - 1];
 	kmax = a->dimensions[ a->nd - 1 ];
 
 	NA_clearFPErrors();
@@ -600,12 +593,12 @@ _innerproduct(PyArrayObject *a, PyArrayObject *b, NumarrayType maxt)
 	Py_DECREF(a);
 	Py_DECREF(b);
 
-	if (NA_checkAndReportFPErrors("innerproduct") < 0)
+	if (NA_checkAndReportFPErrors(kind) < 0)
 	{
 		Py_DECREF(r);
 		return NULL;
 	}
-		
+	
 	return PyArray_Return(r);
 }
 
@@ -661,7 +654,7 @@ innerproduct(PyObject *module, PyObject *args)
 		goto _done;
 	}
 
-	result = _innerproduct(a,b,maxt);
+	result = _innerproduct(a,b,maxt,"innerproduct");
 
   _done:
 	Py_XDECREF(a);
@@ -701,6 +694,7 @@ dot(PyObject *module, PyObject *args)
 	Py_INCREF(b_original);
 	if (!PyArray_ISCARRAY(b)) {
 		b2 = NA_copy(b);
+		if (!b2) goto _done;
 	} else {
 		b2 = b;
 		Py_INCREF(b);
@@ -713,7 +707,7 @@ dot(PyObject *module, PyObject *args)
 		goto _done;
 	}
 
-	result = _innerproduct(a,b2,maxt);
+	result = _innerproduct(a,b2,maxt,"dot");
 
 	if (NA_swapAxes(b_original, -1, -2) < 0) {
 		Py_DECREF(result);
@@ -782,6 +776,8 @@ DL_EXPORT(void)
 	if (PyModule_AddObject(m, "_numarray", (PyObject *) &_numarray_type) < 0)
 		return;
 
+	ADD_VERSION(m);
+
 	import_libnumarray();
 	import_libnumeric();
 }
