@@ -3,6 +3,7 @@
 
 #include "structmember.h"
 #include "libnumarray.h"
+#include "cfunc.h"
 
 /* MEASURE_TIMING is only used on i386-Linux to profile performance */
 #include "tc.h"
@@ -29,6 +30,7 @@ static PyObject *p_copyFromAndConvertFunc;
 static PyObject *p_copyFromAndConvertMissFunc;
 static PyObject *p_copyCacheDict;
 static PyObject *p_getThreadIdent;
+static PyObject *pUnknownOperator;
 
 /* ====================================================================== */
 
@@ -42,6 +44,9 @@ static PyObject *p_getThreadIdent;
 	((PyConverterObject *)conv)->rebuffer(conv, arr, inb)
 #endif
 
+#define ConverterClean(conv, arr) \
+        ((PyConverterObject *)conv)->clean(conv, arr)
+
 /* ====================================================================== */
  
 
@@ -61,13 +66,137 @@ static int deferred_ufunc_init(void);
 
 /* ====================================================================== */
 
+static PyObject * 
+_normalize_results(int nin,     PyObject **inputs,
+		   int nout,    PyObject **outputs,
+		   int nresult, PyObject **results,
+		   int return_rank1)
+{
+	PyObject *rval;
+	int i, r;
+	
+	if ((nresult == 0) 
+	    || ((nout == 1) && (outputs[0] != Py_None)) 
+	    || (nout > 1)) {
+		Py_INCREF(Py_None);
+		return Py_None;
+	}
+	
+	rval = PyTuple_New(nresult);
+	if (!rval) return NULL;
+	
+	if (((PyArrayObject *)*results)->nd == 0) {
+		for (i=0; i<nin; i++) {
+			PyObject *input = inputs[i];
+                        /* At least 1 rank0 -> rank0 result */
+			if (NA_NumArrayCheck(input) && ((PyArrayObject *)input)->nd == 0) {  
+				for (r=0; r<nresult; r++) {
+					PyArrayObject *result = (PyArrayObject *) results[r];
+					if (return_rank1) {
+						result->nd = result->nstrides = 1;
+						result->dimensions[0] = 1;
+						result->strides[0] = result->bytestride;
+					}
+					Py_INCREF(result);
+					PyTuple_SET_ITEM(
+						rval, r, (PyObject*)result);
+				}
+				break;
+			}
+		}
+		if (i == nin) {	/* no rank0 -> scalar results */	
+			for (r=0; r<nresult; r++) {
+				PyObject *new_result = NA_getPythonScalar
+					((PyArrayObject *) results[r], 0L);
+				if (!new_result) return NULL;
+				PyTuple_SET_ITEM(rval, r, new_result);
+			}
+		}
+	} else {
+		for(r=0; r<nresult; r++) {
+			PyTuple_SET_ITEM(rval, r, results[r]);
+			Py_INCREF(results[r]);
+		}	    
+	}
+
+	/* Convert single result tuples into simply the result */
+	if (nresult == 1) {
+		PyObject *result = PyTuple_GetItem(rval, 0);
+		Py_INCREF(result);
+		Py_DECREF(rval);
+		return result;
+	} else {
+		return rval;
+	}
+}
+
+static int
+_tuple_check(PyObject *o, char *name)
+{
+    if (PyTuple_Check(o)) return 0;
+
+    PyErr_Format
+      (PyExc_TypeError, "%s should be a tuple", name);
+
+    return -1;
+}
+
+static PyObject * 
+_Py_normalize_results(PyObject *module, PyObject *args)
+{
+    PyObject *tinputs, *toutputs, *tresults;
+    PyObject *inputs[MAXARRAYS], *outputs[MAXARRAYS], *results[MAXARRAYS];
+    int i, return_rank1 = 0;
+    int nin, nout, nresult;
+
+    if (!PyArg_ParseTuple(args, "OOO|i:_normalize_results", 
+			  &tinputs, &toutputs, &tresults, &return_rank1))
+	return NULL;
+
+    if (_tuple_check(tinputs, "inputs") < 0)
+	return NULL;
+
+    nin = PyTuple_Size(tinputs);
+    for(i=0; i<nin; i++) {
+	inputs[i] = PyTuple_GetItem(tinputs, i);
+	if (!inputs[i]) return NULL;
+    }
+    
+    if (toutputs == Py_None) {
+	nout = 0;
+    } else if (_tuple_check(toutputs, "outputs") < 0) {
+	return NULL;
+    } else {
+	nout = PyTuple_Size(toutputs);
+	for(i=0; i<nout; i++) {
+	    outputs[i] = PyTuple_GetItem(toutputs, i);
+	    if (!outputs[i]) return NULL;
+	}
+    }
+
+    if (_tuple_check(tresults, "results") < 0)
+	return NULL;
+
+    nresult = PyTuple_Size(tresults);
+    for(i=0; i<nresult; i++) {
+	results[i] = PyTuple_GetItem(tresults, i);
+	if (!results[i]) return NULL;
+    }
+
+    if ((return_rank1 != 0) && (return_rank1 != 1))
+	return PyErr_Format(
+	    PyExc_ValueError, "return_rank1 should be 0 or 1.");
+
+    return _normalize_results(
+	nin, inputs, nout, outputs, nresult, results, return_rank1);
+}
+
 /* _firstcol is an array mutator that changes an array into
 a view of its first column.  _firstcol_undo undoes the mutation.
 This approach is an optimization over the Python prototype's clean
 but slow view based approach.  This ugliness avoids 1-2 view method
 callbacks which are fairly expensive.
 */
-
 typedef struct 
 {
 	int nd;
@@ -100,44 +229,10 @@ _firstcol_undo(PyObject *arr, firstcol_undo *undo)
 
 static PyObject *
 CheckFPErrors(PyObject *self, PyObject *args) {
-
-	return Py_BuildValue("i", NA_checkFPErrors());
+    return Py_BuildValue("i", NA_checkFPErrors());
 }
 
 
-static int
-_reportErrors(PyObject *ufunc, PyObject *result, PyObject *out)
-{
-	PyObject *opname = PyObject_GetAttrString(ufunc, "operator");
-	char name[128];
-
-	if (deferred_ufunc_init() < 0) 
-		return -1;
-
-	if (!result) return -1;
-	Py_DECREF(result);  /* Py_None */
-
-	strncpy(name, opname ? PyString_AsString(opname) :
-		"<unknown operator>", 127);
-	Py_XDECREF(opname);
-
-	return NA_checkAndReportFPErrors(name);
-}
-
-static PyObject *
-_returnScalarOrArray(PyObject *out)
-{
-  if (!NA_NumArrayCheck(out))
-    return PyErr_Format(PyExc_TypeError, "_ufunc:_returnScalarOrArray output is not an array");
-  if (((PyArrayObject*)out)->nd == 0) {
-    if (!NA_updateDataPtr((PyArrayObject *)out))
-      return NULL;
-    return NA_getPythonScalar((PyArrayObject *) out, 0);
-  }
-  Py_INCREF(out);
-  return out;
-}
-
 static PyObject *
 _getNewArray(PyObject *master, PyObject *type)
 {
@@ -257,10 +352,22 @@ typedef struct
 
 typedef struct {
 	PyObject_HEAD
+	PyObject *operator;
 	int    arity;
 	ufunc_cache cache;
 } PyUfuncObject;
 
+static int
+_reportErrors(PyObject *ufunc, PyObject *result, PyObject *out)
+{
+	PyUfuncObject *self = (PyUfuncObject *) ufunc;
+	if (deferred_ufunc_init() < 0) 
+		return -1;
+	if (!result) return -1;
+	Py_DECREF(result);  /* Py_None */
+	return NA_checkAndReportFPErrors(PyString_AS_STRING(self->operator));
+}
+
 #if defined(UNTHREADED)
 static long PyThread_get_thread_ident() { return 0; }
 #endif
@@ -400,6 +507,32 @@ _ufunc_arity_set(PyUfuncObject *self, PyObject *s)
 	return 0;
 }
 
+static PyObject *
+_ufunc_operator_get(PyUfuncObject *self)
+{
+	Py_INCREF(self->operator);
+	return self->operator;
+}
+
+static int
+_ufunc_operator_set(PyUfuncObject *self, PyObject *s)
+{
+	if (!s) { 
+		PyErr_Format(PyExc_RuntimeError, 
+			     "_ufunc_operator_set:  operator can't be deleted.");
+		return -1;
+	}
+	if (!PyString_Check(s)) {
+		PyErr_Format(PyExc_TypeError, 
+			     "_ufunc_operator_set:  operator must be a string.");
+		return -1;
+	}
+	Py_INCREF(s);
+	Py_DECREF(self->operator);
+	self->operator = s;
+	return 0;
+}
+
 static PyGetSetDef 
 _ufunc_getsets[] = 
 {
@@ -407,6 +540,10 @@ _ufunc_getsets[] =
 	 (getter)_ufunc_arity_get, 
 	 (setter)_ufunc_arity_set, 
 	 "determines if Ufunc is unary or binary"},
+	{"operator", 
+	 (getter)_ufunc_operator_get, 
+	 (setter)_ufunc_operator_set, 
+	 "name of the ufunc operator"},
 	{0}
 };
 
@@ -455,6 +592,8 @@ deferred_ufunc_init(void)
 	if (!(p_getThreadIdent = NA_initModuleGlobal("numarray.safethread",
 						     "get_ident")))
 		return -1;
+	if (!(pUnknownOperator = PyString_FromString("<unknown operator>")))
+		return -1;
 	inited = 1;
 	return 0;
 }
@@ -479,21 +618,25 @@ _Py_getBufferSize(PyObject *module, PyObject *args)
 static PyObject *
 _ufunc_new(PyTypeObject *type, PyObject  *args, PyObject *kwds)
 {
-	PyUfuncObject *self = 
-		(PyUfuncObject *) PyType_GenericNew(type, args, kwds);
+	PyUfuncObject *self;
+	
+	if (deferred_ufunc_init() < 0) return NULL;
 
-	if (!self)
-		return NULL;
+	self = (PyUfuncObject *) PyType_GenericNew(type, args, kwds);
+	if (!self) return NULL;
 	
 	memset(&self->cache, 0, sizeof(self->cache));
-
+	self->operator = pUnknownOperator;
+	Py_INCREF(pUnknownOperator);
 	return (PyObject *) self;
 }
 
 static void
 _ufunc_dealloc(PyObject *self)
 {
-	_cache_flush(&((PyUfuncObject *)self)->cache);
+	PyUfuncObject *me = (PyUfuncObject*) self;
+	_cache_flush(&me->cache);
+	Py_DECREF(me->operator);
 	self->ob_type->tp_free(self);
 }
 
@@ -507,13 +650,7 @@ static PyObject *
 _ufunc_call( PyUfuncObject *self, PyObject *args)
 {
 	PyObject *in1, *in2 = Py_None, *out = Py_None;
-	char spec[200], *name;
-
-	PyObject *operator = PyObject_GetAttrString((PyObject * ) self, "operator");
-	if (!operator) return NULL;
-	name = PyString_AsString(operator);
-	if (!name) return NULL;
-
+	char spec[200];
 	switch(self->arity) 
 	{
 	case 0: default:
@@ -521,14 +658,12 @@ _ufunc_call( PyUfuncObject *self, PyObject *args)
 			PyExc_RuntimeError, 
 			"_ufunc_call: __call__ is not implemented by base UFunc class");
 	case 1:
-		snprintf(spec, sizeof(spec), "O|O:%s", name);
-		Py_DECREF(operator);
+		snprintf(spec, sizeof(spec), "O|O:%s", PyString_AS_STRING(self->operator));
 		if (!PyArg_ParseTuple(args, spec, &in1, &out))
 			return NULL;
 		return _cached_dispatch1((PyObject *) self, in1, out); 
 	case 2:
-		snprintf(spec, sizeof(spec), "OO|O:%s", name);
-		Py_DECREF(operator);
+		snprintf(spec, sizeof(spec), "OO|O:%s", PyString_AS_STRING(self->operator));
 		if (!PyArg_ParseTuple(args, spec, &in1, &in2, &out))
 			return NULL;
 		return _cached_dispatch2((PyObject *) self, in1, in2, out);
@@ -589,11 +724,12 @@ static PyObject *
 _reduce_out(PyObject *self, PyObject *inarr, PyObject *outarr, PyObject *otype)
 {
 	PyArrayObject *toutarra;
+	PyArrayObject *inarra = (PyArrayObject *) inarr;
 	PyArrayObject *outarra = (PyArrayObject *) outarr;
 	PyObject *result, *toutarr;
 	firstcol_undo fc_in;
 	int nelements;
-	int otypenum = NA_typeObjectToTypeNo(otype);
+	int i, otypenum = NA_typeObjectToTypeNo(otype);
 	if (otypenum < 0) return PyErr_Format(
 		PyExc_ValueError, "_reduce_out: problem with otype");
 
@@ -605,7 +741,8 @@ _reduce_out(PyObject *self, PyObject *inarr, PyObject *outarr, PyObject *otype)
 	if ((outarr == Py_None) || outarra->descr->type_num != otypenum) {
 		toutarr = _getNewArray(inarr, otype);
 	} else {
-		toutarr = PyObject_CallMethod(outarr, "view", NULL);
+		toutarr = outarr;
+		Py_INCREF(outarr);
 	}
 	if (!toutarr) goto _fail;
 
@@ -631,6 +768,16 @@ _reduce_out(PyObject *self, PyObject *inarr, PyObject *outarr, PyObject *otype)
 	if (toutarra->nstrides < MAXDIM-1) {
 		toutarra->strides[ toutarra->nstrides ] = 0;
 		++ toutarra->nstrides;
+		toutarra->nd = inarra->nd;
+		for(i=0; i<inarra->nd; i++)
+			toutarra->dimensions[i] = inarra->dimensions[i];
+/*		if (toutarr->nd < toutarr->nstrides) {
+			diff = toutarr->nstrides - toutarr->nd;
+			for(i=0; i<toutarr->nd; i++)
+				toutarr->strides[i] = toutarr->strides[i+diff];
+			toutarr->nstrides = toutarr->nd;
+		}
+*/
 		NA_updateStatus(toutarra);
 	}
 	return toutarr;
@@ -665,7 +812,7 @@ _cum_lookup(PyObject *self, char *cumop, PyObject *in1, PyObject *out,
 	} else {
 		PyObject *wout;
 		PyObject *otype = PyTuple_GET_ITEM(cached, 1);
-		if (!strcmp(cumop, "reduce")) {
+		if (!strcmp(cumop, "R")) {
 #if _PYTHON_CALLBACKS
 			wout = PyObject_CallMethod(
 			   self, "_reduce_out", "(OOO)", in1, out, otype);
@@ -911,7 +1058,7 @@ static PyObject *
 _cum_cached(PyObject *self, char *cumop, PyObject *in1, PyObject *out,
 	    PyObject *type)
 {
-	int otype;
+	int i, otype;
 	PyObject *params, *cached, *otypeObj;
 	PyArrayObject *wout;
 	if (deferred_ufunc_init() < 0) return NULL;
@@ -926,23 +1073,10 @@ _cum_cached(PyObject *self, char *cumop, PyObject *in1, PyObject *out,
 				"misaligned or byteswapped output numarray not supported by reduce/accumulate");
 		}
 	}
-	if (strcmp(cumop, "reduce") && strcmp(cumop, "accumulate"))
+	if (strcmp(cumop, "R") && strcmp(cumop, "A"))
 		return PyErr_Format(PyExc_ValueError, 
 				    "Unknown cumulative operation");
 
-	if ((PyArray(in1)->nd == 0) || 
-	    ((PyArray(in1)->nd == 1 && PyArray(in1)->dimensions[0] == 0))) {
-		PyObject *_identity = PyObject_GetAttrString(self, "_identity");
-		if (!_identity || _identity == Py_None) {
-			Py_XDECREF(_identity);
-			return PyErr_Format(
-				PyExc_ValueError, 
-				"Empty array in a ufunc that has no identity value.");
-		} else {
-			Py_XDECREF(_identity);
-			return PyObject_CallMethod(self, "_getidentity", NULL);
-		}
-	}	
 #if _PYTHON_CALLBACKS
 	params = PyObject_CallMethod(
 		self, "_cum_lookup", "(sOOO)", cumop, in1, out, type);
@@ -962,8 +1096,19 @@ _cum_cached(PyObject *self, char *cumop, PyObject *in1, PyObject *out,
 		return NULL;
 	}
 
-	if (!strcmp(cumop, "reduce")) {
-		wout->nstrides -= 1;
+	if (!strcmp(cumop, "R")) {
+		PyArrayObject *in1a = (PyArrayObject *) in1;
+		if (in1a->nd) {
+			wout->nd = in1a->nd - 1;
+			for(i=0; i<wout->nd; i++)
+				wout->dimensions[i] = in1a->dimensions[i];
+		} else 
+			wout->nd = 0;
+		if ((wout->nstrides = wout->nd)) {
+			NA_stridesFromShape(
+				wout->nd, wout->dimensions, wout->bytestride, 
+				wout->strides);
+		}
 		if (!wout->nd) {
 			wout->nstrides = wout->nd = 1;
 			wout->dimensions[0] = 1;
@@ -1036,13 +1181,17 @@ _cum_swapped(PyObject *self, PyObject *in1, int dim,
 	     PyObject *out, char *cumop, PyObject *type)
 {
 	PyObject *_out=NULL;
+	PyArrayObject *in1a = (PyArrayObject  *) in1;
 	
 	if (deferred_ufunc_init() < 0) return NULL;
 	
+	if (in1a->nd == 0) /* rank-0 null reduction/accumulation */
+		return PyObject_CallMethod(in1, "copy", NULL);
+	
 	if (dim == PyArray(in1)->nd-1) dim = -1;
 	
 	if (dim != -1) {
-		if (NA_swapAxes((PyArrayObject *) in1, -1, dim) < 0)
+		if (NA_swapAxes(in1a, -1, dim) < 0)
 			goto fail;
 		if (NA_swapAxes((PyArrayObject *) out, -1, dim) < 0)
 			goto fail;
@@ -1055,8 +1204,8 @@ _cum_swapped(PyObject *self, PyObject *in1, int dim,
 	_out = _cum_cached(self, cumop, in1, out, type);
 #endif
 	if (!_out) goto fail;
-	
-	if (!strcmp(cumop, "accumulate")) {
+
+	if (!strcmp(cumop, "A")) {
 		if (dim != -1) {
 			/* either _out == out, or out == None */
 			if (NA_swapAxes((PyArrayObject *)_out, -1, dim) < 0) 
@@ -1070,11 +1219,10 @@ _cum_swapped(PyObject *self, PyObject *in1, int dim,
 			NA_updateStatus(_outa);
 		}
 	}
-	 
-	if (dim != -1) {
-		if (NA_swapAxes((PyArrayObject *) in1, -1, dim) < 0)
-			goto fail;
 
+	if (dim != -1) {
+		if (NA_swapAxes(in1a, -1, dim) < 0)
+			goto fail;
 	} 
 	
 	if (out == Py_None) {
@@ -1164,7 +1312,7 @@ _Py_accumulate(PyObject *self, PyObject *args, PyObject *keywds)
 
 	if (_fixdim(&axis, &dim) < 0) return NULL;
 
-	r = _cum_swapped(self, _in1, axis, out, "accumulate", type);
+	r = _cum_swapped(self, _in1, axis, out, "A", type);
 
 	Py_DECREF(_in1);
 	return r;
@@ -1196,7 +1344,7 @@ _Py_areduce(PyObject *self, PyObject *args, PyObject *keywds)
 
 	if (_fixdim(&axis, &dim) < 0) return NULL;
 
-	r = _cum_swapped(self, _in1, axis,  out, "reduce", type);
+	r = _cum_swapped(self, _in1, axis,  out, "R", type);
 
 	Py_DECREF(_in1);
 	return r;
@@ -1228,18 +1376,15 @@ _Py_reduce(PyObject *self, PyObject *args, PyObject *keywds)
 
 	if (_fixdim(&axis, &dim) < 0) return NULL;
 
-	r = _cum_swapped(self, _in1, axis, out, "reduce", type);
+	r = _cum_swapped(self, _in1, axis, out, "R", type);
 	if (!r) return NULL;
 	
 	/* Convert rank-1-len-1 reduction result of rank-1 to scalar */
 	if (r != Py_None) {
 		PyArrayObject *ra = (PyArrayObject *) r;
 		PyArrayObject *_in1a = (PyArrayObject *) _in1;
-		if (_in1a->nd < 0) {
-			Py_DECREF(r);
-			return NULL;
-		}
-		if ((ra->nd == 1) && (ra->dimensions[0] ==1) && 
+		if ((((ra->nd == 1) && (ra->dimensions[0] ==1)) ||
+		     (ra->nd == 0)) && 
 		    ((_in1a->nd == 1) || (_in1a->nd == 0))) {
 			r = NA_getPythonScalar(ra, 0);
 			Py_DECREF(ra);
@@ -1259,10 +1404,10 @@ static PyMethodDef _ufunc_methods[] = {
 	 "accumulate   performs the operation along the dimension, accumulating subtotals"},
 	{"areduce", (PyCFunction) _Py_areduce, 
 	 METH_VARARGS|METH_KEYWORDS,
-	 "areduce  performs the operation along the specified dimension, eliminating it. Always returns an array."},
+	 "areduce  performs the operation along the specified dimension, eliminating it.  The result of areducing a 1D array is a 1 element 1D array."},
 	{"reduce", (PyCFunction) _Py_reduce, 
 	 METH_VARARGS|METH_KEYWORDS,
-	 "reduce  performs the operation along the specified dimension, eliminating it. Returns scalars rather than rank-0 numarray."},
+	 "reduce  performs the operation along the specified dimension, eliminating it.  The result of reducing a 1D array is a Python scalar."},
 	{"_accumulate_out", _Py_accumulate_out, METH_VARARGS,
 	 "_accumulate_out   creates the output array for an accumulation."},
 	{"_reduce_out", _Py_reduce_out, METH_VARARGS,
@@ -1291,7 +1436,7 @@ static PyMethodDef _ufunc_methods[] = {
 static PyTypeObject _ufunc_type = {
 	PyObject_HEAD_INIT(DEFERRED_ADDRESS(&PyType_Type))
 	0,
-	"_ufunc._ufunc",
+	"numarray._ufunc._ufunc",
 	sizeof(PyUfuncObject),
 	0,
 	_ufunc_dealloc,	                        /* tp_dealloc */
@@ -1686,7 +1831,8 @@ _cache_exec1(PyObject  *ufunc, PyObject *in1, PyObject *out,
 		if (_reportErrors(ufunc, result, out) < 0)
 			return NULL;
 	}
-	return _returnScalarOrArray(out);
+	Py_INCREF(out);
+	return out;
 }
 
 static PyObject *
@@ -1793,8 +1939,7 @@ _Py_cache_lookup1(PyObject *module, PyObject *args)
 static PyObject *
 _cached_dispatch1(PyObject *ufunc, PyObject *in1, PyObject *out)
 {
-	PyObject *result;
-	PyObject *oin1, *oout, *ocached;
+	PyObject *result, *nresult, *oin1, *oout, *ocached;
 
 	result = _cache_lookup1(ufunc, in1, out, &oin1, &oout, &ocached);
 	if (!result) 
@@ -1809,13 +1954,10 @@ _cached_dispatch1(PyObject *ufunc, PyObject *in1, PyObject *out)
 	Py_DECREF(ocached);
 
 	if (!result) return NULL;
-	if (out == Py_None) {
-		return result;
-	} else {
-		Py_DECREF(result);
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+
+	nresult = _normalize_results(1, &in1, 1, &out, 1, &result, 0);
+	Py_DECREF(result);
+	return nresult;
 }
 
 static PyObject *
@@ -2013,7 +2155,8 @@ _cache_exec2(PyObject  *ufunc, PyObject *in1, PyObject *in2, PyObject *out,
 		if (_reportErrors(ufunc, result, out) < 0)
 			return NULL;
 	}
-	return _returnScalarOrArray(out);
+	Py_INCREF(out);
+	return out;
 }
 
 static PyObject *
@@ -2068,14 +2211,10 @@ _cache_lookup2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out,
 		PyObject *insig, *cin1, *cin2;
 		char *insig_str;
 
-		if (!PyTuple_Check(ctuple) || (PyTuple_GET_SIZE(ctuple) != 7)) 
-		{
-			Py_DECREF(ctuple);
-			PyErr_Format(PyExc_RuntimeError,
-				     "_cache_lookup2: invalid cache tuple");
-			goto _fail;
-		}
-		
+		assert(PyTuple_Check(ctuple) && 
+		       (PyTuple_GET_SIZE(ctuple) == 7));
+
+		tc_start_clock("_cache_lookup2 hit pseudos");
 		cin1 = PyTuple_GetItem(ctuple, 5);
 		cin2 = PyTuple_GetItem(ctuple, 6);
 		cin1 = _restuff_pseudo(cin1, in1);
@@ -2087,16 +2226,17 @@ _cache_lookup2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out,
 		if (!(insig_str = PyString_AsString(insig)))
 			return PyErr_Format(PyExc_RuntimeError,
 					    "_cache_lookup2: problem with insig string");
+		tc_stop_clock("_cache_lookup2 hit pseudos");
 
 		tc_start_clock("_cache_lookup2 hit output");
 		if (out == Py_None) {
 			PyObject *otype = PyTuple_GET_ITEM(ctuple, 1);
 
-			if (!strcmp(insig_str, "vector_vector") ||
-			    !strcmp(insig_str, "vector_scalar")) {
+			if (!strcmp(insig_str, "vv") ||
+			    !strcmp(insig_str, "vs")) {
 				out = _getNewArray( cin1, otype);
 				if (!out) goto _fail;
-			} else if (!strcmp(insig_str, "scalar_vector")) {
+			} else if (!strcmp(insig_str, "sv")) {
 				out = _getNewArray( cin2, otype);
 				if (!out) goto _fail;
 			} else { /* scalar_scalar -> rank0 */
@@ -2112,10 +2252,10 @@ _cache_lookup2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out,
 			if (!NA_NumArrayCheck(out))
 				return PyErr_Format(PyExc_TypeError,
 					 "_cache_lookup2: output array was not a numarray");
-			if (!strcmp(insig_str, "vector_vector") ||
-			    !strcmp(insig_str, "vector_scalar")) {
+			if (!strcmp(insig_str, "vv") ||
+			    !strcmp(insig_str, "vs")) {
 				compare_to = cin1;
-			} else if (!strcmp(insig_str, "scalar_vector")) {
+			} else if (!strcmp(insig_str, "sv")) {
 				compare_to = cin2;
 			} else {
 				if (((PyArrayObject *)out)->nd)
@@ -2181,7 +2321,7 @@ _Py_cache_lookup2(PyObject *module, PyObject *args)
 static PyObject *
 _cached_dispatch2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out)
 {
-	PyObject *result, *oin1, *oin2, *oout, *ocached;
+	PyObject *nresult, *result, *oin1, *oin2, *oout, *ocached, *inputs[2];
 
 	tc_start_clock("cached_dispatch2 overall");
 	tc_start_clock("cached_dispatch2 lookup");
@@ -2205,14 +2345,12 @@ _cached_dispatch2(PyObject *ufunc, PyObject *in1, PyObject *in2, PyObject *out)
 	tc_stop_clock("cached_dispatch2 overall");
 
 	if (!result) return NULL;
-
-	if (out == Py_None) {
-		return result;
-	} else {
-		Py_DECREF(result);
-		Py_INCREF(Py_None);
-		return Py_None;
-	}
+	
+	inputs[0] = in1;
+	inputs[1] = in2;
+	nresult = _normalize_results(2, inputs, 1, &out, 1, &result, 0);
+	Py_DECREF(result);
+	return nresult;
 }
 
 static PyObject *
@@ -2265,10 +2403,11 @@ _copyFromAndConvert(PyObject *inarr, PyObject *outarr)
 
  	inbuff = ConverterRebuffer(input, inarr, Py_None);
 	if (!inbuff) return NULL;
-	Py_DECREF(inbuff);
 
  	outbuff = ConverterRebuffer(output, outarr, inbuff);
 	if (!outbuff) return NULL;
+
+	Py_DECREF(inbuff);
 	Py_DECREF(outbuff);
 
 	oshape = NA_intTupleFromMaybeLongs(outarra->nd, outarra->dimensions);
@@ -2299,13 +2438,9 @@ _copyFromAndConvert(PyObject *inarr, PyObject *outarr)
 		Py_DECREF(objects);
 		if (!result) return NULL; 
 
- 		inbuff = ConverterRebuffer(input, Py_None, Py_None);
-		if (!inbuff) return NULL;
-		Py_DECREF(inbuff);
-
- 		outbuff = ConverterRebuffer(output, Py_None, Py_None);
-		if (!outbuff) return NULL;
-		Py_DECREF(outbuff);
+		ConverterClean(input, inarr);
+		ConverterClean(output, inarr);
+		ConverterClean(output, outarr);
 	}
 	Py_DECREF(cached);
 	if (NA_checkAndReportFPErrors("_copyFromAndConvert") < 0) {
@@ -2376,6 +2511,8 @@ static PyMethodDef _ufunc_functions[] = {
 	 "returns the buffer size in bytes used for ufunc blocking"},
 	{"setBufferSize", _Py_setBufferSize, METH_VARARGS,
 	 "modifies size of new ufunc buffers allocated & ufunc blocking"},
+	{"_normalize_results", _Py_normalize_results, METH_VARARGS,
+	 "provides standard ufunc-like result handling"},
 	{NULL,		NULL}		/* sentinel */
 };
 
@@ -2403,7 +2540,10 @@ init_ufunc(void)
 			       (PyObject *) &_ufunc_type) < 0)
 		return;
 
+	ADD_VERSION(m);
+
 	import_libnumarray();
 
 	import_libtc();
 }
+
