@@ -1,13 +1,40 @@
 from distutils.core import Extension
 
 import sys, os
+from os.path import normpath
 
-VERSION = "1.1.1"
-VERSION_HEX = "0x010101000"
+VERSION = "1.3.2"
+VERSION_HEX = "0x01030200"
 
-NUMARRAY_PACKAGES = ["numarray"]
+NUMARRAY_PACKAGES = ["numarray", "numarray.codegenerator"]
 
-NUMARRAY_PACKAGE_DIRS = {"numarray":"Lib"}
+def config_item(s):
+    switch = "--" + s
+    if switch in sys.argv:
+	    sys.argv.remove(switch)
+	    return True
+    elif os.environ.has_key(s.upper()):
+	    return True
+    else:
+	    return False
+
+def normalize(pathlist):
+	if isinstance(pathlist, list):
+		return [ normalize(p) for p in pathlist ]
+	elif isinstance(pathlist, tuple):
+		return tuple([ normalize(p) for p in pathlist ])
+	elif isinstance(pathlist, dict):
+		d = {}
+		for key, value in pathlist.items():
+			d[key] = normalize(value)
+		return d
+	elif isinstance(pathlist, str):
+		return normpath(pathlist)
+	else:
+		return pathlist
+		
+NUMARRAY_PACKAGE_DIRS = {"numarray":"Lib",
+                         "numarray.codegenerator":"Lib/codegenerator"}
 
 MODULES = ["_conv",
            "_sort",
@@ -32,11 +59,12 @@ MODULES = ["_conv",
            "_converter",
            "_operator",
            "_ufunc",
+           "_numerictype",
            "libnumarray",
            "libnumeric",
            ]
 
-NUMARRAY_DATA_FILES = [('numarray', ['LICENSE.txt','Lib/testdata.fits'])]
+NUMARRAY_DATA_FILES = [('numarray', ['LICENSE.txt',normpath('Lib/testdata.fits')])]
 
 EXTRA_LINK_ARGS = []
 EXTRA_COMPILE_ARGS = []
@@ -58,27 +86,29 @@ if "--smp" in sys.argv:
     EXTRA_COMPILE_ARGS.append("-DNA_SMP=1")
     sys.argv.remove("--smp")
 
-if "--selftest" in sys.argv:
-    SELFTEST = 1
-    sys.argv.remove("--selftest")
-else:
-    SELFTEST = 0
-    
-HAS_UINT64 = 1
+USE_LAPACK = config_item("use_lapack")
+USE_ABSOFT = config_item("use_absoft")
+SELFTEST   = config_item("selftest")
+
+GENCODE    = config_item("gencode")
+GENUFUNCS  = GENCODE or config_item("genufuncs")
+GENAPI     = GENCODE or config_item("genapi")
+
+HAS_UINT64 = True
 
 codeargs = ""
 if sys.platform == "osf1V5":
     EXTRA_COMPILE_ARGS.extend(["-ieee"])
 elif sys.platform == "linux2":
-    if sys.maxint > 2**31-1:
+    if sys.maxint > 0x7FFFFFFF:
         pass
     else:
         pass
 elif sys.platform == "sunos5":
     pass
 elif sys.platform == "win32":
-    if sys.version.find("MSC") > 0:
-        HAS_UINT64 = 0
+    if sys.version.find("MSC") >= 0 and sys.version.find("v.13") < 0:
+        HAS_UINT64 = False
 elif sys.platform == "cygwin":
     EXTRA_LINK_ARGS += ["-L/lib", "-lm", "-lc", "-lgcc", "-L/lib/mingw", "-lmingwex"]
 elif sys.platform == "darwin":
@@ -95,23 +125,9 @@ if HAS_UINT64:
     codeargs = "--hasUInt64"
     MODULES.extend(["_ufuncUInt64"]) 
 
-gencode = "--gencode" in sys.argv        # Initial gencode value from switch
-if gencode:
-    sys.argv.remove("--gencode")
-
-genapi = genufuncs = gencode
-
-if "--genufuncs" in sys.argv:
-    genufuncs = 1
-    sys.argv.remove("--genufuncs")
-
-if "--genapi" in sys.argv:
-    genapi = 1
-    sys.argv.remove("--genapi")
-
 if (not os.path.exists("Include/numarray/numconfig.h") or
-   not os.path.exists("Src/convmodule.c")):
-    gencode = 1
+    not os.path.exists("Src/convmodule.c")):
+    GENCODE = True
 
 def locate_headers():
     for a in sys.argv:
@@ -127,39 +143,46 @@ def locate_headers():
             "posix"     : '%(base)s/include/python%(py_version_short)s/numarray',
             }
         try:
-            return template[sys.platform] % locals()
+            return normpath(template[sys.platform] % locals())
         except KeyError:
-            return template["posix"] % locals()
+            return normpath(template["posix"] % locals())
+
+EXTRA_COMPILE_ARGS = EXTRA_COMPILE_ARGS
+EXTRA_LINK_ARGS = EXTRA_LINK_ARGS
 
 class OurExtension(Extension):
     """OurBaseExtension is an Extension with implicit include_dirs,
     extra_compile_args, and extra_link_args.  Used to construct our
     c-api shared library.
     """
-    def __init__(self, module, Sources=None, lib_dirs=[], libs=[]):
-        if Sources is None:
-            Sources = [os.path.join("Src",module+"module.c")]
-        Extension.__init__(self, "numarray."+module, Sources,
-                           include_dirs=["Include/numarray"],
-                           library_dirs=lib_dirs,
-                           libraries=libs,
-                           extra_compile_args=EXTRA_COMPILE_ARGS,
-                           extra_link_args=EXTRA_LINK_ARGS)
+    def _set_default(self, keys, key, default):
+	    if (not keys.has_key(key) or keys[key] is None):
+		    keys[key] = default
+		    
+    def __init__(self, name, sources, **keys):
+	    self._set_default(keys, "extra_compile_args", EXTRA_COMPILE_ARGS)
+	    self._set_default(keys, "extra_link_args", EXTRA_LINK_ARGS)
+	    self._set_default(keys, "include_dirs", [])
+	    keys["include_dirs"].append("Include/numarray")
+	    keys = normalize(keys)
+	    Extension.__init__(self, name, normalize(sources), **keys)
 
-NUMARRAY_EXTENSIONS = map(OurExtension, MODULES)
+NUMARRAY_EXTENSIONS = [ OurExtension("numarray."+m, ["Src/"+ m +"module.c"]) 
+			for m in MODULES ]
 
 def prepare(modules):
     print "Using EXTRA_COMPILE_ARGS =",EXTRA_COMPILE_ARGS
 
     # Generate numarray configuration header file
     
-    f = open(os.path.join("Include","numarray","numconfig.h"),"w")
+    f = open(normpath("Include/numarray/numconfig.h"),"w")
     f.write("""
 /* This file is generated by setup.py.  DO NOT EDIT. */
 
+#define NUMARRAY_VERSION "%s"
 #define NUMARRAY_VERSION_HEX %s
 
-""" % (VERSION_HEX))
+""" % (VERSION, VERSION_HEX))
     f.close()
     del f
 
@@ -171,11 +194,12 @@ def prepare(modules):
 	INCLUDE_DIR = locate_headers()
 
     #  Generate the numinclude.py, the numarray configuration module.
-    f = open(os.path.join("Lib", "numinclude.py"), "w")
+    f = open("Lib/numinclude.py", "w")
     f.write("""
 # This file is generated by setup.py.  DO NOT EDIT.
 
 import _ndarray
+import os
 
 include_dir = '%s'
 version     = '%s'
@@ -184,17 +208,18 @@ LP64        = _ndarray.lp64()
 
 if not len(include_dir):   # default to same directory as numarray .py's
    import numinclude
-   include_dir = "/".join(numinclude.__file__.split("/")[:-1])
+   include_dir = os.sep.join(numinclude.__file__.split(os.sep)[:-1])
    
 """ % ( INCLUDE_DIR, VERSION))
     f.close()
 
     python = sys.executable + " "
-    if genufuncs:
-        os.system(python + os.path.join("Lib","codegenerator.py") + " " + codeargs)
-    if genapi:
-        os.system(python + os.path.join("Include","numarray", "genapis.py") + " " + codeargs)
+    if GENUFUNCS:
+        os.system(python + " " + normpath("Lib/codegenerator/__init__.py") 
+			 + " " + codeargs)
+    if GENAPI:
+        os.system(python + " " + normpath("Include/numarray/genapis.py") 
+			 + " " + codeargs)
 
 # if __name__ == "__main__":
 prepare(MODULES)
-
